package pwc.taxtech.atms.constant.enums;

import java.util.HashMap;
import java.util.Map;

public class RevenueConfEnum {

    /**
     * 账载收入类型
     */
    public enum AccountType {
        Zero(0, "0值"),
        Account(1, "科目"),
        Manual(2, "手工录入");
        private Integer code;
        private String name;
        public static final Map<Integer, String> MAPPING = new HashMap<>();

        AccountType(Integer code, String name) {
            this.code = code;
            this.name = name;
        }

        public Integer getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        static {
            for (RevenueConfEnum.AccountType accountType : RevenueConfEnum.AccountType.values()) {
                MAPPING.put(accountType.getCode(), accountType.getName());
            }
        }
    }

    /**
     * 账载收入类型
     */
    public enum TaxBase {
        Account(1, "账载收入"),
        Invoice(2, "开票收入"),
        Manual(3, "手工录入"),
        Period_Dr(4, "借方发生额"),
        Period_Cr(5, "贷方发生额");
        private Integer code;
        private String name;
        public static final Map<Integer, String> MAPPING = new HashMap<>();

        TaxBase(Integer code, String name) {
            this.code = code;
            this.name = name;
        }

        public Integer getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        static {
            for (RevenueConfEnum.TaxBase taxBase : RevenueConfEnum.TaxBase.values()) {
                MAPPING.put(taxBase.getCode(), taxBase.getName());
            }
        }
    }

    /**
     * 计税方法
     */
    public enum TaxType {
        Common(0, "一般计税"),
        Simple(1, "简易计税"),
        Tax_Exemption(2, "免抵退税"),
        Free(3, "免税");
        private Integer code;
        private String name;
        public static final Map<Integer, String> MAPPING = new HashMap<>();

        TaxType(Integer code, String name) {
            this.code = code;
            this.name = name;
        }

        public Integer getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        static {
            for (RevenueConfEnum.TaxType taxType : RevenueConfEnum.TaxType.values()) {
                MAPPING.put(taxType.getCode(), taxType.getName());
            }
        }
    }

    /**
     * 收入类型
     */
    public enum RevenueType {
        Service(0, "货物及加工修理修配劳务"),
        Assets(1, "服务、不动产和无形资产");
        private Integer code;
        private String name;
        public static final Map<Integer, String> MAPPING = new HashMap<>();

        RevenueType(Integer code, String name) {
            this.code = code;
            this.name = name;
        }

        public Integer getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        static {
            for (RevenueConfEnum.RevenueType revenueType : RevenueConfEnum.RevenueType.values()) {
                MAPPING.put(revenueType.getCode(), revenueType.getName());
            }
        }
    }

    /**
     * 状态
     */
    public enum Status {
        Enable(0, "启用"),
        Disable(1, "停用");
        private Integer code;
        private String name;
        public static final Map<Integer, String> MAPPING = new HashMap<>();

        Status(Integer code, String name) {
            this.code = code;
            this.name = name;
        }

        public Integer getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        static {
            for (RevenueConfEnum.Status status : RevenueConfEnum.Status.values()) {
                MAPPING.put(status.getCode(), status.getName());
            }
        }
    }

}
