package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONArray;
import org.springframework.web.bind.annotation.*;
import com.google.common.collect.Lists;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.dpo.PagingDto;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.DataImportLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.PagingResultDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceParam;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.service.impl.DataImportService;
import pwc.taxtech.atms.service.impl.LgGlBalanceService;

import javax.annotation.Resource;
import java.util.List;

@RestController
@RequestMapping("/api/v1/DataImport/")
public class DataImportController extends BaseController {

    @Resource
    private DataImportService dataImportService;
    @Resource
    private LgGlBalanceService lgGlBalanceService;

    @PostMapping("GetBalanceDataForDisplay")
    public PagingResultDto<TrialBalanceDto> getBalanceDataForDisplay(@RequestBody TrialBalanceParam param) {
        PagingResultDto<TrialBalanceDto> resultDto = new PagingResultDto<>();
        List<TrialBalanceDto> list = dataImportService.getBalanceDataForDisplay(param);
        PagingDto pagingDto = new PagingDto();
//        pagingDto.setPageIndex(1);
//        pagingDto.setPageSize(100);
//        pagingDto.setTotalCount(list);
        resultDto.setPageInfo(pagingDto);
        resultDto.setList(list);
        resultDto.setCalculateData(dataImportService.calculateData(list));
        return resultDto;
    }

    @PostMapping("queryGlBalance")
    public ApiResultDto queryGlBalance(@RequestBody TrialBalanceParam param) {
        ApiResultDto resultDto = new ApiResultDto();
        lgGlBalanceService.queryGlBalance(param.getOrgId(), param.getPeriodStart(), param.getPeriodEnd());
        return resultDto;
    }

    @ResponseBody
    @RequestMapping(value = "PLExcelFile", method = RequestMethod.POST)
    public OperationResultDto importPLExcelFile(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            return dataImportService.importPLExcelFile(file, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importPLExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "BSExcelFile", method = RequestMethod.POST)
    public OperationResultDto importBSExcelFile(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            return dataImportService.importBSExcelFile(file, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importBSExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "RLITExcelFile", method = RequestMethod.POST)
    public OperationResultDto importRLITExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            return dataImportService.importRLITExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importRLITExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "ATExcelFile", method = RequestMethod.POST)
    public OperationResultDto importATExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            return dataImportService.importATExcelFile(file, orgList,periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "CFExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCFExcelFile(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            return dataImportService.importCFExcelFile(file, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "CPRExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCPRExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            return dataImportService.importCPRExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "IDExcelFile", method = RequestMethod.POST)
    public OperationResultDto importIDExcelFile(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            return dataImportService.importIDExcelFile(file, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importIDExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "IRExcelFile", method = RequestMethod.POST)
    public OperationResultDto importIRExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            return dataImportService.importIRExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importIRExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "CILExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCILExcelFile(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            if (null == file) {
                return OperationResultDto.error(ErrorMessage.NoFile);
            }
            return dataImportService.importCILExcelFile(file, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importCILExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "displayImportLog", method = RequestMethod.GET)
    public List<DataImportLogDto> displayImportLog(@RequestParam Integer type) {
        List<DataImportLogDto> dataImportLogDtos = Lists.newArrayList();
        try {
            dataImportLogDtos = dataImportService.displayImportLog(type);
        } catch (Exception e) {
            logger.error("displayImportLog error.", e);
        }
        return dataImportLogDtos;
    }
}
