package pwc.taxtech.atms.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.ebsdto.*;
import pwc.taxtech.atms.service.EbsApiService;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping("/ebs/api/v1/dd")
public class EbsApiController {

    private static final Logger logger = LoggerFactory.getLogger(EbsApiController.class);
    @Resource
    private EbsApiService ebsApiService;

    @RequestMapping(value = "/updateJE", method = RequestMethod.POST)
    public ApiResultDto updateJE(@RequestBody List<JournalEntryQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateJE return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateJE(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateJE error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateTB", method = RequestMethod.POST)
    public ApiResultDto updateTB(@RequestBody List<TrialBalanceQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateTB return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateTB(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateTB error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateCF", method = RequestMethod.POST)
    public ApiResultDto updateCF(@RequestBody List<CashFlowQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateCF return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateCF(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateCF error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateBS", method = RequestMethod.POST)
    public ApiResultDto updateBS(@RequestBody List<BalanceSheetQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateBS return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBS(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateBS error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updatePL", method = RequestMethod.POST)
    public ApiResultDto updatePL(@RequestBody List<ProfitLossStatementQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updatePL return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePL(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updatePL error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateBSprc", method = RequestMethod.POST)
    public ApiResultDto updateBSprc(@RequestBody List<BalanceSheetPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateBSprc return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateBSprc(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateBSprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updatePLprc", method = RequestMethod.POST)
    public ApiResultDto updatePLprc(@RequestBody List<ProfitLossStatementPrcQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updatePLprc return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdatePLprc(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updatePLprc error.", e);
        }
        return ApiResultDto.fail();
    }

    @RequestMapping(value = "/updateOAR", method = RequestMethod.POST)
    public ApiResultDto updateOAR(@RequestBody List<OrganizationAccountingRateQueryDto> items) {
        if (CollectionUtils.isEmpty(items)) {
            logger.debug("the updateOAR return items is empty");
            return ApiResultDto.success(Collections.emptyList());
        }
        try {
            ebsApiService.queryRemoteServerThenUpdateOAR(items);
            return ApiResultDto.success();
        } catch (Exception e) {
            logger.error("updateOAR error.", e);
        }
        return ApiResultDto.fail();
    }

}
