package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.FileDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.vat.entity.PeriodCellTemplateConfig;
import pwc.taxtech.atms.vat.entity.PeriodJob;
import pwc.taxtech.atms.vat.entity.PwcReportAttach;
import pwc.taxtech.atms.vat.entity.VatEnterpriseAccount;
import pwc.taxtech.atms.vat.service.impl.ReportServiceImpl;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping(value = "api/v1/Report")
public class ReportController {
    @Autowired
    ReportServiceImpl reportService;

    @RequestMapping(value = "export", method = RequestMethod.POST)
    public ResponseEntity getExportFile(@RequestBody ReportExportDto report) {
        return ResponseEntity.ok(reportService.export(report.getReportData(), "~"));
    }

    @RequestMapping(value = "template/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return reportService.getReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

    @RequestMapping(value = "filterTemplate/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getFilterTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return reportService.getFilterReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

    @RequestMapping(value = "generateByTotal/{projectId}/{mergeManual}/{period}", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity generateAllData(@PathVariable String projectId, @PathVariable Integer period,
                                          @RequestParam Optional<String> generator, @PathVariable Boolean mergeManual) {
        return ResponseEntity.ok(reportService.generateData(projectId, EnumServiceType.VAT, mergeManual, period, null, generator));
    }

    @RequestMapping(value = "getRunningJob/{projectId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public PeriodJobDto getRunningJob(@PathVariable String projectId, @PathVariable Integer period) {
        PeriodJob job = reportService.getRunningJob(projectId, period);
        if (job != null)
            return new PeriodJobDto().copyFromPeriodJob(job);
        else return null;
    }

    @RequestMapping(value = "getJobStatus/{projectId}//{period}/{jobId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public PeriodJobDto getJobStatus(@PathVariable String projectId, @PathVariable Integer period, @PathVariable String jobId) {
        return new PeriodJobDto().copyFromPeriodJob(reportService.getJobStatus(projectId, period, jobId));
    }

    @RequestMapping(value = "templateReferences/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public List<CellTemplateReferenceDto> getTemplateReferences(@PathVariable int period) {
        return reportService.getTemplateReferences(period);
    }

    @RequestMapping(value = "reportData/{reportId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDataDto> getReportData(@PathVariable Long reportId, @RequestHeader String from) {
        OperationResultDto resultDto = new OperationResultDto();
        if (reportId == null || reportId == 0L) {
            resultDto.setResult(false);
            return resultDto;
        }
        return reportService.getCellData(reportId, from);
    }

    @RequestMapping(value = "report/{templateId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDto> getReportByTemplate(@PathVariable Long templateId, @PathVariable Integer period, @RequestHeader String from) {
        OperationResultDto resultDto = new OperationResultDto();
        if (templateId == null || templateId == 0L || period == null || period == 0) {
            resultDto.setResult(false);
            resultDto.setResultMsg("templateId or period is invalid");
            return resultDto;
        }
        return reportService.getReportByTemplate(templateId, period, from);
    }

    @RequestMapping(value = "getCellTemplateConfig/{reportTemplateId}/{period}/{rowIndex}/{columnIndex}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<PeriodCellTemplateConfig> getCellTemplateConfig(@PathVariable Long reportTemplateId,
                                                                              @PathVariable Integer period,
                                                                              @PathVariable int rowIndex,
                                                                              @PathVariable int columnIndex,
                                                                              @RequestHeader("from") String projectId) {
        return reportService.getCellTemplateConfig(reportTemplateId, period, rowIndex, columnIndex, projectId);
    }

    @RequestMapping(value = "getStdAccountByIndustry/{industryId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<VatEnterpriseAccount>> getStdAccountByIndustry(@PathVariable String industryId, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return reportService.getStdAccountByIndustry(industryId, projectId);
    }

    @RequestMapping(value = "getCellAccountRange/{reportTemplateId}/{period}/{rowIndex}/{columnIndex}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<VatEnterpriseAccountResultDto>> getCellAccountRange(@PathVariable Long reportTemplateId, @PathVariable Integer period, @PathVariable int rowIndex, @PathVariable int columnIndex, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return reportService.getCellAccountRange(reportTemplateId, period, rowIndex, columnIndex, projectId);
    }

    @RequestMapping(value = "addCellManualData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity addCellManualDataSource(@RequestBody ManualDataSourceDto data, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return ResponseEntity.ok(reportService.addCellManualDataSource(data, from));
    }

    @RequestMapping(value = "addDataSource", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> addDataSource(@RequestBody DataSourceDto datasource) {
        return reportService.addDataSource(datasource);
    }

    @RequestMapping(value = "getAllDataItems/{dataSourceType}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> getAllDataItems(@PathVariable Integer dataSourceType) {
        return reportService.getAllDataItems(dataSourceType);
    }


    @RequestMapping(value = "addDataSourceItems", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto addDataSourceItems(@RequestBody DataSourceDto dataSource) {
        return reportService.addDataSourceItems(dataSource);
    }

    @RequestMapping(value = "getDataSourceDetailList/{dataSourceId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> getDataSourceDetailList(@PathVariable Long dataSourceId) {
        return reportService.getDataSourceDetailList(dataSourceId);
    }

    @RequestMapping(value = "hasManualDataSource/{projectId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Boolean hasManualDataSource(@PathVariable String projectId, @PathVariable Integer period) {
        return reportService.hasManualDataSource(projectId, period);
    }


    @RequestMapping(value = "doUpload", method = RequestMethod.POST)
    public OperationResultDto doUploadAttach(@RequestParam("fileName") String fileName, MultipartFile file, String remarks, @RequestParam("activeCol")Long activeCol, @RequestParam("activeRow")Long activeRow, @RequestParam("activeTemplateId")String activeTemplateId){
        System.out.println(activeCol + "----"  + activeRow + "-----" + activeTemplateId);
        OperationResultDto operationResultDto = reportService.doUploadAttach(file, remarks);
        if(!"error".equals(operationResultDto.getResultMsg())){//上传成功绑定
            reportService.bindPwcAttach(activeCol, activeRow, activeTemplateId, (FileDto) operationResultDto.getData());
        }
        return operationResultDto;
    }

    @RequestMapping(value = "loadAttachList", method = RequestMethod.GET)
    public List<PwcReportAttach> loadAttachList(Long col, Long row, String templateId){
        return reportService.loadAttachList(col, row, templateId);
    }


}