package pwc.taxtech.atms.controller;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.ServletContextAware;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.enums.EnumCitImportType;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.dpo.TemplateUniqDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.vatdto.TemplateByGroupDto;
import pwc.taxtech.atms.entity.Template;
import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.exception.BadParameterException;
import pwc.taxtech.atms.exception.NotFoundException;
import pwc.taxtech.atms.service.impl.HttpFileService;
import pwc.taxtech.atms.service.impl.TemplateServiceImpl;

import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.List;

@RestController
@RequestMapping(value = "api/v1/template")
public class TemplateController extends BaseController implements ServletContextAware {

    private ServletContext servletContext;

    @Autowired
    TemplateServiceImpl templateService;

    @Autowired
    private HttpFileService httpFileService;

    @RequestMapping(value = "get", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    List<TemplateDto> get(@RequestParam(name = "templateGroupID") Long templateGroupId, @RequestParam(name = "reportType") String reportType) {
        if (!isParamValid(templateGroupId)) {
            return Collections.emptyList();
        }

        try {
            return templateService.get(templateGroupId, (reportType != null && !"null".equals(reportType)) ? Integer.parseInt(reportType) : null);
        } catch (Exception e) {
            logger.error("GetCellConfigList", e);
        }
        return Collections.emptyList();
    }

    private InputStream handleFomularView(InputStream is){
        Workbook workbook = null;
        try {
            workbook = new XSSFWorkbook(is);
        } catch (IOException e ) {
            logger.error("failed to read workbook!", e);
            throw new ApplicationException("failed to read workbook!");
        }
        for(int i = 0; i < workbook.getNumberOfSheets(); i++){
            Sheet sheet = workbook.getSheetAt(i);
            for(int r = sheet.getFirstRowNum(); r <= sheet.getLastRowNum(); r++){
                Row row = sheet.getRow(r);
                for (int c = row.getFirstCellNum(); c <= row.getLastCellNum(); c++) {
                    Cell cell = row.getCell(c);
                    if(cell != null && !cell.getCellStyle().getLocked()){
                        cell.setCellValue(StringUtils.EMPTY);
                    }
                }
            }
        }
        ByteArrayOutputStream  baos = new ByteArrayOutputStream ();
        try{
            workbook.write(baos);
            byte[] content = baos.toByteArray();
            return new ByteArrayInputStream(content);
        } catch (Exception e){
            logger.error("failed write workbook!", e);
            throw new ApplicationException("failed write workbook!");
        } finally {
            try {
                baos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }

    @RequestMapping(value = "getTemplateJson", method = RequestMethod.POST, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public @ResponseBody
    void getTemplateBlob(@RequestParam(name = "templateID") Long templateId, HttpServletResponse response) throws URISyntaxException {
        MyAsserts.assertTrue(isParamValid(templateId), new BadParameterException());
        String filePath;
        File templateFile;
        InputStream inputStream = null;
        Template template = templateService.getTemplateById(templateId);
        String templatePath = templateService.getTemplatePath(templateId);
        MyAsserts.assertNotEmpty(templatePath, new NotFoundException());
        filePath = this.getClass().getResource("").toURI().getPath();
        String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
        templateFile = new File(tempPath + templatePath);
        OutputStream out = null;
        try {
            //如果是系统报表就取本地文件夹，如果不是就取FTP
            if (template.getIsSystemType()) {
                inputStream = new BufferedInputStream(new FileInputStream(templateFile));
            } else {
                inputStream = httpFileService.getUserTemplate(templatePath);
            }
            //客户端保存的文件名
            String customFileName = "template_" + DateTime.now().toString("yyyyMMddHHmmss") + ".xlsx";
            response.setHeader("Content-Disposition", String.format("inline; filename=\"" + customFileName + "\""));
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
//            int len = 0;
//            byte[] buffer = new byte[1024];
//            out = response.getOutputStream();
//            while ((len = inputStream.read(buffer)) > 0) {
//                out.write(buffer, 0, len);
//            }

            out = response.getOutputStream();
            IOUtils.copy(handleFomularView(inputStream), out);
            out.flush();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (Exception e) {
            logger.error("Error downloading template file template.xlsx", e);
            throw new ApplicationException("Error downloading template file template.xlsx", e);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                out = null;
            }

            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                inputStream = null;
            }
        }
    }

    private File getTempFile(String templatePath) throws URISyntaxException {
        String filePath = this.getClass().getResource("").toURI().getPath();
        String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
        return new File(tempPath + templatePath);

    }

    @RequestMapping(value = "getTemplateUniqList", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    List<TemplateUniqDto> getTemplateUniqList(@RequestParam(value = "serviceTypeID") String serviceTypeId, Integer payTaxType, Integer reportType, @RequestParam(value = "industryIDs") String industryIds) {
        return templateService.getTemplateUniqList(serviceTypeId, payTaxType, reportType, industryIds);
    }

    @RequestMapping(value = "updateTemplateName", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<Object> updateTemplateName(@RequestBody UpateNameParam param) {
        return templateService.updateTemplateName(param);
    }

    @RequestMapping(value = "deleteTemplate", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<String> deleteTemplate(@RequestBody DeleteTemplateParam param) {
        OperationResultDto<String> result = templateService.deleteTemplate(param);
        if (result.getResult() && StringUtils.isNotBlank(result.getData())) {
            try {
                //todo
//                httpFileService.delete(result.getData());
            } catch (Exception e) {
            }
        }
        return result;
    }

    @RequestMapping(value = "rowColName/{id}", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto setRowColName(@PathVariable Long id, @RequestBody List<CellBriefDto> cellInfo) {
        if (isParamValid(id)) {
            return templateService.setRowColName(id, cellInfo);
        } else {
            return null;
        }

    }

    @RequestMapping(value = "getGroupTemplateByGroupID", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<List<TemplateByGroupDto>> getGroupTemplateByGroupId(@RequestParam(name = "templateGroupID") Long templateGroupId, @RequestParam(name = "projectID") String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        if (templateGroupId == null || templateGroupId.equals("undefined") || StringUtils.isBlank(projectId) || projectId.equals("undefined")) {
            resultDto.setResult(false);
            return resultDto;
        }
        return templateService.getByGroupId(templateGroupId, projectId);
    }

    @RequestMapping(value = "addExistTemplate", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<Boolean> addExistTemplate(@RequestBody TemplateAddExistDto templateAddExistDto){
        OperationResultDto resultDto = new OperationResultDto();
        if(templateAddExistDto.getTemplateGroupId() == null || CollectionUtils.isEmpty(templateAddExistDto.getTemplateIdList())){
            resultDto.setResult(false);
            return resultDto;
        }
        templateService.addExistTemplate(templateAddExistDto);
        resultDto.setResult(true);
        return resultDto;
    }

    @RequestMapping(value = "file/downloadTemplate", method = RequestMethod.GET)
    public void fileDownload(@RequestParam Integer fileType, @RequestParam(required=false) Integer serviceType, HttpServletResponse response){
        //获取网站部署路径(通过ServletContext对象)，用于确定下载文件位置，从而实现下载
        String path = servletContext.getRealPath("/")+"\\WEB-INF\\classes";
        String fileName = "";
        if(serviceType == null){
            fileName = getFileName(fileType);
        }else{
            fileName = getCitFileName(fileType);
        }
        response.setContentType("multipart/form-data");
        response.setHeader("Content-Disposition", "attachment;fileName="+fileName+".xlsx");
        ServletOutputStream out;
        File file = new File(path + "\\document\\DataImport\\" + fileName+".xlsx");
        try {
            FileInputStream inputStream = new FileInputStream(file);
            out = response.getOutputStream();
            int b = 0;
            byte[] buffer = new byte[512];
            while (b != -1){
                b = inputStream.read(buffer);
                out.write(buffer,0,b);
            }
            inputStream.close();
            out.close();
            out.flush();
        } catch (Exception e){
            logger.error(String.format("下载模板异常:%s",e.getMessage()));
        }
    }

    private String getFileName(Integer fileType) {
        if(EnumImportType.ProfitLoss.getCode().equals(fileType)){
            return EnumImportType.ProfitLoss.name();
        }else if(EnumImportType.BalanceSheet.getCode().equals(fileType)){
            return EnumImportType.BalanceSheet.name();
        }else if(EnumImportType.RedLetterInfoTable.getCode().equals(fileType)){
            return EnumImportType.RedLetterInfoTable.name();
        }else if(EnumImportType.AdjustmentTable.getCode().equals(fileType)){
            return EnumImportType.AdjustmentTable.name();
        }else if(EnumImportType.CashFlow.getCode().equals(fileType)){
            return EnumImportType.CashFlow.name();
        }else if(EnumImportType.CoupaPurchasingReport.getCode().equals(fileType)){
            return EnumImportType.CoupaPurchasingReport.name();
        }else if(EnumImportType.InvoiceData.getCode().equals(fileType)){
            return EnumImportType.InvoiceData.name();
        }else if(EnumImportType.CertifiedInvoicesList.getCode().equals(fileType)){
            return EnumImportType.CertifiedInvoicesList.name();
        }else if(EnumImportType.InvoiceRecord.getCode().equals(fileType)){
            return EnumImportType.InvoiceRecord.name();
        }else if(EnumImportType.Undefined.getCode().equals(fileType)){
            return EnumImportType.Undefined.name();
        }
        return null;
    }

    private String getCitFileName(Integer fileType) {
        if(EnumCitImportType.JournalAdjust.getCode().equals(fileType)){
            return EnumCitImportType.JournalAdjust.name();
        }else if(EnumCitImportType.TrialBalance.getCode().equals(fileType)){
            return EnumCitImportType.TrialBalance.name();
        }else if(EnumCitImportType.DraftAccountMapping.getCode().equals(fileType)){
            return EnumCitImportType.DraftAccountMapping.name();
        }else if(EnumCitImportType.BalanceSheetPrcAdjust.getCode().equals(fileType)){
            return EnumCitImportType.BalanceSheetPrcAdjust.name();
        }else if(EnumCitImportType.ProfitPrcAdjust.getCode().equals(fileType)){
            return EnumCitImportType.ProfitPrcAdjust.name();
        }else if(EnumCitImportType.EAMAssetsDisposal.getCode().equals(fileType)){
            return EnumCitImportType.EAMAssetsDisposal.name();
        }else if(EnumCitImportType.SalaryAdvance.getCode().equals(fileType)){
            return EnumCitImportType.SalaryAdvance.name();
        }
        return null;
    }


    @Override
    public void setServletContext(ServletContext servletContext) {
        this.servletContext = servletContext;
    }
}
