package pwc.taxtech.atms.dto.vatdto;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import org.apache.commons.lang3.StringUtils;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.entity.Template;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.entity.PeriodJob;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

public class WrapPeriodJobDto {
    public static final String STATUS_BEGIN = "Begin";
    public static final String STATUS_RUNNING = "Running";
    public static final String STATUS_CANCEL = "Cancel";
    public static final String STATUS_ERROR = "Error";
    public static final String STATUS_END = "End";

    public static final String STEP_UPDATE_CONFIG = "UpdateConfig";

    public static PeriodJob createReportGenJob(String projectId, Integer period, List<Template> templates) {
        MyAsserts.assertNotEmpty(templates, Exceptions.NOT_FOUND_TEMPLATE_EXCEPTION);
        PeriodJob job = new PeriodJob();
        job.setCreateTime(new Date());
        job.setName("Gen All Report");
        job.setProjectId(projectId);
        job.setPeriod(period);
        job.setId(UUID.randomUUID().toString());
        job.setCurrentStep(STEP_UPDATE_CONFIG);
        setStatus(job,STEP_UPDATE_CONFIG,STATUS_BEGIN);
        StringBuilder builder = new StringBuilder(STEP_UPDATE_CONFIG);
        templates.forEach(m -> {
            builder.append(",").append(m.getCode());
        });

        job.setStepsCode(builder.toString());
        return job;
    }

    public static class Task {
        String code;
        String status;

        public String getCode() {
            return this.code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getStatus() {
            return this.status;
        }

        public void setStatus(String status) {
            this.status = status;
        }
    }

    public static List<Task> fromJson(String taskJson) {
        return JSON.parseObject(taskJson, new TypeReference<List<Task>>() {});
    }

    public static String toJson(List<Task> tasks) {
        return JSON.toJSONString(tasks);
    }

    public static void setStatus(PeriodJob periodJob, String code, String status) {
        List<Task> list;
        if (StringUtils.isBlank(periodJob.getStatus())) {
            list = new ArrayList<>();
        } else {
            list = fromJson(periodJob.getStatus());
        }

        Task t = new Task();
        t.code = code;
        t.status = status;
        boolean contains = false;
        for (Task m : list) {
            if (m.code.equals(t.code)) {
                m.status = status;
                contains = true;
                break;
            }
        }
        if(!contains)list.add(t);
        periodJob.setStatus(toJson(list));
    }

    public static void setStatus(PeriodJob periodJob, String status) {
        List<Task> list;
        if (StringUtils.isBlank(periodJob.getStatus())) {
            list = new ArrayList<>();
        } else {
            list = fromJson(periodJob.getStatus());
        }
        list.get(list.size()-1).status=status;
        periodJob.setStatus(toJson(list));
    }
}
