package pwc.taxtech.atms.security;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.dao.PermissionMapper;
import pwc.taxtech.atms.dao.RolePermissionMapper;
import pwc.taxtech.atms.dao.UserRoleMapper;
import pwc.taxtech.atms.entity.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class JwtAuthenticationService {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Resource
    private UserRoleMapper userRoleMapper;

    @Resource
    private RolePermissionMapper rolePermissionMapper;

    @Resource
    private PermissionMapper permissionMapper;


    /**
     * 27/12/2018 11:05
     *
     * 用户第一个需鉴权的操作时，从UPM(目前从数据库中查询)获取用户操作权限存入EhCache，至下一次登陆前从EhCache获取权限进行操作鉴权。
     * // todo upm return  response
     * [userName]
     * @author Gary J Li
     * @return List<String>
     */
    @Cacheable(value = "apiAuthCache", key = "#userid")
    public List<String> getApiAuthList(String userid) {
        List<String> apiAuthList = new ArrayList<>();
        // todo 1、修改角色的权限时，查询角色下的用户，刷新缓存
        UserRoleExample userRoleExample = new UserRoleExample();
        userRoleExample.createCriteria().andUserIdEqualTo(userid);
        List<UserRole> userRoles = userRoleMapper.selectByExample(userRoleExample);

        List<String> roleIds = new ArrayList<>();
        userRoles.forEach( ur ->{
            roleIds.add(ur.getRoleId());
        });
        RolePermissionExample rolePermissionExample = new RolePermissionExample();
        rolePermissionExample.createCriteria().andRoleIdIn(roleIds);
        List<RolePermission> rolePermissions = rolePermissionMapper.selectByExample(rolePermissionExample);

        List<String> permissionIds = new ArrayList<>();
        rolePermissions.forEach( rp ->{
            permissionIds.add(rp.getPermissionId());
        });
        PermissionExample permissionExample= new PermissionExample();
        permissionExample.createCriteria().andIdIn(permissionIds);
        List<Permission> permissions = permissionMapper.selectByExample(permissionExample);

        permissions.forEach(permission -> {
            apiAuthList.addAll(Arrays.stream(permission.getOperationPermList().split("\\|")).filter(p -> StringUtils.isNotBlank(p)).collect(Collectors.toList()));
        });

        logger.debug("get Cache from upm :"+"apiAuthCache-"+"key :"+userid + " value :"+String.join("，",apiAuthList));
        return apiAuthList;
    }

    /**
     * 27/12/2018 11:07
     * 用户登陆时服务器EhCache中的用户权限信息。
     * [userName]
     * @author Gary J Li
     *
     */
    @CacheEvict(value = "apiAuthCache", key = "#userid")
    public void removeApiAuthList(String userid) {
        logger.debug("remove Cache :"+"apiAuthCache"+"key :"+userid);
    }

}
