package pwc.taxtech.atms.service.impl;
import java.util.Date;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.config.SystemConfig;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dto.DataImportLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceParam;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DataImportService extends BaseService {

    @Resource
    private GlBalanceMapper glBalanceMapper;
    @Resource
    private OrganizationMapper organizationMapper;
    @Resource
    private StandardAccountMapper standardAccountMapper;
    @Resource
    private SystemConfig systemConfig;
    @Resource
    private EnterpriseAccountSetOrgDao enterpriseAccountSetOrgDao;
    @Resource
    private AccountMappingDao accountMappingDao;
    @Autowired
    private HttpFileService httpFileService;
    @Resource
    private DataImportLogMapper dataImportLogMapper;
    @Resource
    private BalanceSheetMapper balanceSheetMapper;
    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;
    @Resource
    private BalanceSheetManualMapper balanceSheetManualMapper;
    @Resource
    private ProfitLossStatementManualMapper profitLossStatementManualMapper;
    @Resource
    private BalanceSheetFinalMapper balanceSheetFinalMapper;
    @Resource
    private ProfitLossStatementFinalMapper profitLossStatementFinalMapper;
    @Resource
    private RedLetterInfoTableMapper redLetterInfoTableMapper;
    @Resource
    private AdjustmentTableMapper adjustmentTableMapper;
    @Resource
    private CashFlowMapper cashFlowMapper;
    @Resource
    private CoupaPurchasingReportMapper coupaPurchasingReportMapper;
    @Resource
    private InvoiceDataMapper invoiceDataMapper;
    @Resource
    private CertifiedInvoicesListMapper certifiedInvoicesListMapper;
    @Resource
    private InvoiceRecordMapper invoiceRecordMapper;

    /**
     * TB表查询
     *
     * @param param TrialBalanceParam
     * @return List<TrialBalanceDto>
     */
    public List<TrialBalanceDto> getBalanceDataForDisplay(TrialBalanceParam param) {
        if (!ObjectUtils.allNotNull(param, param.getPeriodStart(), param.getPeriodEnd()) || StringUtils.isBlank(param.getOrgId())) {
            return Collections.emptyList();
        }
        Organization organization = organizationMapper.selectByPrimaryKey(param.getOrgId());
        if (null == organization) {
            return Collections.emptyList();
        }
        Optional<EnterpriseAccountSetOrg> accountSetOrg = enterpriseAccountSetOrgDao.getByOrganization(organization.getId());
        if (!accountSetOrg.isPresent()) {
            return Collections.emptyList();
        }
        // mapping
        List<AccountMapping> mappingList = accountMappingDao.getAccountMapping(accountSetOrg.get().getEnterpriseAccountSetId(),
                organization.getId());
        List<String> etCodeList = mappingList.stream().map(AccountMapping::getEnterpriseAccountCode).distinct().collect(Collectors.toList());
        Map<String, List<AccountMapping>> mappingMap = mappingList.stream().collect(Collectors.groupingBy(AccountMapping::getStandardAccountCode));
        //standard account
        StandardAccountExample stdExample = new StandardAccountExample();
        stdExample.createCriteria().andIsLeafEqualTo(Boolean.TRUE);
        List<StandardAccount> stdAccountList = standardAccountMapper.selectByExample(stdExample);
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat dateFormat = new SimpleDateFormat(Constant.DateFormat.YEAR_MONTH);
        Map<String, TrialBalanceDto> resultStdMap = new HashMap<>();
        for (int i = param.getPeriodStart(); i <= param.getPeriodEnd(); i++) {
            calendar.set(Calendar.MONTH, i - 1);
            GlBalanceExample example = new GlBalanceExample();
            //科目段
            example.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime())).andSegment3In(etCodeList);
            List<GlBalance> sg3List = glBalanceMapper.selectByExample(example);

            GlBalanceExample example2 = new GlBalanceExample();
            //明细段
            example2.createCriteria().andSegment1EqualTo(organization.getClientCode())
                    .andPeriodNameEqualTo(dateFormat.format(calendar.getTime())).andSegment4In(etCodeList);
            List<GlBalance> sg4List = glBalanceMapper.selectByExample(example2);

            for (StandardAccount standardAccount : stdAccountList) {
                if (!resultStdMap.containsKey(standardAccount.getCode())) {
                    resultStdMap.put(standardAccount.getCode(), new TrialBalanceDto());
                }
//                if (!mappingMap.containsKey(standardAccount.getCode())) {
//                    continue;
//                }
                TrialBalanceDto dto = resultStdMap.get(standardAccount.getCode());
                dto.setChildren(Collections.emptyList());
                dto.setAcctCode(standardAccount.getCode());
                dto.setAccountName(standardAccount.getName());
                List<AccountMapping> tmp = mappingMap.get(standardAccount.getCode());
                if (CollectionUtils.isEmpty(tmp)) {
                    continue;
                }
                tmp.forEach(o -> {
                    dto.setCreditBal(add(dto.getCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdCr()));
                    dto.setBegCreditBal(add(dto.getBegCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginCrBalance()));
                    dto.setEndCreditBal(add(dto.getEndCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndCrBalance()));
                    dto.setYearCreditBal(add(dto.getYearCreditBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdCr()));
                    dto.setDebitBal(add(dto.getDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getPtdDr()));
                    dto.setBegDebitBal(add(dto.getBegDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getBeginDrBalance()));
                    dto.setEndDebitBal(add(dto.getEndDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getEndDrBalance()));
                    dto.setYearDebitBal(add(dto.getYearDebitBal(), getGlBalance(o.getEnterpriseAccountCode(), sg3List, sg4List).getYtdDr()));
                });
            }
        }

        return new ArrayList<>(resultStdMap.values()).stream().sorted(Comparator.comparing(TrialBalanceDto::getAcctCode))
                .collect(Collectors.toList());
    }

    private BigDecimal add(BigDecimal v1, BigDecimal v2) {
        v1 = null == v1 ? BigDecimal.ZERO : v1;
        v2 = null == v2 ? BigDecimal.ZERO : v2;
        return v1.add(v2);
    }

    private GlBalance getGlBalance(String code, List<GlBalance> seg3, List<GlBalance> seg4) {
        for (GlBalance balance : seg3) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }

        for (GlBalance balance : seg4) {
            if (StringUtils.equals(code, balance.getSegment3())) {
                return balance;
            }
        }
        return new GlBalance();
    }

    /**
     * 计算总和
     *
     * @param list List<TrialBalanceDto>
     * @return TrialBalanceDto
     */
    public TrialBalanceDto calculateData(List<TrialBalanceDto> list) {
        TrialBalanceDto dto = new TrialBalanceDto();
        list.forEach(o -> {
            dto.setCreditBal(add(dto.getCreditBal(), o.getCreditBal()));
            dto.setBegCreditBal(add(dto.getBegCreditBal(), o.getBegCreditBal()));
            dto.setEndCreditBal(add(dto.getEndCreditBal(), o.getEndCreditBal()));
            dto.setYearCreditBal(add(dto.getYearCreditBal(), o.getYearCreditBal()));

            dto.setDebitBal(add(dto.getDebitBal(), o.getDebitBal()));
            dto.setBegDebitBal(add(dto.getBegDebitBal(), o.getBegDebitBal()));
            dto.setEndDebitBal(add(dto.getEndDebitBal(), o.getEndDebitBal()));
            dto.setYearDebitBal(add(dto.getYearDebitBal(), o.getYearDebitBal()));
        });
        return dto;
    }

    /**
     * 14/02/2019 14:57
     * 导入利润表
     * [file, periodDate]
     * @author Gary J Li
     * @return
     * remark:1、period为 201812 201813 记录 tms_period 201812
     *        2、可以考虑加index，未来若需预览可保证顺序
     * errorList:
     * 1、存在非选择期间数据
     * 2、未映射到主体
     */
    public OperationResultDto importPLExcelFile(MultipartFile file, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessage.DIDNOTSELECTPERIOD);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<ProfitLossStatement> pls = Lists.newArrayList();
                String companyCode = sheet.getRow(5).getCell(1).getStringCellValue();
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andCodeEqualTo(companyCode);
                List<Organization> organizations = organizationMapper.selectByExample(example);
                String orgId = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = sheet.getRow(2).getCell(2).getStringCellValue().substring(3);
                String companyName = sheet.getRow(4).getCell(1).getStringCellValue();
                String fileType = sheet.getRow(0).getCell(2).getStringCellValue();

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.ProfitLoss.getCode(), period/100,
                        tmsPeriod%100, period%100,
                        companyName, fileType);

                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period / 1000 != 13 && tmsPeriod != 12;
                if (!tmsPeriod.equals(period) && isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                String currencyCode = sheet.getRow(5).getCell(0).getStringCellValue().replace(" ", "").substring(3);
                String dataDateStr = sheet.getRow(0).getCell(4).getStringCellValue().substring(3).replace(" ", "");
                SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyyHH:mm:ss");
                Date dataDate = sdf.parse(dataDateStr);

                for (int j = 11; j < sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    ProfitLossStatement pl = new ProfitLossStatement();
                    pl.setId(plId);
                    pl.setOrganizationId(orgId);
                    pl.setDate(dataDate);
                    pl.setSource("Excel导入");
                    pl.setStatus("人工导入");
                    pl.setTmsPeriod(tmsPeriod);
                    pl.setPeriod(period);
                    pl.setEntityCode(companyCode);
                    pl.setEntityName(companyName);
                    pl.setLedgerCurrencyCode(currencyCode);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    pl.setItemName(cell1.getStringCellValue());

                    Cell cell2 = row.getCell(1);
                    if (cell2.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = cell2.getStringCellValue().replace(",", "");
                        pl.setPeriodAmt(new BigDecimal(value1));
                    } else {
                        pl.setPeriodAmt(new BigDecimal(cell2.getNumericCellValue()));
                    }

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = cell3.getStringCellValue().replace(",", "");
                        pl.setYtdAmt(new BigDecimal(value2));
                    } else {
                        pl.setYtdAmt(new BigDecimal(cell3.getNumericCellValue()));
                    }
                    pls.add(pl);
                }
                if (pls.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        ProfitLossStatementExample profitLossStatementExample = new ProfitLossStatementExample();
                        profitLossStatementExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        profitLossStatementMapper.deleteByExample(profitLossStatementExample);
                        if (profitLossStatementFinalMapper.countByExample(profitLossStatementExample) > 0) {
                            profitLossStatementFinalMapper.deleteByExample(profitLossStatementExample);
                        }
                    }
                    profitLossStatementManualMapper.insertBatch(pls);
                    profitLossStatementFinalMapper.insertBatch(pls);
                    dataImportLog.setRecordSize(pls.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importPLExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private DataImportLog generalDataImportLog(String companyCode, String orgId, String taxPayerId, Integer type, Integer periodYear,
                                               Integer tmsPeriodMonth, Integer periodMonth, String companyName, String fileType) {
        DataImportLog dataImportLog = new DataImportLog();
        dataImportLog.setId(idService.nextId());
        dataImportLog.setTaxpayerIdNum(taxPayerId);
        dataImportLog.setType(type);
        dataImportLog.setOrganizationId(orgId);
        dataImportLog.setCompanyCode(companyCode);
        dataImportLog.setCompanyName(companyName);
        dataImportLog.setFileType(fileType);
        dataImportLog.setPeriodYear(periodYear);
        dataImportLog.setTmsPeriodMonth(tmsPeriodMonth);
        dataImportLog.setPeriodMonth(periodMonth);
        dataImportLog.setPeriodStatus("人工导入");
        dataImportLog.setOperator(authUserHelper.getCurrentAuditor().get());
        dataImportLog.setImportResult(true);
        return dataImportLog;
    }

    /**
     * 14/02/2019 16:36
     * 导入资产负债表
     * [file, periodDate]
     *
     * @return remark:1、period为 201812 201813 记录 tms_period 201812
     * 2、可以考虑加index，未来若需预览可保证顺序
     * errorList:
     * 1、存在非选择期间数据
     * 2、未映射到主体
     * @author Gary J Li
     */
    public OperationResultDto importBSExcelFile(MultipartFile file, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<BalanceSheet> bls = Lists.newArrayList();
                String companyName = sheet.getRow(5).getCell(1).getStringCellValue();
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andNameEqualTo(companyName);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = sheet.getRow(2).getCell(2).getStringCellValue().substring(3);
                String fileType = sheet.getRow(0).getCell(2).getStringCellValue();

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.BalanceSheet.getCode(), period/100,
                        tmsPeriod%100, period%100,
                        companyName, fileType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period / 1000 != 13 && tmsPeriod != 12;
                if (!tmsPeriod.equals(period) && isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String currencyCode = sheet.getRow(5).getCell(0).getStringCellValue().replace(" ", "").substring(3);
                String dataDateStr = sheet.getRow(0).getCell(4).getStringCellValue().substring(3).replace(" ", "");
                SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyyHH:mm:ss");
                Date dataDate = sdf.parse(dataDateStr);
                for (int j = 11; j < sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    BalanceSheet bl = new BalanceSheet();
                    bl.setId(plId);
                    bl.setOrganizationId(orgId);
                    bl.setDate(dataDate);
                    bl.setSource("Excel导入");
                    bl.setStatus("人工导入");
                    bl.setTmsPeriod(tmsPeriod);
                    bl.setPeriod(period);
                    bl.setEntityName(companyName);
                    bl.setLedgerCurrencyCode(currencyCode);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    bl.setItemName(cell1.getStringCellValue());

                    Cell cell2 = row.getCell(1);
                    if (cell2.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = cell2.getStringCellValue().replace(",", "");
                        bl.setEndBal(new BigDecimal(value1));
                    } else {
                        bl.setEndBal(new BigDecimal(cell2.getNumericCellValue()));
                    }

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = cell3.getStringCellValue().replace(",", "");
                        bl.setBegBal(new BigDecimal(value2));
                    } else {
                        bl.setBegBal(new BigDecimal(cell3.getNumericCellValue()));
                    }
                    bls.add(bl);
                }
                if (bls.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        BalanceSheetExample balanceSheetExample = new BalanceSheetExample();
                        balanceSheetExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        balanceSheetMapper.deleteByExample(balanceSheetExample);
                        if (balanceSheetManualMapper.countByExample(balanceSheetExample) > 0) {
                            balanceSheetManualMapper.deleteByExample(balanceSheetExample);
                        }
                    }
                    balanceSheetMapper.insertBatch(bls);
                    balanceSheetManualMapper.insertBatch(bls);
                    dataImportLog.setRecordSize(bls.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importBSExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    /**
     * 16/02/2019 11:57
     * 导入现金流量表
     * [file, periodDate, importType]
     * @author Gary J Li
     * @return
     */
    public OperationResultDto importCFExcelFile(MultipartFile file, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CashFlow> cfs = Lists.newArrayList();
                String companyName = sheet.getRow(4).getCell(1).getStringCellValue();
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andNameEqualTo(companyName);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = sheet.getRow(4).getCell(8).getStringCellValue();
                String fileType = sheet.getRow(0).getCell(0).getStringCellValue();
                Date dataDate = DateUtils.strToDate2(sheet.getRow(4).getCell(10).getStringCellValue());

                Integer period = DateUtils.strToPeriod(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.CashFlow.getCode(), period/100,
                        tmsPeriod%100, period%100,
                        companyName, fileType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period / 1000 != 13 && tmsPeriod != 12;
                if (!tmsPeriod.equals(period) && isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }

                for (int j = 6; j < sheet.getLastRowNum(); j++) {
                    Long id = idService.nextId();
                    CashFlow cf = new CashFlow();
                    cf.setId(id);
                    cf.setDate(dataDate);
                    cf.setEntityCode(companyCode);
                    cf.setEntityName(companyName);
                    cf.setPeriod(period);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1 || StringUtils.isEmpty(cell1.getStringCellValue())) {
                        continue;
                    }
                    cf.setItemName(cell1.getStringCellValue());

                    Cell cell2 = row.getCell(1);
                    cf.setItemName2(cell2.getStringCellValue());

                    Cell cell3 = row.getCell(2);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value1 = cell3.getStringCellValue().replace(",", "");
                        cf.setPeriodAmt(new BigDecimal(value1));
                    } else {
                        cf.setPeriodAmt(new BigDecimal(cell3.getNumericCellValue()));
                    }

                    Cell cell4 = row.getCell(3);
                    if (cell3.getCellTypeEnum().equals(CellType.STRING)) {
                        String value2 = cell4.getStringCellValue().replace(",", "");
                        cf.setYtdAmt(new BigDecimal(value2));
                    } else {
                        cf.setYtdAmt(new BigDecimal(cell4.getNumericCellValue()));
                    }
                    cfs.add(cf);
                }
                if (cfs.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        CashFlowExample cashFlowExample = new CashFlowExample();
                        cashFlowExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        cashFlowMapper.deleteByExample(cashFlowExample);
                        if (cashFlowMapper.countByExample(cashFlowExample) > 0) {
                            cashFlowMapper.deleteByExample(cashFlowExample);
                        }
                    }
                    cashFlowMapper.insertBatch(cfs);
                    dataImportLog.setRecordSize(cfs.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importCFExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }



    /**
     * 18/02/2019 12:36
     * 导入发票资料
     * [file, periodDate, importType]
     * @author Gary J Li
     * @return OperationResultDto
     */
    public OperationResultDto importIDExcelFile(MultipartFile file, String periodDate, Integer importType) throws ServiceException{
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<InvoiceData> cfs = Lists.newArrayList();
                String[] sheetName = sheet.getSheetName().split("-");

                String companyCode = sheetName[0];


                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andCodeEqualTo(companyCode);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyName = "";
                String taxPayerId = "";
                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyName = organizations.get(0).getName();
                    taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                            organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                }
                String periodStr = sheetName[1].substring(0,6);
                String fileType = "发票资料";

                Integer period = Integer.valueOf(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, taxPayerId,
                        EnumImportType.InvoiceData.getCode(), period/100,
                        tmsPeriod%100, period%100,
                        companyName, fileType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period / 1000 != 13 && tmsPeriod != 12;
                if (!tmsPeriod.equals(period) && isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                List<InvoiceData> iDatas = Lists.newArrayList();
                for (int j = 1; j <= sheet.getRow(0).getLastCellNum(); j++) {
                    InvoiceData iData = new InvoiceData();
                    Long idId = idService.nextId();
                    iData.setId(idId);
                    iData.setOrganizationId(orgId);
                    iData.setCompanyCode(companyCode);
                    iData.setTmsPeriod(tmsPeriod);
                    iData.setPeriod(period);
                    iData.setCountType(j);

                    Cell cell1 = sheet.getRow(1).getCell(j);
                    if (null == cell1 || StringUtils.isEmpty(cell1.getStringCellValue())) {
                        continue;
                    }
                    iData.setCompanyName(cell1.getStringCellValue());

                    Cell cell2 = sheet.getRow(2).getCell(j);
                    iData.setSpecialInvoiceAmount1(getCellLongDecimalValue(cell2));

                    Cell cell3 = sheet.getRow(3).getCell(j);
                    iData.setSpecialInvoiceAmount2(getCellLongDecimalValue(cell3));

                    Cell cell4 = sheet.getRow(4).getCell(j);
                    iData.setSpecialInvoiceAmount3(getCellLongDecimalValue(cell4));

                    Cell cell5 = sheet.getRow(5).getCell(j);
                    iData.setSpecialInvoiceAmount4(getCellLongDecimalValue(cell5));

                    Cell cell6 = sheet.getRow(6).getCell(j);
                    iData.setSpecialInvoiceSalesAmount1(getCellBigDecimalValue(cell6));

                    Cell cell7 = sheet.getRow(7).getCell(j);
                    iData.setSpecialInvoiceSalesAmount2(getCellBigDecimalValue(cell7));

                    Cell cell8 = sheet.getRow(8).getCell(j);
                    iData.setSpecialInvoiceSalesAmount3(getCellBigDecimalValue(cell8));

                    Cell cell9 = sheet.getRow(9).getCell(j);
                    iData.setSpecialInvoiceSalesAmount4(getCellBigDecimalValue(cell9));

                    Cell cell10 = sheet.getRow(10).getCell(j);
                    iData.setSpecialInvoiceSalesAmount5(getCellBigDecimalValue(cell10));

                    Cell cell11 = sheet.getRow(11).getCell(j);
                    iData.setSpecialInvoiceTaxAmount1(getCellBigDecimalValue(cell11));

                    Cell cell12 = sheet.getRow(12).getCell(j);
                    iData.setSpecialInvoiceTaxAmount2(getCellBigDecimalValue(cell12));

                    Cell cell13 = sheet.getRow(13).getCell(j);
                    iData.setSpecialInvoiceTaxAmount3(getCellBigDecimalValue(cell13));

                    Cell cell14 = sheet.getRow(14).getCell(j);
                    iData.setSpecialInvoiceTaxAmount4(getCellBigDecimalValue(cell14));

                    Cell cell15 = sheet.getRow(15).getCell(j);
                    iData.setSpecialInvoiceTaxAmount5(getCellBigDecimalValue(cell15));

                    Cell cell16 = sheet.getRow(16).getCell(j);
                    iData.setInvoiceAmount1(getCellLongDecimalValue(cell16));

                    Cell cell17 = sheet.getRow(17).getCell(j);
                    iData.setInvoiceAmount2(getCellLongDecimalValue(cell17));

                    Cell cell18 = sheet.getRow(18).getCell(j);
                    iData.setInvoiceAmount3(getCellLongDecimalValue(cell18));

                    Cell cell19 = sheet.getRow(19).getCell(j);
                    iData.setInvoiceAmount4(getCellLongDecimalValue(cell19));

                    Cell cell20 = sheet.getRow(20).getCell(j);
                    iData.setInvoiceSalesAmount1(getCellBigDecimalValue(cell20));

                    Cell cell21 = sheet.getRow(21).getCell(j);
                    iData.setInvoiceSalesAmount2(getCellBigDecimalValue(cell21));

                    Cell cell22 = sheet.getRow(22).getCell(j);
                    iData.setInvoiceSalesAmount3(getCellBigDecimalValue(cell22));

                    Cell cell23 = sheet.getRow(23).getCell(j);
                    iData.setInvoiceSalesAmount4(getCellBigDecimalValue(cell23));

                    Cell cell24 = sheet.getRow(24).getCell(j);
                    iData.setInvoiceSalesAmount5(getCellBigDecimalValue(cell24));

                    Cell cell25 = sheet.getRow(25).getCell(j);
                    iData.setInvoiceTaxAmount1(getCellBigDecimalValue(cell25));

                    Cell cell26 = sheet.getRow(26).getCell(j);
                    iData.setInvoiceTaxAmount2(getCellBigDecimalValue(cell26));

                    Cell cell27 = sheet.getRow(27).getCell(j);
                    iData.setInvoiceTaxAmount3(getCellBigDecimalValue(cell27));

                    Cell cell28 = sheet.getRow(28).getCell(j);
                    iData.setInvoiceTaxAmount4(getCellBigDecimalValue(cell28));

                    Cell cell29 = sheet.getRow(29).getCell(j);
                    iData.setInvoiceTaxAmount5(getCellBigDecimalValue(cell29));

                    iDatas.add(iData);
                }
                if (iDatas.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgidata period删除记录
                        InvoiceDataExample invoiceDataExample = new InvoiceDataExample();
                        invoiceDataExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        invoiceDataMapper.deleteByExample(invoiceDataExample);
                        if (invoiceDataMapper.countByExample(invoiceDataExample) > 0) {
                            invoiceDataMapper.deleteByExample(invoiceDataExample);
                        }
                    }
                    invoiceDataMapper.insertBatch(iDatas);
                    dataImportLog.setRecordSize(iDatas.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importIDExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }

    }

    public OperationResultDto importRLITExcelFile(MultipartFile file,List<String> orgIds, String periodDate, Integer importType) throws ServiceException{
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<RedLetterInfoTable> rlits = generalRLITs(sheet);
                if (rlits.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<RedLetterInfoTable>> atsGroupRes =
                        rlits.stream().collect(Collectors.groupingBy(RedLetterInfoTable::getSubjectNum));
                atsGroupRes.forEach((k, v) -> processRlits(k, v, orgIds, tmsPeriod, importType,dataImportLogs));
            }
            addDataImportLog(dataImportLogs);
            int isSuccess = (int)dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        }  catch (Exception e) {
            logger.error("importRLITExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processRlits(String companyCode, List<RedLetterInfoTable> rlits,List<String> orgIds,
                              Integer tmsPeriod, Integer importType,List<DataImportLog> dataImportLogs ) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);

        Integer period = rlits.get(0).getFillInDate();

        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);
        DataImportLog dataImportLog = generalDataImportLog(rlits.get(0).getSubjectNum(),"", "",
                EnumImportType.RedLetterInfoTable.getCode(), period/100,
                tmsPeriod%100, period%100,
                rlits.get(0).getSubjectName(), "");
        dataImportLog.setRecordSize(rlits.size());

        if(orgs.size()<1){
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if(!orgIds.contains(orgs.get(0).getId())){
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgs.get(0).getId());
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());
        boolean validateRes = true;
        for (RedLetterInfoTable rlit : rlits) {
            // 校验该主体导入所有记录区间是否一致
            if (!rlit.getFillInDate().equals(period)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!rlit.getFillInDate().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
        }
        if (validateRes&&EnumTbImportType.CoverImport.getCode().equals(importType)) {
            RedLetterInfoTableExample redLetterInfoTableExample = new RedLetterInfoTableExample();
            redLetterInfoTableExample.createCriteria().andSubjectNumEqualTo(companyCode).andFillInDateEqualTo(rlits.get(0).getFillInDate());
            redLetterInfoTableMapper.deleteByExample(redLetterInfoTableExample);
            redLetterInfoTableMapper.insertBatch(rlits);
        }
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    public OperationResultDto importATExcelFile(MultipartFile file,List<String> orgIds,String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<AdjustmentTable> ats = generalATs(sheet);
                if (ats.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<AdjustmentTable>> atsGroupRes =
                        ats.stream().collect(Collectors.groupingBy(AdjustmentTable::getSegment1));
                atsGroupRes.forEach((k, v) -> processAts(k, v, orgIds, tmsPeriod, importType,dataImportLogs));
            }
            addDataImportLog(dataImportLogs);
            int isSuccess = (int)dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        }  catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processAts(String companyCode, List<AdjustmentTable> ats, List<String> orgIds,
                            Integer tmsPeriod, Integer importType,List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);
        DataImportLog dataImportLog = generalDataImportLog(ats.get(0).getSegment1(),"", "",
                EnumImportType.AdjustmentTable.getCode(), ats.get(0).getPeriod()/100,
                tmsPeriod%100, ats.get(0).getPeriod()%100,
                ats.get(0).getSegment1Name(), "");
        dataImportLog.setRecordSize(ats.size());

        if(orgs.size()<1){
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if(!orgIds.contains(orgs.get(0).getId())){
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgs.get(0).getId());
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = ats.get(0).getPeriod();
        boolean validateRes = true;
        for (AdjustmentTable at : ats) {
            // 校验该主体导入所有记录区间是否一致
            if (!at.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!at.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
        }
        if (validateRes&&EnumTbImportType.CoverImport.getCode().equals(importType)) {
            AdjustmentTableExample adjustmentTableExample = new AdjustmentTableExample();
            adjustmentTableExample.createCriteria().andSegment1EqualTo(companyCode).andPeriodEqualTo(ats.get(0).getPeriod());
            adjustmentTableMapper.deleteByExample(adjustmentTableExample);
            adjustmentTableMapper.insertBatch(ats);
        }
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    public OperationResultDto importCPRExcelFile(MultipartFile file,List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CoupaPurchasingReport> ats = generalCPRs(sheet);
                if (ats.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<CoupaPurchasingReport>> atsGroupRes =
                        ats.stream().collect(Collectors.groupingBy(CoupaPurchasingReport::getCompanyCode));
                atsGroupRes.forEach((k, v) -> processCprs(k, v,orgIds, tmsPeriod, importType,dataImportLogs));
            }
            int isSuccess = (int)dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            addDataImportLog(dataImportLogs);
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        }  catch (Exception e) {
            logger.error("importCPRExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processCprs(String companyCode, List<CoupaPurchasingReport> cprs,List<String> orgIds,
                             Integer tmsPeriod, Integer importType,List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);

        DataImportLog dataImportLog = generalDataImportLog(cprs.get(0).getCompanyCode(),"", "",
                EnumImportType.CoupaPurchasingReport.getCode(), cprs.get(0).getPeriod()/100,
                tmsPeriod%100, cprs.get(0).getPeriod()%100,
                "", "");
        dataImportLog.setRecordSize(cprs.size());

        if(orgs.size()<1){
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if(!orgIds.contains(orgs.get(0).getId())){
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgs.get(0).getId());
        dataImportLog.setCompanyName(orgs.get(0).getName());
        dataImportLog.setFileType("Coupa采购报告");
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = cprs.get(0).getPeriod();
        boolean validateRes = true;
        for (CoupaPurchasingReport cpr : cprs) {
            // 校验该主体导入所有记录区间是否一致
            if (!cpr.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!cpr.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
            cpr.setOrganizationId(orgs.get(0).getId());
        }
        if (validateRes&&EnumTbImportType.CoverImport.getCode().equals(importType)) {
            CoupaPurchasingReportExample coupaPurchasingReportExample = new CoupaPurchasingReportExample();
            coupaPurchasingReportExample.createCriteria().andCompanyCodeEqualTo(companyCode).andPeriodEqualTo(cprs.get(0).getPeriod());
            coupaPurchasingReportMapper.deleteByExample(coupaPurchasingReportExample);
            coupaPurchasingReportMapper.insertBatch(cprs);
        }
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    /**
     * 22/02/2019 12:50
     * 导入发票记录表
     * [file, periodDate, importType]
     * @author Gary J Li
     * @return
     */
    public OperationResultDto importCILExcelFile(MultipartFile file, String periodDate, Integer importType) throws ServiceException {

        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<CertifiedInvoicesList> cils = Lists.newArrayList();
                String tayPayerIdNum = sheet.getRow(1).getCell(1).getStringCellValue();
                OrganizationExample example = new OrganizationExample();
                example.createCriteria().andTaxPayerNumberEqualTo(tayPayerIdNum);
                List<Organization> organizations = organizationMapper.selectByExample(example);

                String orgId = "";
                String companyCode = "";

                if (organizations.size() > 0) {
                    orgId = organizations.get(0).getId();
                    companyCode = organizations.get(0).getCode();
                }
                String periodStr = sheet.getRow(1).getCell(4).getStringCellValue();
                String fileType = sheet.getRow(0).getCell(0).getStringCellValue();

                Integer period = DateUtils.strToPeriod2(periodStr);
                Integer tmsPeriod = DateUtils.strToPeriod(periodDate);

                DataImportLog dataImportLog = generalDataImportLog(companyCode, orgId, tayPayerIdNum,
                        EnumImportType.CertifiedInvoicesList.getCode(), period/100,
                        tmsPeriod%100, period%100,
                        "", fileType);
                if (organizations.size() < 1) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String companyName = organizations.get(0).getName();
                dataImportLog.setCompanyName(companyName);
                // 所选期间与导入文件期间不相等 且非 所选期间12期导入文件为13期
                boolean isThirteenPeriod = period / 1000 != 13 && tmsPeriod != 12;
                if (!tmsPeriod.equals(period) && isThirteenPeriod) {
                    dataImportLog.setImportResult(false);
                    dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                    dataImportLogs.add(dataImportLog);
                    continue;
                }
                String unit = sheet.getRow(1).getCell(11).getStringCellValue().replace(" ", "").substring(3);
                for (int j = 3; j < sheet.getLastRowNum(); j++) {
                    Long plId = idService.nextId();
                    CertifiedInvoicesList cil = new CertifiedInvoicesList();
                    cil.setId(plId);
                    cil.setTaxpayerIdNum(tayPayerIdNum);
                    cil.setOrganizationId(orgId);
                    cil.setTmsPeriod(tmsPeriod);
                    cil.setTmsPeriod(tmsPeriod);
                    cil.setPeriod(period);
                    cil.setUnit(unit);

                    Row row = sheet.getRow(j);
                    Cell cell1 = row.getCell(0);
                    if (null == cell1) {
                        continue;
                    }
                    cil.setSeqNo(getCellIntegerValue(cell1));

                    Cell cell2 = row.getCell(1);
                    cil.setInvoiceCode(getCellStringValue(cell2));

                    Cell cell3 = row.getCell(2);
                    cil.setInvoiceNum(getCellStringValue(cell3));

                    Cell cell4 = row.getCell(3);
                    cil.setBillingDate(DateUtils.strToDate4(getCellStringValue(cell4)));

                    Cell cell5 = row.getCell(4);
                    cil.setSalesTaxNum(getCellStringValue(cell5));

                    Cell cell6 = row.getCell(5);
                    cil.setSalespersonName(getCellStringValue(cell6));

                    Cell cell7 = row.getCell(6);
                    cil.setAmount(getCellBigDecimalValue(cell7));

                    Cell cell8 = row.getCell(7);
                    cil.setTaxAmount(getCellBigDecimalValue(cell8));

                    Cell cell9 = row.getCell(8);
                    cil.setVerificationMethod(getCellStringValue(cell9));

                    Cell cell10 = row.getCell(9);
                    cil.setCertifiedDate(DateUtils.strToDate4(getCellStringValue(cell10)));

                    Cell cell11 = row.getCell(10);
                    cil.setInvoiceType(getCellStringValue(cell11));

                    Cell cell12 = row.getCell(11);
                    cil.setInvoiceStatus(getCellStringValue(cell12));

                    cils.add(cil);
                }
                if (cils.size() > 0) {
                    // 1、写入最终表
                    // 2、记录数据导入记录与数据处理校验记录
                    if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
                        // 根据orgId period删除记录
                        CertifiedInvoicesListExample certifiedInvoicesListExample = new CertifiedInvoicesListExample();
                        certifiedInvoicesListExample.createCriteria().andOrganizationIdEqualTo(orgId).andPeriodEqualTo(period);
                        certifiedInvoicesListMapper.deleteByExample(certifiedInvoicesListExample);
                        if (certifiedInvoicesListMapper.countByExample(certifiedInvoicesListExample) > 0) {
                            certifiedInvoicesListMapper.deleteByExample(certifiedInvoicesListExample);
                        }
                    }
                    certifiedInvoicesListMapper.insertBatch(cils);
                    dataImportLog.setRecordSize(cils.size());
                    dataImportLog.setImportResult(true);
                    dataImportLogs.add(dataImportLog);
                }
            }
            if (addDataImportLog(dataImportLogs) < 1) {
                return OperationResultDto.error(ErrorMessage.SystemError);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            logger.error("importCILExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    public OperationResultDto importIRExcelFile(MultipartFile file,List<String> orgIds, String periodDate, Integer importType) throws ServiceException {
        try {
            InputStream inputStream = file.getInputStream();
            Workbook workbook = WorkbookFactory.create(inputStream);
            if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
                throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
            }
            Integer tmsPeriod = DateUtils.strToPeriod(periodDate);
            List<DataImportLog> dataImportLogs = Lists.newArrayList();
            for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                Sheet sheet = workbook.getSheetAt(i);
                if (isSheetEmpty(sheet)) continue;
                List<InvoiceRecord> irs = generalIRs(sheet);
                if (irs.size() < 1) continue;
                // 根据主体分组处理
                Map<String, List<InvoiceRecord>> atsGroupRes =
                        irs.stream().collect(Collectors.groupingBy(InvoiceRecord::getBillingBody));
                atsGroupRes.forEach((k, v) -> processIrs(k, v,orgIds, tmsPeriod, importType,dataImportLogs));
            }
            int isSuccess = (int)dataImportLogs.stream().filter(di -> Boolean.TRUE.equals(di.getImportResult())).count();
            addDataImportLog(dataImportLogs);
            if (isSuccess < 1) {
                return OperationResultDto.error(ErrorMessage.ImportFailed);
            }
            return OperationResultDto.success();
        } catch (ServiceException e) {
            throw e;
        }  catch (Exception e) {
            logger.error("importIRExcelFile error.", e);
            throw new ServiceException(ErrorMessage.SystemError);
        }
    }

    private void processIrs(String billingBody, List<InvoiceRecord> irs,List<String> orgIds,
                             Integer tmsPeriod, Integer importType,List<DataImportLog> dataImportLogs) {
        OrganizationExample organizationExample = new OrganizationExample();
        organizationExample.createCriteria().andCodeEqualTo(billingBody);
        List<Organization> orgs = organizationMapper.selectByExample(organizationExample);

        DataImportLog dataImportLog = generalDataImportLog(irs.get(0).getBillingBody(),"", "",
                EnumImportType.InvoiceRecord.getCode(), irs.get(0).getPeriod()/100,
                tmsPeriod%100, irs.get(0).getPeriod()%100,
                "", "");
        dataImportLog.setRecordSize(irs.size());

        if(orgs.size()<1){
            dataImportLog.setErrorMsg(ErrorMessageCN.NoCompanyError);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        if(!orgIds.contains(orgs.get(0).getId())){
            dataImportLog.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            dataImportLog.setImportResult(false);
            dataImportLogs.add(dataImportLog);
            return;
        }

        dataImportLog.setOrganizationId(orgs.get(0).getId());
        dataImportLog.setCompanyName(orgs.get(0).getName());
        dataImportLog.setFileType("已开增值税发票记录");
        dataImportLog.setTaxpayerIdNum(StringUtils.isNotEmpty(orgs.get(0).getLegalCode()) ?
                orgs.get(0).getLegalCode() : orgs.get(0).getTaxPayerNumber());

        Integer period1 = irs.get(0).getPeriod();
        boolean validateRes = true;
        for (InvoiceRecord ir : irs) {
            // 校验该主体导入所有记录区间是否一致
            if (!ir.getPeriod().equals(period1)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.InconsistentPeriod);
                validateRes = false;
                break;
            }
            if (!ir.getPeriod().equals(tmsPeriod)) {
                dataImportLog.setImportResult(false);
                dataImportLog.setErrorMsg(ErrorMessageCN.ExistDataPeriodsError);
                validateRes = false;
                break;
            }
            ir.setOrganizationId(orgs.get(0).getId());
            ir.setTmsPeriod(tmsPeriod);
            ir.setOrganizationId(orgs.get(0).getId());
        }
        if (validateRes&&EnumTbImportType.CoverImport.getCode().equals(importType)) {
            InvoiceRecordExample invoiceRecordExample = new InvoiceRecordExample();
            invoiceRecordExample.createCriteria().andBillingBodyEqualTo(billingBody).andPeriodEqualTo(irs.get(0).getPeriod());
            invoiceRecordMapper.deleteByExample(invoiceRecordExample);
            invoiceRecordMapper.insertBatch(irs);
        }
        // 1、记录数据导入记录与数据处理校验记录
        dataImportLogs.add(dataImportLog);
    }

    private List<RedLetterInfoTable> generalRLITs(Sheet sheet) {
        List<RedLetterInfoTable> rlits = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            RedLetterInfoTable rlit = new RedLetterInfoTable();
            Long plId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            rlit.setId(plId);
            rlit.setSubjectNum(getCellStringValue(row.getCell(0)));
            rlit.setSubjectName(getCellStringValue(row.getCell(1)));
            rlit.setRedLetterInvoiceInfoTableNum(getCellStringValue(row.getCell(2)));
            rlit.setFillInDate(DateUtils.dateToPeriod(row.getCell(3).getDateCellValue()));
            rlit.setSalesTaxNumber(getCellStringValue(row.getCell(4)));
            rlit.setSalespersonName(getCellStringValue(row.getCell(5)));
            rlit.setTotalAmount(getCellBigDecimalValue(row.getCell(6)));
            rlit.setTotalTaxAmount(getCellBigDecimalValue(row.getCell(7)));
            rlit.setApplicationDescription(getCellStringValue(row.getCell(8)));
            rlit.setApplicantManager(getCellStringValue(row.getCell(9)));
            rlit.setInvoiceCode(getCellStringValue(row.getCell(10)));
            rlit.setInvoiceNumber(getCellStringValue(row.getCell(11)));
            rlits.add(rlit);
        }
        return rlits;
    }

    private List<InvoiceRecord> generalIRs(Sheet sheet) {
        List<InvoiceRecord> irs = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            InvoiceRecord ir = new InvoiceRecord();
            Long irId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            ir.setId(irId);
            ir.setSeqNo(getCellIntegerValue(row.getCell(0)));
            ir.setBillingBody(getCellStringValue(row.getCell(1)));
            ir.setCustomerCompanyName(getCellStringValue(row.getCell(2)));
            ir.setInvoiceType(getCellStringValue(row.getCell(3)));
            ir.setBillingContent(getCellStringValue(row.getCell(4)));
            ir.setInvoiceAmount(getCellBigDecimalValue(row.getCell(5)));
            ir.setApplicant(getCellStringValue(row.getCell(6)));
            ir.setOaApplicationNum(getCellStringValue(row.getCell(7)));
            ir.setContractNo(getCellStringValue(row.getCell(8)));
            ir.setContractAmount(getCellBigDecimalValue(row.getCell(9)));
            ir.setDepartment(getCellStringValue(row.getCell(10)));
            ir.setApplicationDate(row.getCell(11).getDateCellValue());
            ir.setBillingDate(row.getCell(12).getDateCellValue());
            ir.setPeriod(DateUtils.dateToPeriod(ir.getBillingDate()));
            ir.setBillingMonth(row.getCell(13).getDateCellValue());
            ir.setInvoiceCode(getCellStringValue(row.getCell(14)));
            ir.setInvoiceNum(getCellStringValue(row.getCell(15)));
            ir.setRemarks(getCellStringValue(row.getCell(16)));
            ir.setInvoicesAmount(getCellIntegerValue(row.getCell(17)));
            ir.setCustomerCompanyTaxNum(getCellStringValue(row.getCell(18)));
            ir.setContractSourceSystem(getCellStringValue(row.getCell(19)));
            ir.setTaxRate(getCellBigDecimalValue(row.getCell(20)));
            ir.setTaxAmount(getCellBigDecimalValue(row.getCell(21)));
            ir.setInvoiceStatus(getCellStringValue(row.getCell(22)));
            ir.setSource(getCellStringValue(row.getCell(23)));
            irs.add(ir);
        }
        return irs;
    }

    private List<AdjustmentTable> generalATs(Sheet sheet) {
        List<AdjustmentTable> ats = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            AdjustmentTable at = new AdjustmentTable();
            Long plId = idService.nextId();
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            at.setId(plId);
            String periodStr = getCellStringValue(row.getCell(0));
            Integer period = Integer.valueOf(periodStr.substring(0,4)+periodStr.substring(5,7));
            at.setPeriod(period);
            at.setSegment1(getCellStringValue(row.getCell(1)));
            at.setSegment2(getCellStringValue(row.getCell(2)));
            at.setSegment3(getCellStringValue(row.getCell(3)));
            at.setSegment4(getCellStringValue(row.getCell(4)));
            at.setSegment5(getCellStringValue(row.getCell(5)));
            at.setSegment6(getCellStringValue(row.getCell(6)));
            at.setSegment7(getCellStringValue(row.getCell(7)));
            at.setSegment8(getCellStringValue(row.getCell(8)));
            at.setSegment9(getCellStringValue(row.getCell(9)));
            at.setSegment10(getCellStringValue(row.getCell(10)));
            at.setSegment1Name(getCellStringValue(row.getCell(11)));
            at.setSegment2Name(getCellStringValue(row.getCell(12)));
            at.setSegment3Name(getCellStringValue(row.getCell(13)));
            at.setSegment4Name(getCellStringValue(row.getCell(14)));
            at.setSegment5Name(getCellStringValue(row.getCell(15)));
            at.setSegment6Name(getCellStringValue(row.getCell(16)));
            at.setSegment7Name(getCellStringValue(row.getCell(17)));
            at.setSegment8Name(getCellStringValue(row.getCell(18)));
            at.setSegment9Name(getCellStringValue(row.getCell(19)));
            at.setSegment10Name(getCellStringValue(row.getCell(20)));
            at.setPeriodDrBeq(getCellBigDecimalValue(row.getCell(21)));
            at.setPeriodCrBeq(getCellBigDecimalValue(row.getCell(22)));
            at.setEndBalBeq(getCellBigDecimalValue(row.getCell(23)));
            ats.add(at);
        }
        return ats;
    }

    private List<CoupaPurchasingReport> generalCPRs(Sheet sheet) {
        List<CoupaPurchasingReport> cprs = Lists.newArrayList();
        for (int j = 1; j <= sheet.getLastRowNum(); j++) {
            CoupaPurchasingReport cpr = new CoupaPurchasingReport();
            Long id = idService.nextId();
            cpr.setId(id);
            Row row = sheet.getRow(j);
            if (null == row.getCell(0)) {
                continue;
            }
            cpr.setInvoiceNum(getCellStringValue(row.getCell(0)));
            cpr.setLineNum(getCellIntegerValue(row.getCell(1)));
            cpr.setHeadUpStatus(getCellStringValue(row.getCell(2)));
            cpr.setInvoiceDate(DateUtils.strToDate3(getCellStringValue(row.getCell(3))));
            cpr.setInvoiceReceiptDate(DateUtils.strToDate3(getCellStringValue(row.getCell(4))));
            cpr.setPayExpiration(getCellStringValue(row.getCell(5)));
            cpr.setPurchaseOrderNum(getCellStringValue(row.getCell(6)));
            cpr.setPurchaseOrderLineNum(getCellIntegerValue(row.getCell(7)));
            cpr.setDescription(getCellStringValue(row.getCell(8)));
            cpr.setSupplierNum(getCellStringValue(row.getCell(9)));
            cpr.setSupplier(getCellStringValue(row.getCell(10)));
            cpr.setTotalAmount(getCellBigDecimalValue(row.getCell(11)));
            cpr.setCurrency(getCellStringValue(row.getCell(12)));
            String billNum = getCellStringValue(row.getCell(13));
            cpr.setBillNum(billNum);
            cpr.setCompanyCode(StringUtils.isNotEmpty(billNum)?billNum.substring(0,6):"");
            cpr.setPaid(getCellBolleanValue(row.getCell(14)));
            cpr.setAmortization(getCellBolleanValue(row.getCell(15)));
            cpr.setPrepayments(getCellBolleanValue(row.getCell(16)));
            cpr.setReviewReason(getCellStringValue(row.getCell(17)));
            cpr.setCommodity(getCellStringValue(row.getCell(18)));
            cpr.setCommodityCode(getCellStringValue(row.getCell(19)));
            cpr.setFounder(getCellStringValue(row.getCell(20)));
            cpr.setCreateDate(DateUtils.strToDate3(getCellStringValue(row.getCell(21))));
            cpr.setCurrentApprover(getCellStringValue(row.getCell(22)));
            cpr.setBillingDate(DateUtils.strToDate3(getCellStringValue(row.getCell(23))));
            cpr.setPeriod(DateUtils.dateToPeriod(cpr.getBillingDate()));
            cpr.setLastUpdatedDate(DateUtils.strToDate3(getCellStringValue(row.getCell(24))));
            cpr.setLastUpdatedPerson(getCellStringValue(row.getCell(25)));
            cpr.setTotalTaxAmount(getCellBigDecimalValue(row.getCell(26)));
            cpr.setTotalTaxAmountHeader(getCellBigDecimalValue(row.getCell(27)));
            cpr.setBillingNote(getCellStringValue(row.getCell(28)));
            cprs.add(cpr);
        }
        return cprs;
    }

    private String getCellStringValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return cell.getStringCellValue();
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return String.valueOf(cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Boolean getCellBolleanValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            switch (cell.getStringCellValue()) {
                case "TRUE":
                    return Boolean.TRUE;
                case "FALSE":
                    return Boolean.FALSE;
                case "是":
                    return Boolean.TRUE;
                case "否":
                    return Boolean.FALSE;
                default:
                    return null;
            }
        }else if (cell.getCellTypeEnum().equals(CellType.BOOLEAN)) {
            return cell.getBooleanCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Integer getCellIntegerValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Integer.valueOf(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return (int)cell.getNumericCellValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private BigDecimal getCellBigDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return new BigDecimal(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new BigDecimal(cell.getNumericCellValue());
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }

    private Long getCellLongDecimalValue(Cell cell) {
        if (cell.getCellTypeEnum().equals(CellType.STRING)) {
            return Long.valueOf(cell.getStringCellValue());
        } else if(cell.getCellTypeEnum().equals(CellType.NUMERIC)) {
            return new Double(cell.getNumericCellValue()).longValue();
        }
        logger.warn("获取单元格数据类型未匹配");
        return null;
    }


    private int addDataImportLog(List<DataImportLog> dataImportLogs) {
        dataImportLogMapper.displayOld(dataImportLogs.get(0).getType());
        int res = 0;
        for (DataImportLog dataImportLog : dataImportLogs) {
            res += dataImportLogMapper.insertSelective(dataImportLog);
        }
        return res;
    }

    public List<DataImportLogDto> displayImportLog(Integer type) {
        List<DataImportLogDto> dataImportLogDtos = Lists.newArrayList();
        DataImportLogExample dataImportLogExample = new DataImportLogExample();
        if(EnumImportType.Undefined.getCode().equals(type)){
            dataImportLogExample.setOrderByClause("update_time desc");
        }else{
            dataImportLogExample.createCriteria().andTypeEqualTo(type).andDisplayEqualTo(true);
            dataImportLogExample.setOrderByClause("update_time desc");
        }
        dataImportLogMapper.selectByExample(dataImportLogExample).forEach( dil ->{
            DataImportLogDto dataImportLogDto = new DataImportLogDto();
            beanUtil.copyProperties(dil,dataImportLogDto);
            dataImportLogDtos.add(dataImportLogDto);
        });
        return dataImportLogDtos;
    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (sheet.getLastRowNum() > 0 &&
                (null == sheet.getRow(0).getCell(0) || "Version".equals(sheet.getRow(0).getCell(0).getStringCellValue())) &&
                     null == sheet.getRow(0).getCell(2)) {
            return true;
        }
        return false;
    }
}
