package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.dto.vatdto.dd.*;
import pwc.taxtech.atms.dto.vatdto.dd.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.excelheader.CashFlowHeader;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.*;
import pwc.taxtech.atms.vat.dpo.TrialBalanceCondition;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.*;

/**
 * @Auther: Gary J Li
 * @Date: 01/02/2019 10:36
 * @Description:
 */
@Service
public class DataPreviewSerivceImpl extends BaseService {

    @Resource
    private TrialBalanceMapper trialBalanceMapper;

    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;

    @Resource
    private JournalEntryMapper journalEntryMapper;

    @Resource
    private BalanceSheetMapper balanceSheetMapper;

    @Resource
    private CashFlowMapper cashFlowMapper;

    @Resource
    private InvoiceRecordMapper invoiceRecordMapper;

    @Resource
    private RedLetterInfoTableMapper redLetterInfoTableMapper;

    @Resource
    private CoupaPurchasingReportMapper coupaPurchasingReportMapper;

    @Resource
    private CertifiedInvoicesListMapper certifiedInvoicesListMapper;

    @Resource
    private OrganizationMapper organizationMapper;

    @Resource
    private CommonDocumentHelper commonDocumentHelper;

    public PageInfo<TrialBalanceDto> getTBDataForDisplay(TrialBalanceParam param) {

        TrialBalanceCondition condition = beanUtil.copyProperties(param, new TrialBalanceCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<TrialBalance> trialBalances = trialBalanceMapper.selectByCondition(condition);
        List<TrialBalanceDto> trialBalanceDtos = Lists.newArrayList();

        trialBalances.forEach(tb -> {
            TrialBalanceDto trialBalanceDto = new TrialBalanceDto();
            beanUtil.copyProperties(tb, trialBalanceDto);
            trialBalanceDtos.add(trialBalanceDto);
        });

        PageInfo<TrialBalanceDto> pageInfo =new PageInfo<>(trialBalanceDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<ProfitLossStatementDto> getPLDataForDisplay(ProfitLossStatementParam param) {

        ProfitLossStatementCondition condition = beanUtil.copyProperties(param, new ProfitLossStatementCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<ProfitLossStatement> profitLossStatements = profitLossStatementMapper.selectByCondition(condition);
        List<ProfitLossStatementDto> profitLossDtos = Lists.newArrayList();

        profitLossStatements.forEach(pl -> {
            ProfitLossStatementDto profitLossDto = new ProfitLossStatementDto();
            beanUtil.copyProperties(pl, profitLossDto);
            profitLossDtos.add(profitLossDto);
        });

        PageInfo<ProfitLossStatementDto> pageInfo =new PageInfo<>(profitLossDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<CashFlowDto> getCFDataForDisplay(CashFlowParam param) {
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        //Integer totalCount=cashFlowMapper.selectCountByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        List<CashFlow> cashFlows = cashFlowMapper.selectByCondition(condition);
        //使用page的getTotal()
        Long total = page.getTotal();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        PageInfo<CashFlowDto> pageInfo=new PageInfo<>(cashFlowDtos);
        pageInfo.setTotal(total);
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public HttpServletResponse exportCashFlowList(HttpServletResponse response, CashFlowParam param, String fileName) {
        //Boolean isEn = StringUtils.equals(language, "en-us");
        logger.debug("start export input invoice list to excel");
        //String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow"+(isEn?"":"_cn") + ".xlsx";
        String excelTemplatePathInClassPath = "/vat_excel_template/cash_flow.xlsx";
        CashFlowCondition condition = new CashFlowCondition();
        beanUtil.copyProperties(param, condition);
        PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        List<CashFlow> cashFlows = cashFlowMapper.selectByCondition(condition);
        List<CashFlowDto> cashFlowDtos = Lists.newArrayList();
        cashFlows.forEach(cf -> {
            CashFlowDto cashFlowDto = new CashFlowDto();
            beanUtil.copyProperties(cf, cashFlowDto);
            cashFlowDtos.add(cashFlowDto);
        });
        CashFlowHeader cashFlowHeader=new CashFlowHeader();
        if(cashFlowDtos.size()>0){
            Organization org = organizationMapper.selectByPrimaryKey(param.getOrgId());
            cashFlowHeader.setCompanyNameCn(org.getName());
            cashFlowHeader.setPeriodStart(param.getPeriodStart());
            cashFlowHeader.setPeriodEnd(param.getPeriodEnd());
            cashFlowHeader.setLedgerName(cashFlowDtos.get(0).getLedgerName());
            cashFlowHeader.setLedgerCurrencyCode(cashFlowDtos.get(0).getLedgerCurrencyCode());
            cashFlowHeader.setStatus(cashFlowDtos.get(0).getStatus());
        }
        OutputStream outputStream = commonDocumentHelper.toXlsxFileUsingJxls(cashFlowHeader, cashFlowDtos, excelTemplatePathInClassPath);

        try {
            return responseMessageBuilder.getDownloadTmpResponseMessage(response, outputStream, fileName);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
        return null;
    }

    public PageInfo<JournalEntryDto> getJEDataForDisplay(JournalEntryParam param) {

        JournalEntryCondition condition = beanUtil.copyProperties(param, new JournalEntryCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<JournalEntry> journalEntrys = journalEntryMapper.selectByCondition(condition);
        List<JournalEntryDto> journalEntryDtos = Lists.newArrayList();

        journalEntrys.forEach(je -> {
            JournalEntryDto journalEntryDto = new JournalEntryDto();
            beanUtil.copyProperties(je, journalEntryDto);
            journalEntryDtos.add(journalEntryDto);
        });

        PageInfo<JournalEntryDto> pageInfo =new PageInfo<>(journalEntryDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<BalanceSheetDto> getBSDataForDisplay(BalanceSheetParam param) {

        BalanceSheetCondition condition = beanUtil.copyProperties(param, new BalanceSheetCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<BalanceSheet> balanceSheets = balanceSheetMapper.selectByCondition(condition);
        List<BalanceSheetDto> balanceSheetDtos = Lists.newArrayList();

        balanceSheets.forEach(bs -> {
            BalanceSheetDto balanceSheetDto = new BalanceSheetDto();
            beanUtil.copyProperties(bs, balanceSheetDto);
            balanceSheetDtos.add(balanceSheetDto);
        });

        PageInfo<BalanceSheetDto> pageInfo =new PageInfo<>(balanceSheetDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<InvoiceRecordDto> getIRDataForDisplay(InvoiceRecordParam param) {

        InvoiceRecordCondition condition = beanUtil.copyProperties(param, new InvoiceRecordCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<InvoiceRecord> invoiceRecords = invoiceRecordMapper.selectByCondition(condition);

        List<InvoiceRecordDto> invoiceRecordDtos = Lists.newArrayList();

        invoiceRecords.forEach(bs -> {
            InvoiceRecordDto balanceSheetDto = beanUtil.copyProperties(bs, new InvoiceRecordDto());
            invoiceRecordDtos.add(balanceSheetDto);
        });

        PageInfo<InvoiceRecordDto> pageInfo =new PageInfo<>(invoiceRecordDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }

    public PageInfo<RedLetterInfoTableDto> getRLITDataForDisplay(RedLetterInfoTableParam param) {

        RedLetterInfoTableCondition condition = beanUtil.copyProperties(param, new RedLetterInfoTableCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<RedLetterInfoTable> redLetterInfoTables = redLetterInfoTableMapper.selectByCondition(condition);
        List<RedLetterInfoTableDto> redLetterInfoDtos = Lists.newArrayList();

        redLetterInfoTables.forEach(bs -> {
            RedLetterInfoTableDto redLetterInfoDto = beanUtil.copyProperties(bs, new RedLetterInfoTableDto());
            redLetterInfoDtos.add(redLetterInfoDto);
        });

        PageInfo<RedLetterInfoTableDto> pageInfo =new PageInfo<>(redLetterInfoDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }


    public PageInfo<CoupaPurchasingReportDto> getCPRDataForDisplay(CoupaPurchasingReportParam param) {

        CoupaPurchasingReportCondition condition = beanUtil.copyProperties(param, new CoupaPurchasingReportCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());

        List<CoupaPurchasingReport> coupaPurchasingReports = coupaPurchasingReportMapper.selectByCondition(condition);
        List<CoupaPurchasingReportDto> coupaPurchasingReportDtos = Lists.newArrayList();

        coupaPurchasingReports.forEach(cpr -> {
            CoupaPurchasingReportDto coupaPurchasingReportDto = beanUtil.copyProperties(cpr, new CoupaPurchasingReportDto());
            coupaPurchasingReportDtos.add(coupaPurchasingReportDto);
        });

        PageInfo<CoupaPurchasingReportDto> pageInfo =new PageInfo<>(coupaPurchasingReportDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());

        return pageInfo;
    }


    public PageInfo<CertifiedInvoicesListDto> getCILDataForDisplay(CertifiedInvoicesListParam param) {

        CertifiedInvoicesListCondition condition = beanUtil.copyProperties(param, new CertifiedInvoicesListCondition());

        Page page = PageHelper.startPage(condition.getPageInfo().getPageIndex(), condition.getPageInfo().getPageSize());
        List<CertifiedInvoicesList> certifiedInvoicesLists = certifiedInvoicesListMapper.selectByCondition(condition);
        List<CertifiedInvoicesListDto> certifiedInvoicesListDtos = Lists.newArrayList();

        certifiedInvoicesLists.forEach(cil -> {
            CertifiedInvoicesListDto CertifiedInvoicesListDto = beanUtil.copyProperties(cil, new CertifiedInvoicesListDto());
            certifiedInvoicesListDtos.add(CertifiedInvoicesListDto);
        });
        PageInfo<CertifiedInvoicesListDto> pageInfo =new PageInfo<>(certifiedInvoicesListDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(param.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public int getTBDownloadFilePath(TrialBalanceParam param, OutputStream os) {
        try {
            TrialBalanceCondition condition = new TrialBalanceCondition();
            beanUtil.copyProperties(param, condition);
            List<TrialBalance> trialBalances = trialBalanceMapper.selectByCondition(condition);

            Map<String, String> header = generalTBHeader();
            List<TrialBalanceExportDto> cellList = new ArrayList<>();
            trialBalances.forEach(tb -> {
                TrialBalanceExportDto d = new TrialBalanceExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("科目余额表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }


    public int getPLDownloadFilePath(ProfitLossStatementParam param, OutputStream os) {
        try {

            ProfitLossStatementCondition condition = new ProfitLossStatementCondition();
            beanUtil.copyProperties(param, condition);
            List<ProfitLossStatement> profitLossStatements = profitLossStatementMapper.selectByCondition(condition);
            Map<String, String> header = generalPLHeader();
            List<ProfitLossStatementExportDto> cellList = new ArrayList<>();
            profitLossStatements.forEach(pl -> {
                ProfitLossStatementExportDto d = new ProfitLossStatementExportDto();
                d = beanUtil.copyProperties(pl, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("利润表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getJEDownloadFilePath(JournalEntryParam param, OutputStream os) {
        try {
            JournalEntryCondition condition = new JournalEntryCondition();
            beanUtil.copyProperties(param, condition);
            List<JournalEntry> journalEntries = journalEntryMapper.selectByCondition(condition);

            Map<String, String> header = generalJEHeader();
            List<JournalEntryExportDto> cellList = new ArrayList<>();
            journalEntries.forEach(tb -> {
                JournalEntryExportDto d = new JournalEntryExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("日记账导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    public int getBSDownloadFilePath(BalanceSheetParam param, OutputStream os) {
        try {
            BalanceSheetCondition condition = new BalanceSheetCondition();
            beanUtil.copyProperties(param, condition);
            List<BalanceSheet> balanceSheets = balanceSheetMapper.selectByCondition(condition);

            Map<String, String> header = generalBSHeader();
            List<BalanceSheetExportDto> cellList = new ArrayList<>();
            balanceSheets.forEach(bs -> {
                BalanceSheetExportDto d = new BalanceSheetExportDto();
                d = beanUtil.copyProperties(bs, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("资产负债表导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    private Map<String, String> generalTBHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date","数据日期");
        header.put("Source","来源");
        header.put("Period","期间");
        header.put("LedgerId","账套ID");
        header.put("LedgerName","账套名称");
        header.put("CurrencyCode","账套币种");
        header.put("Status","关账标识");
        header.put("Category","主体性质");
        header.put("AccountCategory","科目性质");
        header.put("AcctCode1","一级科目代码");

        header.put("AcctName1","一级科目说明");
        header.put("AcctName2","二级科目说明");
        header.put("AcctName3","三级科目说明");
        header.put("Segment1","主体代码");
        header.put("Segment2","成本中心代码");
        header.put("Segment3","科目代码");
        header.put("Segment4","辅助科目代码");
        header.put("Segment5","利润中心代码");
        header.put("Segment6","产品代码");
        header.put("Segment7","项目代码");

        header.put("Segment8","公司间代码");
        header.put("Segment9","备用1代码");
        header.put("Segment10","备用2代码");
        header.put("Segment1Name","主体说明");
        header.put("Segment2Name","成本中心说明");
        header.put("Segment3Name","科目说明");
        header.put("Segment4Name","辅助科目说明");
        header.put("Segment5Name","利润中心说明");
        header.put("Segment6Name","产品说明");
        header.put("Segment7Name","项目说明");

        header.put("Segment8Name","公司间说明");
        header.put("Segment9Name","备用1说明");
        header.put("Segment10Name","备用2说明");
        header.put("BegBal","原币本期期初余额");
        header.put("PeriodDr","原币本期借方发生额");
        header.put("PeriodCr","原币本期贷方发生额");
        header.put("EndBal","原币本期期末余额");
        header.put("QtdDr","原币本季借方发生额");
        header.put("QtdCr","原币本季贷方发生额");
        header.put("YtdDr","原币本年借方发生额");

        header.put("YtdCr","原币本年贷方发生额");
        header.put("BegBalBeq","本位币本期期初余额");
        header.put("PeriodDrBeq","本位币本期借方发生额");
        header.put("PeriodCrBeq","本位币本期贷方发生额");
        header.put("EndBalBeq","本位币本期期末余额");
        header.put("QtdDrBeq","本位币本季借方发生额");
        header.put("QtdCrBeq","本位币本季贷方发生额");
        header.put("YtdDrBeq","本位币本年借方发生额");
        header.put("YtdCrBeq","本位币本年贷方发生额");
        return header;
    }

    private Map<String, String> generalBSHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date","数据日期");
        header.put("Source","来源");
        header.put("Period","期间");
        header.put("Status","关账标识");
        header.put("LedgerId","账套ID");
        header.put("LedgerName","账套名称");
        header.put("LedgerCurrencyCode","账套币种");
        header.put("EntityCode","机构编码");
        header.put("EntityName","机构名称");
        header.put("Category","主体性质");
        header.put("Frequency","频度");
        header.put("ItemName","项目名称");
        header.put("EndBal","期末余额");
        header.put("BegBal","年初余额");
        header.put("PrcFlag","是否为国外");
        return header;
    }



    private Map<String, String> generalPLHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date","数据日期");
        header.put("Source","来源");
        header.put("Period","期间");
        header.put("Status","关账标识");
        header.put("LedgerId","账套ID");
        header.put("LedgerName","账套名称");
        header.put("LedgerCurrencyCode","账套币种");
        header.put("EntityCode","机构编码");
        header.put("EntityName","机构名称");
        header.put("Category","主体性质");

        header.put("Frequency","频度");
        header.put("ItemName","项目名称");
        header.put("PeriodAmt","本期发生额");
        header.put("YtdAmt","本年累计");
        header.put("PrcFlag","是否为国外");
        return header;
    }



    private Map<String, String> generalJEHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("Date","数据日期");
        header.put("Source","来源");
        header.put("LedgerId","账套ID");
        header.put("LedgerName","账套名称");
        header.put("CurrencyCode","账套币种");
        header.put("Status","关账标识");
        header.put("HeaderId","日记账头ID");
        header.put("LineNum","日记账行号");
        header.put("ApprovalStatus","审批状态");
        header.put("PostedStatus","过账");

        header.put("Period","会计期间");
        header.put("AccountingDate","凭证日期");
        header.put("JournalSource","日记账来源");
        header.put("Category","日记账类别");
        header.put("Name","日记账名称");
        header.put("VoucherNum","凭证编号");
        header.put("Description","摘要");
        header.put("Segment1","主体代码");
        header.put("Segment2","成本中心");
        header.put("Segment3","科目代码");

        header.put("Segment4","辅助科目");
        header.put("Segment5","利润中心");
        header.put("Segment6","产品");
        header.put("Segment7","项目");
        header.put("Segment8","公司间");
        header.put("Segment9","备用1");
        header.put("Segment10","备用2");
        header.put("Segment1Name","主体说明");
        header.put("Segment2Name","成本中心说明");
        header.put("Segment3Name","科目说明");

        header.put("Segment4Name","辅助科目说明");
        header.put("Segment5Name","利润中心说明");
        header.put("Segment6Name","产品说明");
        header.put("Segment7Name","项目说明");
        header.put("Segment8Name","公司间说明");
        header.put("Segment9Name","备用1说明");
        header.put("Segment10Name","备用2说明");
        header.put("JournalCurrencyCode","币种");
        header.put("SobCurrencyCode","本位币币种");
        header.put("AccountedDr","借方金额");

        header.put("AccountedCr","贷方金额");
        header.put("EnteredDr","本位币借方金额");
        header.put("EnteredCr","本位币贷方金额");
        header.put("CfItem","现金流量表项");
        header.put("Attribute1","城市");
        header.put("Attribute2","交易日期");
        header.put("Attribute3","对方银行账号");
        header.put("Attribute4","银行流水号");
        header.put("Attribute5","供应商编号");
        header.put("Attribute6","交易单号");

        header.put("Attribute7","供应商名称");
        header.put("Attribute8","接收编码");
        header.put("Attribute9","制单人");
        header.put("Attribute10","审核人");
        header.put("Attribute11","成本中心部门描述1");
        header.put("Attribute12","成本中心部门描述2");
        header.put("Attribute13","成本中心部门描述3");
        header.put("Attribute14","成本中心部门描述4");
        header.put("Attribute15","成本中心部门描述5");
        header.put("Attribute16","成本中心部门描述6");

        header.put("CreatedBy","创建人");
        header.put("CreatedDate","创建日期");
        header.put("LateUpdatedBy","最后更新人");
        header.put("LateUpdatedDate","最后更新日期");
        return header;
    }
}
