package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.nutz.lang.Lang;
import org.nutz.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.util.BeanUtil;
import pwc.taxtech.atms.dao.OrganizationAccountingRateMapper;
import pwc.taxtech.atms.dto.ebsdto.*;
import pwc.taxtech.atms.entity.OrganizationAccountingRate;
import pwc.taxtech.atms.entity.OrganizationAccountingRateExample;
import pwc.taxtech.atms.service.EbsApiService;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class EbsApiServiceImpl implements EbsApiService {

    @Resource
    private JournalEntryMapper journalEntryMapper;
    @Resource
    private TrialBalanceMapper trialBalanceMapper;
    @Resource
    private CashFlowMapper cashFlowMapper;
    @Resource
    private BalanceSheetMapper balanceSheetMapper;
    @Resource
    private ProfitLossStatementMapper profitLossStatementMapper;
    @Resource
    private OrganizationAccountingRateMapper organizationAccountingRateMapper;
    @Resource
    private DistributedIdService distributedIdService;
    @Resource
    private BeanUtil beanUtil;

    private static final Logger logger = LoggerFactory.getLogger(EbsApiServiceImpl.class);

    @Override
    public void queryRemoteServerThenUpdateJE(List<JournalEntryQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateJE 日记账表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS JournalEntry 日记账表 response, skip processing");
            return;
        }
        logger.debug("requestJEItems 日记账表条目：" + JSON.toJSONString(items));
        for (JournalEntryQueryDto a : items) {
            try {
                processJE(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateJE 日记账表,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdateTB(List<TrialBalanceQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateTB 科目余额表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS TrialBalance 科目余额表 response, skip processing");
            return;
        }
        logger.debug("requestTBItems 科目余额表条目：" + JSON.toJSONString(items));
        for (TrialBalanceQueryDto a : items) {
            try {
                processTB(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateTB 科目余额表,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdateCF(List<CashFlowQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateCF 现金流量表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS CashFlow 现金流量表 response, skip processing");
            return;
        }
        logger.debug("requestCFItems 现金流量表条目：" + JSON.toJSONString(items));
        for (CashFlowQueryDto a : items) {
            try {
                processCF(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateCF 现金流量表,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdateBS(List<BalanceSheetQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateBS 资产负债表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS BalanceSheet 资产负债表 response, skip processing");
            return;
        }
        logger.debug("requestBSItems 资产负债表条目：" + JSON.toJSONString(items));
        for (BalanceSheetQueryDto a : items) {
            try {
                processBS(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateBS 资产负债表,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdateBSprc(List<BalanceSheetPrcQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateBSprc 资产负债表PRC");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS BSprc 资产负债表PRC response, skip processing");
            return;
        }
        logger.debug("requestBSprcItems 资产负债表PRC条目：" + JSON.toJSONString(items));
        for (BalanceSheetPrcQueryDto a : items) {
            try {
                processBSprc(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateBSprc 资产负债表PRC,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdatePL(List<ProfitLossStatementQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdatePL 利润表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS ProfitLossStatement 利润表 response, skip processing");
            return;
        }
        logger.debug("requestPLItems 利润表条目：" + JSON.toJSONString(items));
        for (ProfitLossStatementQueryDto a : items) {
            try {
                processPL(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdatePL 利润表,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdatePLprc(List<ProfitLossStatementPrcQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdatePLprc 利润表PRC");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS ProfitLossStatement 利润表PRC response, skip processing");
            return;
        }
        logger.debug("requestPLItems 利润表PRC条目：" + JSON.toJSONString(items));
        for (ProfitLossStatementPrcQueryDto a : items) {
            try {
                processPLprc(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdatePLprc 利润表PRC,took [{}] ms", System.currentTimeMillis() - start);
    }

    @Override
    public void queryRemoteServerThenUpdateOAR(List<OrganizationAccountingRateQueryDto> items) {
        long start = System.currentTimeMillis();
        logger.debug("start queryRemoteServerThenUpdateOAR 汇率表");
        //判断数据是否存在
        if (items.size() == 0) {
            logger.error("empty EBS OrganizationAccountingRate 汇率表 response, skip processing");
            return;
        }
        logger.debug("requestPLItems 汇率表：" + JSON.toJSONString(items));
        for (OrganizationAccountingRateQueryDto a : items) {
            try {
                processOAR(a);
            } catch (Exception e) {
                logger.error("break loop as catch:" + e, e);
            }
        }
        logger.debug("end queryRemoteServerThenUpdateOAR 汇率表,took [{}] ms", System.currentTimeMillis() - start);
    }

    private void processJE(JournalEntryQueryDto item) {
        JournalEntryExample journalEntryExample = new JournalEntryExample();
        journalEntryExample.createCriteria().andHeaderIdEqualTo(item.getHeaderId()).andLineNumEqualTo(item.getLineNum());
        List<JournalEntry> journalEntryList = journalEntryMapper.selectByExample(journalEntryExample);
        //唯一则更新否则插入
        JournalEntry journalEntry = new JournalEntry();
        if (journalEntryList.size() == 1) {
            logger.info("exit and update journalEntry headerId:{},lineNum:{}", item.getHeaderId(), item.getLineNum());
            journalEntry = journalEntryList.get(0);
            populateFieldsJE(item, journalEntry);
            journalEntry.setId(journalEntryList.get(0).getId());
            journalEntryMapper.updateByPrimaryKey(journalEntry);
        } else {
            logger.info("miss and insert journalEntry headerId:{},lineNum:{}", item.getHeaderId(), item.getLineNum());
            populateFieldsJE(item, journalEntry);
            journalEntry.setId(distributedIdService.nextId());
            journalEntry.setCreatedBy("");
            journalEntry.setCreatedDate(new Date());
            journalEntry.setCreateTime(new Date());
            journalEntryMapper.insertSelective(journalEntry);
        }
    }

    private void processTB(TrialBalanceQueryDto item) {
        TrialBalanceExample example = new TrialBalanceExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<TrialBalance> itemList = trialBalanceMapper.selectByExample(example);
        //唯一则更新否则插入
        TrialBalance result = new TrialBalance();
        if (itemList.size() == 1) {
            logger.info("exit and update TrialBalance OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsTB(item, result);
            result.setId(itemList.get(0).getId());
            trialBalanceMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update TrialBalance OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsTB(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateBy("");
            result.setCreateTime(new Date());
            result.setCreateTime(new Date());
            trialBalanceMapper.insertSelective(result);
        }
    }

    private void processCF(CashFlowQueryDto item) {
        CashFlowExample example = new CashFlowExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<CashFlow> itemList = cashFlowMapper.selectByExample(example);
        //唯一则更新否则插入
        CashFlow result = new CashFlow();
        if (itemList.size() == 1) {
            logger.info("exit and update CashFlow OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsCF(item, result);
            result.setId(itemList.get(0).getId());
            cashFlowMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update CashFlow OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsCF(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateTime(new Date());
            result.setCreateBy("");
            cashFlowMapper.insertSelective(result);
        }
    }

    private void processBS(BalanceSheetQueryDto item) {
        BalanceSheetExample example = new BalanceSheetExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<BalanceSheet> itemList = balanceSheetMapper.selectByExample(example);
        //唯一则更新否则插入
        BalanceSheet result = new BalanceSheet();
        if (itemList.size() == 1) {
            logger.info("exit and update BalanceSheet OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsBS(item, result);
            result.setId(itemList.get(0).getId());
            balanceSheetMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update BalanceSheet OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsBS(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateBy("");
            result.setCreateTime(new Date());
            balanceSheetMapper.insertSelective(result);
        }
    }

    private void processBSprc(BalanceSheetPrcQueryDto item) {
        BalanceSheetExample example = new BalanceSheetExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<BalanceSheet> itemList = balanceSheetMapper.selectByExample(example);
        //唯一则更新否则插入
        BalanceSheet result = new BalanceSheet();
        if (itemList.size() == 1) {
            logger.info("exit and update BalanceSheetPrc OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsBSprc(item, result);
            result.setId(itemList.get(0).getId());
            balanceSheetMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update BalanceSheetPrc OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsBSprc(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateBy("");
            result.setCreateTime(new Date());
            balanceSheetMapper.insertSelective(result);
        }
    }

    private void processPL(ProfitLossStatementQueryDto item) {
        ProfitLossStatementExample example = new ProfitLossStatementExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<ProfitLossStatement> itemList = profitLossStatementMapper.selectByExample(example);
        //唯一则更新否则插入
        ProfitLossStatement result = new ProfitLossStatement();
        if (itemList.size() == 1) {
            logger.info("exit and update ProfitLossStatement OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsPLS(item, result);
            result.setId(itemList.get(0).getId());
            profitLossStatementMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update ProfitLossStatement OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsPLS(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateTime(new Date());
            result.setCreateBy("");
            profitLossStatementMapper.insertSelective(result);
        }
    }

    private void processPLprc(ProfitLossStatementPrcQueryDto item) {
        ProfitLossStatementExample example = new ProfitLossStatementExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<ProfitLossStatement> itemList = profitLossStatementMapper.selectByExample(example);
        //唯一则更新否则插入
        ProfitLossStatement result = new ProfitLossStatement();
        if (itemList.size() == 1) {
            logger.info("exit and update ProfitLossStatementPrc OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsPLSprc(item, result);
            result.setId(itemList.get(0).getId());
            profitLossStatementMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update ProfitLossStatementPrc OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsPLSprc(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateTime(new Date());
            result.setCreateBy("");
            profitLossStatementMapper.insertSelective(result);
        }
    }

    private void processOAR(OrganizationAccountingRateQueryDto item) {
        OrganizationAccountingRateExample example = new OrganizationAccountingRateExample();
        example.createCriteria().andOrganizationIdEqualTo(item.getOrganizationId()).andPeriodEqualTo(item.getPeriod());
        List<OrganizationAccountingRate> itemList = organizationAccountingRateMapper.selectByExample(example);
        //唯一则更新否则插入
        OrganizationAccountingRate result = new OrganizationAccountingRate();
        if (itemList.size() == 1) {
            logger.info("exit and update OrganizationAccountingRate OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            result = itemList.get(0);
            populateFieldsOAR(item, result);
            result.setId(itemList.get(0).getId());
            organizationAccountingRateMapper.updateByPrimaryKey(result);
        } else {
            logger.info("exit and update OrganizationAccountingRate OrganizationId:{},Period:{}", item.getOrganizationId(), item.getPeriod());
            populateFieldsOAR(item, result);
            result.setId(distributedIdService.nextId());
            result.setCreateTime(new Date());
            result.setCreateBy("");
            organizationAccountingRateMapper.insertSelective(result);
        }
    }


    private void populateFieldsJE(JournalEntryQueryDto item, JournalEntry result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setCurrencyCode(trimLimit(item.getCurrencyCode(), 20));
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setHeaderId(trimLimit(item.getHeaderId(), 128));
        result.setLineNum(trimLimit(item.getLineNum(), 300));
        result.setApprovalStatus(trimLimit(item.getApprovalStatus(), 20));
        result.setPostedStatus(trimLimit(item.getPostedStatus(), 20));
        result.setPeriod(StringUtils.isBlank(item.getPeriod().toString()) ? 0 : item.getPeriod());
        result.setAccountingDate(item.getAccountingDate());
        result.setJournalSource(trimLimit(item.getJournalSource(), 20));
        result.setCategory(trimLimit(item.getCategory(), 50));
        result.setName(trimLimit(item.getName(), 300));
        result.setVoucherNum(trimLimit(item.getVoucherNum(), 128));
        result.setDescription(trimLimit(item.getDescription(), 500));
        result.setSegment1(trimLimit(item.getSegment1(), 300));
        result.setSegment2(trimLimit(item.getSegment2(), 300));
        result.setSegment3(trimLimit(item.getSegment3(), 300));
        result.setSegment4(trimLimit(item.getSegment4(), 300));
        result.setSegment5(trimLimit(item.getSegment5(), 300));
        result.setSegment6(trimLimit(item.getSegment6(), 300));
        result.setSegment7(trimLimit(item.getSegment7(), 300));
        result.setSegment8(trimLimit(item.getSegment8(), 300));
        result.setSegment9(trimLimit(item.getSegment9(), 300));
        result.setSegment10(trimLimit(item.getSegment10(), 300));
        result.setSegment1Name(trimLimit(item.getSegment1Name(), 300));
        result.setSegment2Name(trimLimit(item.getSegment2Name(), 300));
        result.setSegment3Name(trimLimit(item.getSegment3Name(), 300));
        result.setSegment4Name(trimLimit(item.getSegment4Name(), 300));
        result.setSegment5Name(trimLimit(item.getSegment5Name(), 300));
        result.setSegment6Name(trimLimit(item.getSegment6Name(), 300));
        result.setSegment7Name(trimLimit(item.getSegment7Name(), 300));
        result.setSegment8Name(trimLimit(item.getSegment8Name(), 300));
        result.setSegment9Name(trimLimit(item.getSegment9Name(), 300));
        result.setSegment10Name(trimLimit(item.getSegment10Name(), 300));
        result.setJournalCurrencyCode(trimLimit(item.getJournalCurrencyCode(), 20));
        result.setSobCurrencyCode(trimLimit(item.getSobCurrencyCode(), 20));
        result.setAccountedDr(Optional.ofNullable(item.getAccountedDr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setAccountedCr(Optional.ofNullable(item.getAccountedCr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setEnteredDr(Optional.ofNullable(item.getEnteredDr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setEnteredCr(Optional.ofNullable(item.getEnteredCr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setCfItem(trimLimit(item.getCfItem(), 50));
        result.setAttribute1(trimLimit(item.getAttribute1(), 300));
        result.setAttribute2(item.getAttribute2());
        result.setAttribute3(trimLimit(item.getAttribute3(), 300));
        result.setAttribute4(trimLimit(item.getAttribute4(), 300));
        result.setAttribute5(trimLimit(item.getAttribute5(), 300));
        result.setAttribute6(trimLimit(item.getAttribute6(), 300));
        result.setAttribute7(trimLimit(item.getAttribute7(), 300));
        result.setAttribute8(trimLimit(item.getAttribute8(), 300));
        result.setAttribute9(trimLimit(item.getAttribute9(), 300));
        result.setAttribute10(trimLimit(item.getAttribute10(), 300));
        result.setAttribute11(trimLimit(item.getAttribute11(), 300));
        result.setAttribute12(trimLimit(item.getAttribute12(), 300));
        result.setAttribute13(trimLimit(item.getAttribute13(), 300));
        result.setAttribute14(trimLimit(item.getAttribute14(), 300));
        result.setAttribute15(trimLimit(item.getAttribute15(), 300));
        result.setAttribute16(trimLimit(item.getAttribute16(), 300));
        result.setLateUpdatedBy("");
        result.setLateUpdatedDate(new Date());
        result.setUpdateTime(new Date());
    }

    private void populateFieldsTB(TrialBalanceQueryDto item, TrialBalance result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setCurrencyCode(trimLimit(item.getCurrencyCode(), 20));
        result.setStatus(trimLimit(item.getStatus(), 20));
        result.setCategory(trimLimit(item.getCategory(), 10));
        result.setAccountCategory(trimLimit(item.getAccountCategory(), 20));
        result.setAcctCode1(trimLimit(item.getAcctCode1(), 300));
        result.setAcctName1(trimLimit(item.getAcctName1(), 300));
        result.setAcctName2(trimLimit(item.getAcctName2(), 300));
        result.setAcctName3(trimLimit(item.getAcctName3(), 300));
        result.setSegment1(trimLimit(item.getSegment1(), 300));
        result.setSegment2(trimLimit(item.getSegment2(), 300));
        result.setSegment3(trimLimit(item.getSegment3(), 300));
        result.setSegment4(trimLimit(item.getSegment4(), 300));
        result.setSegment5(trimLimit(item.getSegment5(), 300));
        result.setSegment6(trimLimit(item.getSegment6(), 300));
        result.setSegment7(trimLimit(item.getSegment7(), 300));
        result.setSegment8(trimLimit(item.getSegment8(), 300));
        result.setSegment9(trimLimit(item.getSegment9(), 300));
        result.setSegment10(trimLimit(item.getSegment10(), 300));
        result.setSegment1Name(trimLimit(item.getSegment1Name(), 300));
        result.setSegment2Name(trimLimit(item.getSegment2Name(), 300));
        result.setSegment3Name(trimLimit(item.getSegment3Name(), 300));
        result.setSegment4Name(trimLimit(item.getSegment4Name(), 300));
        result.setSegment5Name(trimLimit(item.getSegment5Name(), 300));
        result.setSegment6Name(trimLimit(item.getSegment6Name(), 300));
        result.setSegment7Name(trimLimit(item.getSegment7Name(), 300));
        result.setSegment8Name(trimLimit(item.getSegment8Name(), 300));
        result.setSegment9Name(trimLimit(item.getSegment9Name(), 300));
        result.setSegment10Name(trimLimit(item.getSegment10Name(), 300));
        result.setBegBal(Optional.ofNullable(item.getBegBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPeriodDr(Optional.ofNullable(item.getPeriodDr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPeriodCr(Optional.ofNullable(item.getPeriodCr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setEndBal(Optional.ofNullable(item.getEndBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setQtdDr(Optional.ofNullable(item.getQtdDr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setQtdCr(Optional.ofNullable(item.getQtdCr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdDr(Optional.ofNullable(item.getYtdDr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdCr(Optional.ofNullable(item.getYtdCr())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setBegBalBeq(Optional.ofNullable(item.getBegBalBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPeriodDrBeq(Optional.ofNullable(item.getPeriodDrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPeriodCrBeq(Optional.ofNullable(item.getPeriodCrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setEndBalBeq(Optional.ofNullable(item.getEndBalBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setQtdDrBeq(Optional.ofNullable(item.getQtdDrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setQtdCrBeq(Optional.ofNullable(item.getQtdCrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdDrBeq(Optional.ofNullable(item.getYtdDrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdCrBeq(Optional.ofNullable(item.getYtdCrBeq())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setUpdateBy("");
        result.setUpdateTime(new Date());
    }

    private void populateFieldsCF(CashFlowQueryDto item, CashFlow result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 128));
        result.setTmsPeriod(item.getTmsPeriod());
        result.setPeriod(item.getPeriod());
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setLedgerCurrencyCode(trimLimit(item.getLedgerCurrencyCode(), 128));
        result.setEntityCode(trimLimit(item.getEntityCode(), 128));
        result.setEntityName(trimLimit(item.getEntityName(), 300));
        result.setCategory(trimLimit(item.getCategory(), 300));
        result.setFrequency(trimLimit(item.getFrequency(), 300));
        result.setItemName(trimLimit(item.getItemName(), 300));
        result.setItemName2(trimLimit(item.getItemName2(), 300));
        result.setPeriodAmt(Optional.ofNullable(item.getPeriodAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdAmt(Optional.ofNullable(item.getYtdAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setUpdateBy("");
        result.setUpdateTime(new Date());
    }

    private void populateFieldsBS(BalanceSheetQueryDto item, BalanceSheet result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setTmsPeriod(item.getTmsPeriod());
        result.setPeriod(item.getPeriod());
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setLedgerCurrencyCode(trimLimit(item.getLedgerCurrencyCode(), 20));
        result.setEntityCode(trimLimit(item.getEntityCode(), 300));
        result.setEntityName(trimLimit(item.getEntityName(), 300));
        result.setCategory(trimLimit(item.getCategory(), 300));
        result.setFrequency(trimLimit(item.getFrequency(), 300));
        result.setItemName(trimLimit(item.getItemName(), 300));
        result.setEndBal(Optional.ofNullable(item.getEndBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setBegBal(Optional.ofNullable(item.getBegBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPrcFlag(item.getPrcFlag());
        result.setUpdateBy("");
        result.setUpdateTime(new Date());
    }

    private void populateFieldsBSprc(BalanceSheetPrcQueryDto item, BalanceSheet result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setTmsPeriod(item.getTmsPeriod());
        result.setPeriod(item.getPeriod());
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setLedgerCurrencyCode(trimLimit(item.getLedgerCurrencyCode(), 20));
        result.setEntityCode(trimLimit(item.getEntityCode(), 300));
        result.setEntityName(trimLimit(item.getEntityName(), 300));
        result.setCategory(trimLimit(item.getCategory(), 300));
        result.setFrequency(trimLimit(item.getFrequency(), 300));
        result.setItemName(trimLimit(item.getItemName(), 300));
        result.setEndBal(Optional.ofNullable(item.getEndBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setBegBal(Optional.ofNullable(item.getBegBal())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPrcFlag(item.getPrcFlag());
        result.setUpdateBy("");
        result.setUpdateTime(new Date());
    }

    private void populateFieldsPLS(ProfitLossStatementQueryDto item, ProfitLossStatement result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setTmsPeriod(item.getTmsPeriod());
        result.setPeriod(item.getPeriod());
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setLedgerCurrencyCode(trimLimit(item.getLedgerCurrencyCode(), 20));
        result.setEntityCode(trimLimit(item.getEntityCode(), 300));
        result.setEntityName(trimLimit(item.getEntityName(), 300));
        result.setCategory(trimLimit(item.getCategory(), 300));
        result.setFrequency(trimLimit(item.getFrequency(), 300));
        result.setItemName(trimLimit(item.getItemName(), 300));
        result.setPeriodAmt(Optional.ofNullable(item.getPeriodAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdAmt(Optional.ofNullable(item.getYtdAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPrcFlag(item.getPrcFlag());
        result.setUpdateTime(new Date());
        result.setUpdateBy("");
    }

    private void populateFieldsPLSprc(ProfitLossStatementPrcQueryDto item, ProfitLossStatement result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setProjectId(trimLimit(item.getProjectId(), 128));
        result.setDate(item.getDate());
        result.setSource(trimLimit(item.getSource(), 20));
        result.setTmsPeriod(item.getTmsPeriod());
        result.setPeriod(item.getPeriod());
        result.setStatus(trimLimit(item.getStatus(), 10));
        result.setLedgerId(trimLimit(item.getLedgerId(), 128));
        result.setLedgerName(trimLimit(item.getLedgerName(), 300));
        result.setLedgerCurrencyCode(trimLimit(item.getLedgerCurrencyCode(), 20));
        result.setEntityCode(trimLimit(item.getEntityCode(), 300));
        result.setEntityName(trimLimit(item.getEntityName(), 300));
        result.setCategory(trimLimit(item.getCategory(), 300));
        result.setFrequency(trimLimit(item.getFrequency(), 300));
        result.setItemName(trimLimit(item.getItemName(), 300));
        result.setPeriodAmt(Optional.ofNullable(item.getPeriodAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setYtdAmt(Optional.ofNullable(item.getYtdAmt())
                .map(x -> x.setScale(4, RoundingMode.HALF_UP))
                .orElse(BigDecimal.ZERO));
        result.setPrcFlag(item.getPrcFlag());
        result.setUpdateTime(new Date());
        result.setUpdateBy("");
    }

    private void populateFieldsOAR(OrganizationAccountingRateQueryDto item, OrganizationAccountingRate result) {
        beanUtil.copyProperties(item, result);
        result.setOrganizationId(trimLimit(item.getOrganizationId(), 128));
        result.setPeriod(item.getPeriod());
        result.setConvertionType(trimLimit(item.getConvertionType(), 100));
        result.setCurrencyFrom(trimLimit(item.getCurrencyFrom(), 10));
        result.setCurrencyTo(trimLimit(item.getCurrencyTo(), 10));
        result.setRate(item.getRate());
        result.setUpdateBy("");
        result.setUpdateTime(new Date());
    }

    /**
     * Trim字符串，并限定字符串的长度. 如果是输入值是空指针，会返回空字符串
     */
    public String trimLimit(String str, int limit) {
        if (Strings.isBlank(str)) {
            return "";
        }
        String tmp = str.trim();
        String result = cutString(tmp, limit);
        if (result == null) {
            result = "";
        }
        return result;
    }

    /**
     * 限定字符串的长度.
     */
    public String cutString(String s, int length) {
        if (Lang.isEmpty(length) || Lang.isEmpty(s)) {
            return "";
        } else if (s.length() <= length) {
            return s;
        } else {
            return s.substring(0, length);
        }
    }
}
