package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.constant.enums.StdAccountEnum;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dto.BarAndLineDto;
import pwc.taxtech.atms.dto.vatdto.BurdenRateDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.invoice.InputInvoiceMapper;
import pwc.taxtech.atms.vat.dao.PeriodCellDataMapper;
import pwc.taxtech.atms.vat.dao.PeriodCellTemplateMapper;
import pwc.taxtech.atms.vat.dao.PeriodDataSourceMapper;
import pwc.taxtech.atms.vat.dao.PeriodTemplateMapper;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Service
public class ReportAnalysisService extends BaseService {
    @Resource
    private GlBalanceDao glBalanceDao;
    @Resource
    private AccountMappingMapper accountMappingMapper;
    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private OrganizationMapper organizationMapper;
    @Resource
    private StandardAccountDao standardAccountDao;
    @Resource
    private PeriodTemplateMapper periodTemplateMapper;
    @Resource
    private PeriodCellTemplateMapper periodCellTemplateMapper;
    @Resource
    private PeriodCellDataMapper periodCellDataMapper;
    @Resource
    private PeriodDataSourceMapper periodDataSourceMapper;
    @Resource
    private InputInvoiceMapper inputInvoiceMapper;

    /**
     * 增值税负担率
     */
    public BurdenRateDto getBurdenRateData(String projectId) {
        BurdenRateDto dto = new BurdenRateDto();
        List<BigDecimal> vatAmountList = Lists.newArrayList();
        List<BigDecimal> incomeList = Lists.newArrayList();
        List<BigDecimal> rateList = Lists.newArrayList();
        for (int p = 1; p <= 12; p++) {
            BigDecimal vat = getCellValue(projectId, p, "VAT001", 32, 18);
            BigDecimal income = getTbValueByDirection("600101", projectId, p)
                    .add(getTbValueByDirection("600102", projectId, p))
                    .add(getTbValueByDirection("600103", projectId, p))
                    .add(getTbValueByDirection("600104", projectId, p))
                    .add(getTbValueByDirection("600105", projectId, p))
                    .add(getTbValueByDirection("600107", projectId, p))
                    .add(getTbValueByDirection("600111", projectId, p))
                    .add(getTbValueByDirection("600190", projectId, p));
            vatAmountList.add(vat);
            incomeList.add(income);
            BigDecimal rate = income.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vat.divide(income, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            rateList.add(rate);
        }
        dto.setVatAmount(vatAmountList);
        dto.setIncome(incomeList);
        dto.setRate(rateList);
        return dto;
    }

    /**
     * 增值税申报表收入与会计收入差异离散度分析
     */
    public BurdenRateDto getDispersion(String projectId) {
        BurdenRateDto dto = new BurdenRateDto();
        List<BigDecimal> rateList = Lists.newArrayList();
        for (int p = 1; p <= 12; p++) {
            BigDecimal val2 = getTbValueByDirection("600101", projectId, p)
                    .add(getTbValueByDirection("600102", projectId, p)).add(getTbValueByDirection("600103", projectId, p))
                    .add(getTbValueByDirection("600104", projectId, p)).add(getTbValueByDirection("600105", projectId, p))
                    .add(getTbValueByDirection("600107", projectId, p)).add(getTbValueByDirection("600111", projectId, p))
                    .add(getTbValueByDirection("600190", projectId, p)).add(getTbValueByDirection("605101", projectId, p))
                    .add(getTbValueByDirection("612101", projectId, p)).add(getTbValueByDirection("630101", projectId, p))
                    .add(getTbValueByDirection("630102", projectId, p)).add(getTbValueByDirection("630103", projectId, p))
                    .add(getTbValueByDirection("630104", projectId, p)).add(getTbValueByDirection("630105", projectId, p))
                    .add(getTbValueByDirection("630106", projectId, p)).add(getTbValueByDirection("630107", projectId, p))
                    .add(getTbValueByDirection("630108", projectId, p)).add(getTbValueByDirection("630198", projectId, p));
            BigDecimal val = getCellValue(projectId, p, "VAT001", 9, 18) //栏次1
                    .add(getCellValue(projectId, p, "VAT001", 13, 18)) //栏次5
                    .add(getCellValue(projectId, p, "VAT001", 15, 18)) //栏次7
                    .subtract(val2);
            BigDecimal rate = val2.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO : val.divide(val2, 2, BigDecimal.ROUND_HALF_UP);
            rateList.add(rate);
        }
        dto.setRate(rateList);
        return dto;
    }

    /**
     * 进项税额占主营业务成本的比例
     */
    public BurdenRateDto getIncomeRate(String projectId) {
        BurdenRateDto dto = new BurdenRateDto();
        List<BigDecimal> vatAmountList = Lists.newArrayList();
        List<BigDecimal> incomeList = Lists.newArrayList();
        List<BigDecimal> rateList = Lists.newArrayList();
        for (int p = 1; p <= 12; p++) {
            BigDecimal vat = getCellValue(projectId, p, "VAT001", 20, 18);
            BigDecimal income = getTbValueByDirection("640101", projectId, p)
                    .add(getTbValueByDirection("640102", projectId, p))
                    .add(getTbValueByDirection("640103", projectId, p))
                    .add(getTbValueByDirection("640104", projectId, p))
                    .add(getTbValueByDirection("640105", projectId, p))
                    .add(getTbValueByDirection("640106", projectId, p))
                    .add(getTbValueByDirection("640107", projectId, p))
                    .add(getTbValueByDirection("640190", projectId, p))
                    .add(getTbValueByDirection("640201", projectId, p));
            vatAmountList.add(vat);
            incomeList.add(income);
            BigDecimal rate = income.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vat.divide(income, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            rateList.add(rate);
        }
        dto.setVatAmount(vatAmountList);
        dto.setIncome(incomeList);
        dto.setRate(rateList);
        return dto;
    }

    /**
     * 进项税转出额波动率
     */
    public BurdenRateDto getIncomeVolatility(String projectId) {
        BurdenRateDto dto = new BurdenRateDto();
        List<BigDecimal> vatAmountList = Lists.newArrayList();
        List<BigDecimal> incomeList = Lists.newArrayList();
        List<BigDecimal> rateList = Lists.newArrayList();
        for (int p = 1; p <= 12; p++) {
            BigDecimal vat = getCellValue(projectId, p, "VAT001", 22, 18);
            BigDecimal income = getCellValue(projectId, p, "VAT001", 20, 18);
            vatAmountList.add(vat);
            incomeList.add(income);
            BigDecimal rate = income.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vat.divide(income, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            rateList.add(rate);
        }
        dto.setVatAmount(vatAmountList);
        dto.setIncome(incomeList);
        dto.setRate(rateList);
        return dto;
    }

    /**
     * 增值税应税收入、增值税免、抵、退办法出口收入 占会计收入比例
     */
    @Deprecated
    public BurdenRateDto getVatIncomeRate(String projectId, Integer period) {
        BurdenRateDto dto = new BurdenRateDto();
        BigDecimal val = getCellValue(projectId, period, "VAT001", 19, 18);
        BigDecimal val2 = getCellValue(projectId, period, "VAT005", 26, 12);
        BigDecimal total = getTbValueByDirection("600101", projectId, period)
                .add(getTbValueByDirection("600102", projectId, period))
                .add(getTbValueByDirection("600103", projectId, period))
                .add(getTbValueByDirection("600104", projectId, period))
                .add(getTbValueByDirection("600105", projectId, period))
                .add(getTbValueByDirection("600107", projectId, period))
                .add(getTbValueByDirection("600111", projectId, period))
                .add(getTbValueByDirection("600190", projectId, period))
                .add(getTbValueByDirection("605101", projectId, period))
                .add(getTbValueByDirection("612101", projectId, period))
                .add(getTbValueByDirection("630101", projectId, period))
                .add(getTbValueByDirection("630102", projectId, period))
                .add(getTbValueByDirection("630103", projectId, period))
                .add(getTbValueByDirection("630104", projectId, period))
                .add(getTbValueByDirection("630105", projectId, period))
                .add(getTbValueByDirection("630106", projectId, period))
                .add(getTbValueByDirection("630107", projectId, period))
                .add(getTbValueByDirection("630108", projectId, period))
                .add(getTbValueByDirection("630198", projectId, period));
        dto.setVatAmount(Lists.newArrayList(val));
        dto.setIncome(Lists.newArrayList(val2));
        dto.setRate(Lists.newArrayList(total.subtract(val).subtract(val2)));
        return dto;
    }

    /**
     * 增值税应税收入、增值税免、抵、退办法出口收入 占会计收入比例 趋势
     */
    public BarAndLineDto getVatIncomeLine(String projectId) {
        BarAndLineDto dto = new BarAndLineDto();
        List<BigDecimal> val1List = Lists.newArrayList();
        List<BigDecimal> val2List = Lists.newArrayList();
        List<BigDecimal> val3List = Lists.newArrayList();
        List<BigDecimal> rate1List = Lists.newArrayList();
        List<BigDecimal> rate2List = Lists.newArrayList();
        for (int p = 1; p <= 12; p++) {
            BigDecimal val = getCellValue(projectId, p, "VAT001", 9, 18) //栏次1
                    .add(getCellValue(projectId, p, "VAT001", 13, 18)); //栏次5
            BigDecimal val2 = getCellValue(projectId, p, "VAT001", 15, 18); //栏次7
            BigDecimal sum = getTbValueByDirection("600101", projectId, p)
                    .add(getTbValueByDirection("600102", projectId, p))
                    .add(getTbValueByDirection("600103", projectId, p))
                    .add(getTbValueByDirection("600104", projectId, p))
                    .add(getTbValueByDirection("600105", projectId, p))
                    .add(getTbValueByDirection("600107", projectId, p))
                    .add(getTbValueByDirection("600111", projectId, p))
                    .add(getTbValueByDirection("600190", projectId, p))
                    .add(getTbValueByDirection("605101", projectId, p))
                    .add(getTbValueByDirection("612101", projectId, p))
                    .add(getTbValueByDirection("630101", projectId, p))
                    .add(getTbValueByDirection("630102", projectId, p))
                    .add(getTbValueByDirection("630103", projectId, p))
                    .add(getTbValueByDirection("630104", projectId, p))
                    .add(getTbValueByDirection("630105", projectId, p))
                    .add(getTbValueByDirection("630106", projectId, p))
                    .add(getTbValueByDirection("630107", projectId, p))
                    .add(getTbValueByDirection("630108", projectId, p))
                    .add(getTbValueByDirection("630198", projectId, p));

            BigDecimal rate = sum.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    val.divide(sum, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            BigDecimal rate2 = sum.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    val2.divide(sum, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            val1List.add(val);
            val2List.add(val2);
            val3List.add(sum);
            rate1List.add(rate);
            rate2List.add(rate2);
        }
        dto.setVal1(val1List);
        dto.setVal2(val2List);
        dto.setVal3(val3List);
        dto.setRate1(rate1List);
        dto.setRate2(rate2List);
        return dto;
    }

    /**
     * 已抵扣进项税波动率
     */
    public BurdenRateDto getDeduction(String projectId) {
        BurdenRateDto dto = new BurdenRateDto();
        List<BigDecimal> rateList = Lists.newArrayList();
        Project project = projectMapper.selectByPrimaryKey(projectId);
        Organization org = organizationMapper.selectByPrimaryKey(project.getOrganizationId());
        for (int p = 1; p <= 12; p++) {
            BigDecimal val = getCellValue(projectId, p, "VAT001", 19, 18);
            InputInvoiceExample example = new InputInvoiceExample();
            example.createCriteria().andGFSHEqualTo(org.getTaxPayerNumber()).andFPZTIn(Lists.newArrayList("0", "2"))
                    .andLRRQBetween(DateUtils.getPeriodBeginFormat(project.getYear(), p), DateUtils.getPeriodEndFormat(project.getYear(), p));
            BigDecimal sum = inputInvoiceMapper.selectByExample(example).stream().map(o ->
                    NumberUtils.isParsable(o.getHJSE()) ? NumberUtils.createBigDecimal(o.getHJSE()) : BigDecimal.ZERO)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal rate = sum.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    val.divide(sum, 2, BigDecimal.ROUND_HALF_UP).setScale(2, BigDecimal.ROUND_HALF_UP);
            rateList.add(rate);
        }
        dto.setRate(rateList);
        return dto;
    }

    /**
     * 根据借贷方向获取TB表科目的发生额
     */
    public BigDecimal getTbValueByDirection(String code, String projectId, Integer period) {
        BigDecimal result = BigDecimal.ZERO;
        try {
            if (StringUtils.isBlank(projectId)) {
                return result;
            }
            Project project = projectMapper.selectByPrimaryKey(projectId);
            if (null == project) {
                return result;
            }
            AccountMappingExample example = new AccountMappingExample();
            example.createCriteria().andOrganizationIdEqualTo(project.getOrganizationId())
                    .andEnterpriseAccountSetIdEqualTo(project.getEnterpriseAccountSetId())
                    .andIndustryIdEqualTo(project.getIndustryId()).andStandardAccountCodeEqualTo(code);
            List<AccountMapping> mappingList = accountMappingMapper.selectByExample(example).stream()
                    .filter(distinctByKey(AccountMapping::getEnterpriseAccountCode)).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(mappingList)) {
                return result;
            }
            for (AccountMapping mapping : mappingList) {
                Optional<StandardAccount> accountOptional = standardAccountDao.getByCodeAndIndustryId(
                        mapping.getStandardAccountCode(), mapping.getIndustryId()).stream().findFirst();
                Optional<GlBalance> optional = glBalanceDao.get(code, project.getClientCode(), project.getYear() + "-" +
                        (period < 10 ? "0" + period : period));
                BigDecimal val;
                if (optional.isPresent()) {
                    if (accountOptional.get().getDirection() == StdAccountEnum.Direction.Debit.getCode()) {
                        val = optional.get().getPtdDr();
                    } else {
                        val = optional.get().getPtdCr();
                    }
                    result = result.add(val);
                }
            }

        } catch (Exception e) {
            logger.error("getTbValueByDirection error.", e);
        }
        return result;
    }

    /**
     * 获取单元格计算的值
     *
     * @param projectId projectId
     * @param period    period
     * @param sheet     VAT001
     * @param rowIndex  rowIndex
     * @param colIndex  colIndex
     * @return BigDecimal
     */
    public BigDecimal getCellValue(String projectId, Integer period, String sheet, Integer rowIndex, Integer colIndex) {
        try {
            PeriodTemplateExample templateExample = new PeriodTemplateExample();
            templateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andCodeEqualTo(sheet);
            Optional<PeriodTemplate> templateOptional = periodTemplateMapper.selectByExample(templateExample).stream().findFirst();
            if (!templateOptional.isPresent()) {
                return BigDecimal.ZERO;
            }
            PeriodCellTemplateExample cellTemplateExample = new PeriodCellTemplateExample();
            cellTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                    .andReportTemplateIdEqualTo(templateOptional.get().getTemplateId()).andRowIndexEqualTo(rowIndex)
                    .andColumnIndexEqualTo(colIndex);
            Optional<PeriodCellTemplate> cellTemplateOptional = periodCellTemplateMapper.selectByExample(cellTemplateExample).stream().findFirst();
            if (!cellTemplateOptional.isPresent()) {
                return BigDecimal.ZERO;
            }
            //cell data数据源
            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
            cellDataExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andCellTemplateIdEqualTo(
                    cellTemplateOptional.get().getCellTemplateId());
            BigDecimal sum = periodCellDataMapper.selectByExample(cellDataExample).stream().map(o -> NumberUtils.createBigDecimal(o.getData()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);

            //manual data数据源
            PeriodDataSourceExample dataSourceExample = new PeriodDataSourceExample();
            dataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andCellTemplateIdEqualTo(
                    cellTemplateOptional.get().getCellTemplateId()).andTypeEqualTo(FormulaDataSourceType.KeyInSource.getCode());
            sum = periodDataSourceMapper.selectByExample(dataSourceExample).stream().map(o -> o.getAmount()).reduce(sum, BigDecimal::add);
            return sum;
        } catch (Exception e) {
            logger.error("getCellValue error", e);
        }
        return BigDecimal.ZERO;
    }

    private static <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }
}
