package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.enums.RevenueConfEnum;
import pwc.taxtech.atms.dpo.OrgSelectDto;
import pwc.taxtech.atms.dto.revenuconf.RevConfAddDto;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfParam;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfResult;
import pwc.taxtech.atms.vat.dao.RevenueConfigMapper;
import pwc.taxtech.atms.vat.entity.RevenueConfig;
import pwc.taxtech.atms.vat.entity.RevenueConfigExample;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class RevenueConfService extends BaseService {

    @Resource
    private RevenueConfigMapper revenueConfigMapper;
    @Resource
    private OrganizationServiceImpl organizationService;

    /**
     * 分页查询可查看的配置信息
     *
     * @param param RevenueConfParam
     * @return PageInfo
     */
    public PageInfo<RevenueConfResult> queryPage(RevenueConfParam param) {
        List<OrgSelectDto> orgDtoList = organizationService.getMyOrgList();
        if (CollectionUtils.isEmpty(orgDtoList)) {
            return new PageInfo<>();
        }
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        RevenueConfigExample example = new RevenueConfigExample();
        example.createCriteria().andOrgIdIn(orgDtoList.stream().map(OrgSelectDto::getId).collect(Collectors.toList()));
        PageInfo<RevenueConfResult> pageInfo = new PageInfo<>(revenueConfigMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new RevenueConfResult())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    /**
     * 新增配置
     *
     * @param config RevenueConfig
     */
    public void addConfig(RevenueConfig config) {
        //todo 重复校验
        config.setId(idService.nextId());
        revenueConfigMapper.insertSelective(parseEntity(config));
    }

    /**
     * 前台批量新增
     *
     * @param addDto RevConfAddDto
     */
    public void addConfig(RevConfAddDto addDto) {
        if (!CollectionUtils.isEmpty(addDto.getOrgList())) {
            addDto.getOrgList().forEach(id -> {
                addDto.setId(idService.nextId());
                addDto.setOrgId(id);
                revenueConfigMapper.insertSelective(parseEntity(addDto));
            });
        }
    }

    /**
     * 更新配置
     *
     * @param config RevenueConfig
     */
    public void updateConfig(RevenueConfig config) {
        //todo 重复校验
        revenueConfigMapper.updateByPrimaryKeySelective(parseEntity(config));
    }

    private RevenueConfig parseEntity(RevenueConfig config) {
        if (config.getAccountType() != RevenueConfEnum.AccountType.Account.getCode().intValue()) {
            config.setTbSegment3(StringUtils.EMPTY);
            config.setTbSegment5(StringUtils.EMPTY);
            config.setTbSegment6(StringUtils.EMPTY);
        }
        if (config.getTaxBase() != RevenueConfEnum.TaxBase.Period_Cr.getCode().intValue()) {
            config.setBaseCrCode(StringUtils.EMPTY);
        }
        if (config.getTaxBase() != RevenueConfEnum.TaxBase.Period_Dr.getCode().intValue()) {
            config.setBaseDrCode(StringUtils.EMPTY);
        }
        return config;
    }

}
