package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import com.grapecity.documents.excel.C;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.TemplateGroupType;
import pwc.taxtech.atms.dpo.TemplateAndGroupDbDto;
import pwc.taxtech.atms.dpo.TemplateUniqDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.vatdto.TemplateByGroupDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.dao.PeriodTemplateMapper;
import pwc.taxtech.atms.vat.entity.PeriodReportExample;
import pwc.taxtech.atms.vat.entity.PeriodTemplateExample;

import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;

@Service
public class TemplateServiceImpl extends AbstractService {

    @Autowired
    private PeriodReportMapper periodReportMapper;

    public void addExistTemplate(TemplateAddExistDto templateAddExistDto){
        TemplateExample t = new TemplateExample();
        t.createCriteria().andTemplateGroupIdEqualTo(templateAddExistDto.getTemplateGroupId());
        List<Template> selfTemplates = templateMapper.selectByExample(t);
        int orderIndex = selfTemplates.size();
        TemplateExample templateExample = new TemplateExample();
        templateExample.createCriteria().andIdIn(templateAddExistDto.getTemplateIdList());
        List<Template> templates = templateMapper.selectByExample(templateExample);
        List<CellTemplateConfig> cellTemplateConfigList = Lists.newArrayList();
        for(Template x : templates){
            // 复制添加template

            Long originTemplateId = x.getId();
            x.setId(distributedIdService.nextId());
            x.setTemplateGroupId(templateAddExistDto.getTemplateGroupId());
            x.setOrderIndex(++orderIndex);
            templateMapper.insertSelective(x);

            // 复制添加cellTemplate
            CellTemplateExample cellTemplateExample = new CellTemplateExample();
            cellTemplateExample.createCriteria().andReportTemplateIdEqualTo(originTemplateId);
            List<CellTemplate> cellTemplates = cellTemplateMapper.selectByExample(cellTemplateExample);

            if(cellTemplates.isEmpty()) return;
            cellTemplates.forEach(cellTemplate -> {
                // 复制添加cellTemplateConfig
                CellTemplateConfigExample cellTemplateConfigExample = new CellTemplateConfigExample();
                cellTemplateConfigExample.createCriteria().andCellTemplateIdEqualTo(cellTemplate.getId())
                        .andReportTemplateIdEqualTo(originTemplateId);
                List<CellTemplateConfig> cellTemplateConfigs = cellTemplateConfigMapper.selectByExample(cellTemplateConfigExample);
                cellTemplate.setId(distributedIdService.nextId());
                cellTemplate.setReportTemplateId(x.getId());
                if(CollectionUtils.isNotEmpty(cellTemplateConfigs)){
                    CellTemplateConfig cellTemplateConfig = cellTemplateConfigs.get(0);
                    cellTemplateConfig.setId(distributedIdService.nextId());
                    cellTemplateConfig.setCellTemplateId(cellTemplate.getId());
                    cellTemplateConfig.setReportTemplateId(cellTemplate.getReportTemplateId());
                    cellTemplateConfigList.add(cellTemplateConfig);
                }
            });
            cellTemplateMapper.batchInsert2(cellTemplates);
        }
        if(cellTemplateConfigList.isEmpty()) return ;
        cellTemplateConfigMapper.batchInsert2(cellTemplateConfigList);
    }

    public List<TemplateDto> get(Long templateGroupId, Integer reportType) {

        TemplateExample example = new TemplateExample();

        TemplateExample.Criteria criteria = example.createCriteria().andTemplateGroupIdEqualTo(templateGroupId);

        if (reportType != null) {
            criteria.andReportTypeEqualTo(reportType.intValue());
        }

        example.setOrderByClause("order_index asc");

        List<Template> templates = templateMapper.selectByExample(example);
        List<TemplateDto> templateDtos = new ArrayList<>();
        for (Template template : templates) {
            TemplateDto dto = new TemplateDto();
            CommonUtils.copyProperties(template, dto);
            dto.setId(template.getId().toString());
            templateDtos.add(dto);
        }
        return templateDtos;
    }

    public String getTemplatePath(Long templateId) {
        String result = "";
        Template template = templateMapper.selectByPrimaryKey(templateId);
        if (template != null) {

            String PREFIX_VALUE = "~";
            if (template.getPath().startsWith(PREFIX_VALUE)) {
                result = template.getPath().substring(1, template.getPath().length());
            } else {
                result = template.getPath();
            }
        }
        return result;
    }

    public Template getTemplateById(Long templateId) {
        return templateMapper.selectByPrimaryKey(templateId);
    }

    public List<TemplateUniqDto> getTemplateUniqList(String serviceTypeId, Integer payTaxType, Integer reportType, String industryIds) {

        Map<String, Object> map = new HashMap<>(4);
        map.put("serviceTypeId", serviceTypeId);
        map.put("payTaxType", payTaxType);
        map.put("reportType", reportType);
        map.put("industryID", industryIds);

        List<TemplateUniqDto> templatedbList = templateMapper.getTemplateUniqDtosByTemplateAndTemplateGroup(map);
        Map<Long, Map<String, List<TemplateUniqDto>>> groupList = templatedbList.stream()
                .filter(sa -> sa.getIsActiveAssociation())
                .collect(groupingBy(TemplateUniqDto::getTemplateGroupId, Collectors.groupingBy(TemplateUniqDto::getTemplateGroupName)));
        List<NameDto> groupListResult = new ArrayList<>();
        groupList.forEach((key, value) -> {
            NameDto temp = new NameDto();
            temp.setId(key);
            value.forEach((key2, value2) -> {
                temp.setName(key2);
            });
            groupListResult.add(temp);
        });

        List<TemplateUniqDto> result = new ArrayList<>();

        for (NameDto group : groupListResult) {
            TemplateUniqDto parent = new TemplateUniqDto();
            parent.setId(group.getId());
            parent.setCode("");
            parent.setName(group.getName());
            result.add(parent);

            List<TemplateUniqDto> subList = templatedbList.stream()
                    .filter(sa -> sa.getTemplateGroupId() == group.getId() && sa.getIsActiveAssociation())
                    .collect(Collectors.toList());

            subList.forEach(sa -> {
                sa.setParentId(sa.getTemplateGroupId());
                sa.setParentName(sa.getTemplateGroupName());
            });
            result.addAll(subList);
        }

        List<TemplateUniqDto> noGroupTemplateList = templatedbList.stream().filter(sa -> !sa.getIsActiveAssociation()).collect(Collectors.toList());

        if (!noGroupTemplateList.isEmpty()) {
            TemplateUniqDto parent = new TemplateUniqDto();
            parent.setId(0L);
            parent.setCode("");
            parent.setName(Constant.Other);

            result.add(parent);

            noGroupTemplateList.forEach(sa -> {
                sa.setParentId(parent.getId());
                sa.setParentName(parent.getName());
            });

            result.addAll(noGroupTemplateList);
        }
        return result;
    }

    public OperationResultDto<Object> updateTemplateName(UpateNameParam param) {
        OperationResultDto<Object> result = new OperationResultDto<>();

        TemplateAndGroupDbDto templateDb = templateMapper.getTemplateAndGroupById(param.getId());

            if (templateDb == null) {
            result.setResult(false);
            result.setResultMsg("TemplateNotExist");
            return result;
        }
        Map<String, Object> map = new HashMap<>(3);
        map.put("templateGroupId", templateDb.getTemplateGroupId());
        map.put("templateId", param.getId());
        map.put("templateName", param.getName());

        long templateDbCount = templateMapper.getCountOfTemplateAndGroupByTemplateName(map);

        if (templateDbCount > 0) {
            result.setResult(false);
            result.setData(templateDb.getTemplateCode() + "_" + param.getName());
            result.setResultMsg("SameNameTemplateRepeat");
            return result;
        }

        Template updateTemplate = templateMapper.selectByPrimaryKey(templateDb.getTemplateId());
        updateTemplate.setName(param.getName());
        updateTemplate.setUpdateTime(new Date());
        templateMapper.updateByPrimaryKey(updateTemplate);

        result.setResult(true);
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public OperationResultDto<String> deleteTemplate(DeleteTemplateParam param) {
        OperationResultDto<String> result = new OperationResultDto<>();
        Template templateDb = templateMapper.selectByPrimaryKey(param.getId());

        String path = "";
        if (templateDb != null) {
            TemplateExample example = new TemplateExample();
            example.createCriteria().andCodeEqualTo(templateDb.getCode()).andIdEqualTo(templateDb.getId()).andIsActiveAssociationEqualTo(false);
            long count = templateMapper.countByExample(example);
            if (count > 0) {
                path = templateDb.getPath();
                //物理删除
                deleteIsAtiveAssociation(templateDb);
            } else {
                if (param.isDeletePermanent() && !templateDb.getIsSystemType()) {
                    path = templateDb.getPath();
                    //物理删除
                    deleteIsAtiveAssociation(templateDb);
                } else {
                    //逻辑删除
                    logicDeleteIsActiveAssociation(templateDb);
                }
            }
        }

        result.setResult(true);
        result.setData(path);
        return result;
    }

    public OperationResultDto setRowColName(Long id, List<CellBriefDto> cellInfo) {
        OperationResultDto result = new OperationResultDto();
        CellTemplateExample example = new CellTemplateExample();
        example.createCriteria().andReportTemplateIdEqualTo(id);
        example.setOrderByClause("ColumnIndex,RowIndex");
        List<CellTemplate> cellTemplates = cellTemplateMapper.selectByExample(example);

        for (CellTemplate cellTemplate : cellTemplates) {
            boolean isUpdate = false;
            Optional<CellBriefDto> rowNameCell = cellInfo.stream().filter(a -> a.getRowIndex() == cellTemplate.getRowIndex()).findFirst();
            if (rowNameCell.isPresent()) {
                cellTemplate.setRowName(rowNameCell.get().getValue());
                isUpdate = true;
            }

            Optional<CellBriefDto> colNameCell = cellInfo.stream().filter(a -> a.getColumnIndex() == cellTemplate.getColumnIndex()).findFirst();
            if (colNameCell.isPresent()) {
                cellTemplate.setColumnName(colNameCell.get().getValue());
                isUpdate = true;
            }

            if (isUpdate) {
                cellTemplateMapper.updateByPrimaryKey(cellTemplate);
            }
        }

        result.setResult(true);
        return result;
    }

    public OperationResultDto<List<TemplateByGroupDto>> getByGroupId(Long templateGroupId, String projectId) {
        OperationResultDto result = new OperationResultDto();
//        Long bspl = getVatTemplateGroup(projectId);
        TemplateExample example = new TemplateExample();
        List<Long> ids = new ArrayList<>();
        ids.add(templateGroupId);
//        if(!bspl.equals(null)) {
//            ids.add(bspl);
//        }
        example.createCriteria().andTemplateGroupIdIn(ids);
        example.setOrderByClause("order_index");
        List<Template> templates = templateMapper.selectByExample(example);
        List<TemplateByGroupDto> wrappList = new ArrayList<>();
//        if (bspl != null) {
//            TemplateByGroupDto bsplGroup = new TemplateByGroupDto();
//            bsplGroup.setId(bspl);
//            bsplGroup.setName(TemplateGroupType.FinancialReturn.name());
//            List<TemplateDto> templateDtos = new ArrayList<>();
//
//            templates.stream().filter(x -> x.getTemplateGroupId().equals(bspl)).collect(Collectors.toList()).forEach(a -> {
//                TemplateDto templateDto = new TemplateDto();
//                CommonUtils.copyProperties(a, templateDto);
//                templateDtos.add(templateDto);
//            });
//            bsplGroup.setChildren(templateDtos);
//            bsplGroup.setOrderIndex(1);
//            wrappList.add(bsplGroup);
//        }

        if (templateGroupId != null) {
            TemplateByGroupDto taxReturnGroup = new TemplateByGroupDto();
            taxReturnGroup.setId(templateGroupId);
            taxReturnGroup.setName(TemplateGroupType.TaxReturn.name());
            List<TemplateDto> templateDtos2 = new ArrayList<>();
            templates.stream().filter(x -> {
                PeriodReportExample periodReportExample = new PeriodReportExample();
                periodReportExample.createCriteria().andTemplateIdEqualTo(x.getId())
                        .andProjectIdEqualTo(projectId);
               return x.getTemplateGroupId().equals(templateGroupId) && periodReportMapper.selectByExample(periodReportExample).size() > 0;
            }).collect(Collectors.toList()).forEach(a -> {
                TemplateDto templateDto = new TemplateDto();
                CommonUtils.copyProperties(a, templateDto);
                templateDtos2.add(templateDto);
            });
            taxReturnGroup.setChildren(templateDtos2);
            taxReturnGroup.setOrderIndex(2);
            wrappList.add(taxReturnGroup);
        }

        if (templateGroupId != null) {
            result.setResult(true);
            result.setData(wrappList);
        } else {
            result.setResult(false);
        }
        return result;
    }

    private Long getVatTemplateGroup(String projectId) {
        ProjectExample example = new ProjectExample();
        example.createCriteria().andIdEqualTo(projectId);
        List<Project> projects = projectMapper.selectByExample(example);
        Optional<String> industryId = projects.stream().map(Project::getIndustryId).findFirst();
        if (industryId == null) {
            return null;
        }
        List<TemplateGroup> allTemplateGroups = templateGroupMapper.selectByExample(new TemplateGroupExample());

        Optional<TemplateGroup> bsplTemplateGroup = allTemplateGroups.stream().filter(a -> a.getIndustryIds().equals(industryId)
                && a.getGroupType().equals(TemplateGroupType.TaxReturn.getCode())).findFirst();

        if (!bsplTemplateGroup.isPresent()) {
            bsplTemplateGroup = allTemplateGroups.stream().filter(a -> a.getIndustryIds().contains(industryId.get())
                    && a.getGroupType().equals(TemplateGroupType.TaxReturn.getCode())).findFirst();
        }

        if (!bsplTemplateGroup.isPresent()) {
            bsplTemplateGroup = industryMapper.getTemplateGroup().stream().findFirst();
        }

        if (!bsplTemplateGroup.isPresent()) {
            return null;
        }
        return bsplTemplateGroup.get().getId();
    }

    private void logicDeleteIsActiveAssociation(Template templateDb) {
        templateDb.setIsActiveAssociation(false);
        templateMapper.updateByPrimaryKeySelective(templateDb);
    }

    private void deleteIsAtiveAssociation(Template templateDb) {
        cellTemplateConfigMapper.deleteCellTemplateConfigByCellTemplate(templateDb.getId());
        keyValueReferenceMapper.deleteKeyValueReferenceByCellTemplate(templateDb.getId());
        CellTemplateExample example = new CellTemplateExample();
        example.createCriteria().andReportTemplateIdEqualTo(templateDb.getId());
        cellTemplateMapper.deleteByExample(example);
    }
}
