package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dao.FormulaAdminMapper;
import pwc.taxtech.atms.dao.ProjectServiceTypeMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dpo.GroupId;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.entity.ProjectServiceType;
import pwc.taxtech.atms.entity.ProjectServiceTypeExample;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.CellDataMapper;
import pwc.taxtech.atms.vat.dao.PeriodCellDataMapper;
import pwc.taxtech.atms.vat.dao.PeriodDataSourceMapper;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.List;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Component
public class FormulaAgent {
    @Autowired
    private FormulaAdminMapper adminMp;
    @Autowired
    private PeriodReportMapper periodReportMapper;
    @Autowired
    private CellDataMapper cellDataMapper;
    @Autowired
    private PeriodCellDataMapper periodCellDataMapper;
    @Autowired
    private ProjectServiceTypeMapper projectServiceTypeMapper;
    @Autowired
    private PeriodDataSourceMapper periodDataSourceMapper;

    public List<CellTemplatePerGroupDto> getCellTemplateGroupDto(Long templateGroupId, String projectId, String code, Integer rowIndex, Integer columnIndex, Integer period) {
        return adminMp.getCellTemplatePerGroupDto(templateGroupId, code, rowIndex, columnIndex, projectId, period);
    }

    /**
     * 根据列index及row名称查询项目报表模板
     *
     * @param templateGroupId
     * @param projectId
     * @param code
     * @param rowName
     * @param columnIndex
     * @param period
     * @return
     */
    public List<CellTemplatePerGroupDto> getCellTemplateByNameAndIndex(Long templateGroupId, String projectId, String code, Integer rowColumnIndex, String rowName, Integer columnIndex, Integer period) {
        return adminMp.getCellTemplateByNameAndIndex(templateGroupId, code, rowColumnIndex, rowName, columnIndex, projectId, period);
    }

    /**
     * 根据收入类型配置及列index查询报表模板
     * @param templateGroupId
     * @param code
     * @param rowColumnIndex
     * @param taxRate
     * @param revenueType
     * @param taxType
     * @param columnIndex
     * @param projectId
     * @param period
     * @param orgId
     * @param startDate
     * @param endDate
     * @return
     */
    public List<CellTemplatePerGroupDto> getCellTemplateByTypeAndIndex(Long templateGroupId,
                                                                       String projectId,
                                                                       String code,
                                                                       Integer rowColumnIndex,
                                                                       BigDecimal taxRate,
                                                                       Integer revenueType,
                                                                       Integer taxType,
                                                                       Integer columnIndex,
                                                                       Integer period,
                                                                       String orgId,
                                                                       String startDate,
                                                                       String endDate) {
        return adminMp.getCellTemplateByTypeAndIndex(templateGroupId, projectId,code, rowColumnIndex, taxRate, revenueType, taxType, columnIndex, period, orgId, startDate, endDate);
    }

    public Project getFixedProject(String projectId, Integer year) {
        return adminMp.getFixedProject(projectId, year);
    }

    public List<PeriodDataSource> queryManualDataSource(Long templateId, String projectId, Integer period) {
        PeriodDataSourceExample example = new PeriodDataSourceExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andCellTemplateIdEqualTo(templateId).andTypeEqualTo(10);
        return periodDataSourceMapper.selectByExample(example);

    }

    private PeriodReport getReportByTemplate(String templateId, Integer periodId, String projectId) {
        PeriodReportExample example = new PeriodReportExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(Long.valueOf(templateId))
                .andPeriodEqualTo(periodId);
        List<PeriodReport> reports = periodReportMapper.selectByExample(example);

        if (reports != null && !reports.isEmpty()) return reports.get(FIRST_OR_DEFAULT);
        return null;
    }

    private PeriodCellData getCellDataListByTemplate(String templateId, Long reportId) {
        PeriodCellDataExample dataExample = new PeriodCellDataExample();
        dataExample.createCriteria().andCellTemplateIdEqualTo(Long.valueOf(templateId)).andReportIdEqualTo(reportId);
        List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(dataExample);

        if (cellDataList != null && !cellDataList.isEmpty()) return cellDataList.get(FIRST_OR_DEFAULT);
        return null;
    }

    public String getPastProjectId(int year, String orgId) {
        return adminMp.getPastProjectId(year, orgId);
    }

    public PeriodCellData getCellData(String templateId, String cellId, int periodId, String projectId) {
        PeriodReport report = getReportByTemplate(templateId, periodId, projectId);
        MyAsserts.assertNotNull(report, Exceptions.BB_REPORT_NULL);

        PeriodCellData cellData = getCellDataListByTemplate(cellId, report.getId());
        MyAsserts.assertNotNull(cellData, Exceptions.BB_CELL_DATA_NULL);
        return cellData;
    }

}
