package pwc.taxtech.atms.vat.service.impl;

import com.mysql.jdbc.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.constant.DataImport;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.InvoiceMappingDto;
import pwc.taxtech.atms.dto.vatdto.VoucherMappingDto;
import pwc.taxtech.atms.vat.dao.InvoiceMappingMapper;
import pwc.taxtech.atms.vat.dao.VoucherMappingMapper;
import pwc.taxtech.atms.vat.entity.InvoiceMapping;
import pwc.taxtech.atms.vat.entity.InvoiceMappingExample;
import pwc.taxtech.atms.vat.entity.VoucherMapping;
import pwc.taxtech.atms.vat.entity.VoucherMappingExample;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class VoucherInvoiceMappingServiceImpl {
    @Autowired
    private InvoiceMappingMapper invoiceMappingMapper;
    @Autowired
    private VoucherMappingMapper voucherMappingMapper;

    public List<VoucherMapping> getVourcherMappings(Integer period) {
        VoucherMappingExample example = new VoucherMappingExample();
        example.createCriteria().andPeriodEqualTo(period);
        List<VoucherMapping> result = voucherMappingMapper.selectByExample(example);
        result.sort(Comparator.comparing(invoiceMapping -> invoiceMapping.getTranCode() + "_"
                + invoiceMapping.getGroup() + "_" + invoiceMapping.getItemId()));

        return result;
    }

    public List<InvoiceMapping> getInvoiceMappings(Integer period) {
        InvoiceMappingExample example = new InvoiceMappingExample();
        example.createCriteria().andPeriodEqualTo(period);
        List<InvoiceMapping> result = invoiceMappingMapper.selectByExample(example);
        result.sort(Comparator.comparing(invoiceMapping -> invoiceMapping.getPeriod() + "_"
                + invoiceMapping.getInvoiceCode() + "_" + invoiceMapping.getInvoiceNumber()));

        return result;
    }

    public OperationResultDto<List<OperationResultDto<List<VoucherMappingDto>>>> saveVourcherMappings(List<VoucherMappingDto> list,
                                                                                                      int period, boolean isAddition) {
        OperationResultDto<List<OperationResultDto<List<VoucherMappingDto>>>> result = new OperationResultDto<>();
        result.setResult(true);
        result.setData(new ArrayList<OperationResultDto<List<VoucherMappingDto>>>());

        List<VoucherMappingDto> emptyItems = list.stream().filter(m ->
                StringUtils.isEmptyOrWhitespaceOnly(m.TranCode)
                        || StringUtils.isEmptyOrWhitespaceOnly(m.vid)
                        || StringUtils.isEmptyOrWhitespaceOnly(m.Group)
                        || StringUtils.isEmptyOrWhitespaceOnly(m.ItemId)).collect(Collectors.toList());

        if (emptyItems.size() > 0) {
            result.setResult(false);
            OperationResultDto<List<VoucherMappingDto>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.EmptyField);
            result.getData().add(rr);
        }

        emptyItems = list.stream().filter(dto ->
                dto.TranCode != null && dto.TranCode.length() > 50
                        || (dto.vid != null && dto.vid.length() > 50) ||
                        (dto.Group != null && dto.Group.length() > 50) ||
                        (dto.ItemId != null && dto.ItemId.length() > 50)
        ).collect(Collectors.toList());

        if (emptyItems.size() > 0) {
            result.setResult(false);
            OperationResultDto<List<VoucherMappingDto>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.OverLengthField);
            result.getData().add(rr);
        }

        if (result.getResult()) {
            if (!isAddition) {
                VoucherMappingExample voucherMappingExample = new VoucherMappingExample();
                voucherMappingExample.createCriteria().andPeriodEqualTo(period);
                voucherMappingMapper.deleteByExample(voucherMappingExample);
            }
            list.forEach(m -> voucherMappingMapper.insert(m.extractVoucherMapping()));
        }

//        ValidDBVoucherMapping(period); todo:[validation] to be continue (neo)
        return result;
    }

    public OperationResultDto<List<OperationResultDto<List<InvoiceMappingDto>>>> SaveInvoiceMappings(List<InvoiceMappingDto> list,
                                                                                                     int period, boolean isAddition) {
        OperationResultDto<List<OperationResultDto<List<InvoiceMappingDto>>>> result = new OperationResultDto<>();
        result.setResult(true);
        result.setData(new ArrayList<OperationResultDto<List<InvoiceMappingDto>>>());

        List<InvoiceMappingDto> emptyItems = list.stream().filter(m ->
                StringUtils.isEmptyOrWhitespaceOnly(m.TranCode)
                        || StringUtils.isEmptyOrWhitespaceOnly(m.InvoiceCode)
                        || StringUtils.isEmptyOrWhitespaceOnly(m.InvoiceNumber)).collect(Collectors.toList());

        if (emptyItems.size() > 0) {
            result.setResult(false);
            OperationResultDto<List<InvoiceMappingDto>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.EmptyField);
            result.getData().add(rr);
        }

        emptyItems = list.stream().filter(dto ->
                dto.TranCode != null && dto.TranCode.length() > 50
                        || (dto.InvoiceCode != null && dto.InvoiceCode.length() > 50) ||
                        (dto.InvoiceNumber != null && dto.InvoiceNumber.length() > 50)
        ).collect(Collectors.toList());

        if (emptyItems.size() > 0) {
            result.setResult(false);
            OperationResultDto<List<InvoiceMappingDto>> rr = new OperationResultDto<>();
            rr.setData(emptyItems);
            rr.setResult(false);
            rr.setResultMsg(DataImport.OverLengthField);
            result.getData().add(rr);
        }

        if (result.getResult()) {
            if (!isAddition) {
                InvoiceMappingExample invoiceMappingExample = new InvoiceMappingExample();
                invoiceMappingExample.createCriteria().andPeriodEqualTo(period);
                invoiceMappingMapper.deleteByExample(invoiceMappingExample);
            }
            list.forEach(m -> invoiceMappingMapper.insert(m.extractInvoiceMapping()));
        }

//        ValidDBInvoiceMapping(period);todo:[validation] to be continue (neo)

        return result;
    }
}
