package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.WorkbookEvaluator;
import org.apache.poi.ss.formula.eval.*;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.BBParasBo;
import pwc.taxtech.atms.dto.vatdto.CurrentPeriodBo;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.FormulaException;
import pwc.taxtech.atms.vat.entity.PeriodCellData;
import pwc.taxtech.atms.vat.entity.PeriodDataSource;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

public class BB extends FunctionBase implements FreeRefFunction {
    static final Logger LOGGER = LoggerFactory.getLogger(BB.class);
    final static ValueEval defaultEval = new StringEval("0");
    final static BigDecimal defaultBigDecimal = new BigDecimal("0");

    public BB(FormulaContext context) {
        super(context);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        try {
            return wrapExceptionEval(args, ec);
        } catch (Exception e) {
            if (e instanceof FormulaException)
                LOGGER.warn("Formula Exception || {}", e.getMessage());

            e.printStackTrace();
            return defaultEval;
        }
    }

    public ValueEval wrapExceptionEval(ValueEval[] args, OperationEvaluationContext ec) throws Exception {
        List<Object> ds = new ArrayList<>();
        return new NumberEval(bb(new BBParasBo(args, ec), ec, ds, null).doubleValue());
    }

    public BigDecimal bb(BBParasBo bo, OperationEvaluationContext ec, List<Object> dataSource, BBParasBo rootBo) throws Exception {

        CurrentPeriodBo curPeriod = CurrentPeriodBo.getPeriod(bo.getPeriod().intValue(), formulaContext.getPeriod());
        curPeriod.fixedCurYear(getYear(bo.getYear()));

        ReportCellDataSourceDto nullCellDto = ReportCellDataSourceDto.nullDataSource(bo, curPeriod);
        dataSource.add(nullCellDto);

        BigDecimal cellValue = BigDecimal.ZERO;

        try {
            List<CellTemplatePerGroupDto> cellTemplateDataList;
            if (curPeriod.getCurYear() != formulaContext.getYear()) {
                Project project = agent.getFixedProject(formulaContext.getProjectId(), curPeriod.getCurYear());
                MyAsserts.assertNotNull(project, Exceptions.PROJECT_NOT_FOUND);
                cellTemplateDataList = agent.getCellTemplateGroupDto(formulaContext.getReportTemplateGroupId(),
                        project.getId(), bo.getReportCode(), bo.getRowIndex() - 1, bo.getColumnIndex() - 1, curPeriod.getCurPeriod());
            } else {
                cellTemplateDataList = agent.getCellTemplateGroupDto(formulaContext.getReportTemplateGroupId(),
                        formulaContext.getProjectId(), bo.getReportCode(), bo.getRowIndex() - 1, bo.getColumnIndex() - 1, curPeriod.getCurPeriod());
            }
            MyAsserts.assertNotEmpty(cellTemplateDataList, Exceptions.BB_CELL_TEMP_NULL);
            CellTemplatePerGroupDto cellTemplateData = cellTemplateDataList.get(0);
            nullCellDto.fixedWithGroup(cellTemplateData);

            // todo: fix datasource name by templateList(neo)
            if (curPeriod.getCurPeriod() == -99) {
                dataSource.clear();
                BigDecimal returnEval = defaultBigDecimal;
                if (formulaContext.getPeriod() <= 1) {
                    return defaultBigDecimal;
                }

                for (int p = 1; p < formulaContext.getPeriod(); p++) {
                    try {
                        returnEval = returnEval.add(bb(new BBParasBo(bo, p, curPeriod.getCurYear()), ec, dataSource, bo));
                    } catch (Exception e) {
                        if (e instanceof FormulaException) {
                            LOGGER.warn("Formula Exception || {}", e.getMessage());
                        }
                    }
                }

                return returnEval;
            } else if (bo.getPeriod().intValue() == 0) {
                int index = ec.getWorkbook().getSheetIndex(bo.getReportCode());
                cellValue = getCellValue(index, ec, formulaContext, agent, bo.getRowIndex() - 1, bo.getColumnIndex() - 1,
                        Long.parseLong(cellTemplateData.getCellTemplateId()));

                nullCellDto.extractFromGroup(bo, formulaContext.getPeriod(), formulaContext.getYear(), cellTemplateData);
                nullCellDto.setAmount(cellValue);
                return cellValue;
            }

            PeriodCellData cellData = null;

            String projectId = agent.getPastProjectId(curPeriod.getCurYear(),
                    formulaContext.getOrganizationId());

            MyAsserts.assertNotEmpty(projectId, Exceptions.PROJECT_EMPTY);
            cellData = agent.getCellData(cellTemplateData.getReportTemplateId(),
                    cellTemplateData.getCellTemplateId(), curPeriod.getCurPeriod(), formulaContext.getProjectId());
            List<PeriodDataSource> dss = agent.queryManualDataSource(Long.parseLong(cellTemplateData.getCellTemplateId()),
                    formulaContext.getProjectId(), bo.getPeriod());

            nullCellDto.extractFromGroup(bo, curPeriod, cellData, cellTemplateData);


            if (cellData.getData() == null && !dss.isEmpty()) {
                cellValue = dss.get(0).getAmount();
            } else if (cellData.getData() != null && dss.isEmpty()) {
                cellValue = new BigDecimal(cellData.getData()).setScale(4,
                        BigDecimal.ROUND_HALF_DOWN);
            } else if (cellData.getData() != null && !dss.isEmpty()) {
                cellValue = dss.get(0).getAmount().add(new BigDecimal(cellData.getData()).setScale(4,
                        BigDecimal.ROUND_HALF_DOWN));
            } else throw Exceptions.BB_CELL_DATA_NULL;

            nullCellDto.setAmount(cellValue);
            if (rootBo != null) {
                rootBo.putPeriodCellTempate(curPeriod.getCurPeriod(), Long.parseLong(cellTemplateData.getCellTemplateId()));
            } else {
                bo.putPeriodCellTempate(curPeriod.getCurPeriod(), Long.parseLong(cellTemplateData.getCellTemplateId()));
            }

            LOGGER.debug("cell static value ");
            return cellValue;
        } catch (Exception e) {
            logger.warn("[BB_Exception] some error {}", bo.toString());
            throw e;
        } finally {
            if (rootBo == null) {
                LOGGER.warn("[BB_Exception] error for bb cacls for {} and current for {}", bo.toString(), curPeriod.toString());
                Long dataSourceId = saveDataSource(ec, dataSource, FormulaDataSourceDetailType.ReportCellDataSourceDto,
                        cellValue, formulaContext.getPeriod(),
                        formulaContext.getReportTemplateGroupId(), bo.getColumnIndex() - 1, bo.getRowIndex() - 1,
                        formulaContext.getProjectId());
                saveFormulaBlock(formulaContext.getPeriod(), ec,
                        bo.expression(), cellValue, dataSourceId, formulaContext.getProjectId());
            }
        }
    }

}
