package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class DFFS extends FunctionBase implements FreeRefFunction {
    public DFFS(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 5) {
            return null;
        }

        int type = getIntParam(args[0], ec);
        String code = getStringParam(args[1], ec);
        int year = getIntParam(args[2], ec);
        int period = getIntParam(args[3], ec);
        int sourceDataType = getIntParam(args[4],ec);
        String formulaExpression = "DFFS(" + type + ",\"" + code + "\","
                + year + "," + period + ","+sourceDataType+")";
        logger.debug(formulaExpression);

        year = getYear(year);

        Integer yearOffset = 0;
        period = FormulaHelper.getPeriod(period, yearOffset, formulaContext);
        String orgId = formulaContext.getOrganizationId();
        year = year + yearOffset;

        List<ReportCellDataSourceDto> dataSource = Lists.newArrayList();
        if(type == 0){
            EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
            example.createCriteria().andEffectiveDateLessThanOrEqualTo(DateUtils.getNow()).andExpiredDateGreaterThanOrEqualTo(DateUtils.getNow()).andOrganizationIdEqualTo(orgId);
            List<EnterpriseAccountSetOrg> enterpriseAccountSetOrgs = SpringContextUtil.enterpriseAccountSetOrgMapper.selectByExample(example);
            if(CollectionUtils.isEmpty(enterpriseAccountSetOrgs)){
                return NumberEval.ZERO;
            }
            AccountMappingExample accountMappingExample = new AccountMappingExample();
            accountMappingExample.createCriteria().andOrganizationIdEqualTo(orgId).andEnterpriseAccountSetIdEqualTo(
                    enterpriseAccountSetOrgs.get(0).getEnterpriseAccountSetId()).andStandardAccountCodeEqualTo(code);
            List<AccountMapping> accountMappings = SpringContextUtil.accountMappingMapper.selectByExample(accountMappingExample);
            double result = 0;

            for(AccountMapping a : accountMappings){
                if(sourceDataType == 1){
                    result += countForTrialBalance(a.getEnterpriseAccountCode(), dataSource, period, year, orgId);
                }else if(sourceDataType == 2){
                    result += countForAdjBalance(a.getEnterpriseAccountCode(), dataSource, period, year,orgId);
                }else if(sourceDataType == 3){
                    result += countForTrialFinalBalance(a.getEnterpriseAccountCode(), dataSource, period, year,orgId);
                }else{
                    return NumberEval.ZERO;
                }
            }
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.FormulaDataSourceDto,
                    new BigDecimal(result), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());

           return new NumberEval(result);
        } else if(type == 1){
            double result = 0.00;
            if(sourceDataType == 1){
                result = countForTrialBalance(code, dataSource, period, year, orgId);
            }else if(sourceDataType == 2){
                result = countForAdjBalance(code, dataSource, period, year, orgId);
            }else if(sourceDataType == 3){
                result = countForTrialFinalBalance(code, dataSource, period, year, orgId);
            }else{
                return NumberEval.ZERO;
            }
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.FormulaDataSourceDto,
                    new BigDecimal(result), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
            return new NumberEval(result);
        }

        return NumberEval.ZERO;
    }

    private int pardePeriod(int period, int year){
        return Integer.parseInt(""+year  + (period > 9 ? period : ("0" + period)));
    }


    private double countForTrialBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId){
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceExample glBalanceExample = new TrialBalanceExample();
        TrialBalanceExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        TrialBalanceExample.Criteria c2 = glBalanceExample.createCriteria().andSegment4EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        if(organization != null){
            c1.andOrganizationIdEqualTo(organization.getId());
            c2.andOrganizationIdEqualTo(organization.getId());
        }
        glBalanceExample.or(c2);
        List<TrialBalance> list = SpringContextUtil.trialBalanceMapper.selectByExample(glBalanceExample);
        if(CollectionUtils.isEmpty(list)){
            return 0.0;
        }
        List<TrialBalance> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(temp)){
            temp = list;
        }

        for(TrialBalance balance : temp){
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodCr());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodCr().doubleValue()).sum();
    }
    private double countForAdjBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId){
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        AdjustmentTableExample glBalanceExample = new AdjustmentTableExample();
        AdjustmentTableExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        AdjustmentTableExample.Criteria c2 = glBalanceExample.createCriteria().andSegment4EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        if(organization != null){
            c1.andOrganizationIdEqualTo(organization.getId());
            c2.andOrganizationIdEqualTo(organization.getId());
        }
        glBalanceExample.or(c2);
        List<AdjustmentTable> list = SpringContextUtil.adjustmentTableMapper.selectByExample(glBalanceExample);
        if(CollectionUtils.isEmpty(list)){
            return 0.0;
        }
        List<AdjustmentTable> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(temp)){
            temp = list;
        }

        for(AdjustmentTable balance : temp){
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodCrBeq());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodCrBeq().doubleValue()).sum();
    }
    private double countForTrialFinalBalance(String code, List<ReportCellDataSourceDto> contain, int period, int year, String orgId){
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceFinalExample glBalanceExample = new TrialBalanceFinalExample();
        TrialBalanceFinalExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        TrialBalanceFinalExample.Criteria c2 = glBalanceExample.createCriteria().andSegment4EqualTo(code)
                .andPeriodEqualTo(pardePeriod(period, year));
        if(organization != null){
            c1.andOrganizationIdEqualTo(organization.getId());
            c2.andOrganizationIdEqualTo(organization.getId());
        }
        glBalanceExample.or(c2);
        List<TrialBalanceFinal> list = SpringContextUtil.trialBalanceFinalMapper.selectByExample(glBalanceExample);
        if(CollectionUtils.isEmpty(list)){
            return 0.0;
        }
        List<TrialBalanceFinal> temp = list.stream().filter(x -> code.equalsIgnoreCase(x.getSegment3())).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(temp)){
            temp = list;
        }

        for(TrialBalanceFinal balance : temp){
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(balance.getPeriodCr());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return temp.stream().mapToDouble(a -> a.getPeriodCr().doubleValue()).sum();
    }
}
