package pwc.taxtech.atms.vat.service.impl.report.functions;

import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

public class FormulaHelper {

    /**
     * Excel column index begin 1
     *
     * @param colStr
     * @param length
     * @return
     */
    public static int excelColStrToNum(String colStr, int length) {
        int num = 0;
        int result = 0;
        for (int i = 0; i < length; i++) {
            char ch = colStr.charAt(length - i - 1);
            num = (int) (ch - 'A' + 1);
            num *= Math.pow(26, i);
            result += num;
        }
        return result;
    }

    /**
     * Excel column index begin 1
     *
     * @param columnIndex
     * @return
     */
    public static String excelColIndexToStr(int columnIndex) {
        if (columnIndex <= 0) {
            return null;
        }
        String columnStr = "";
        columnIndex--;
        do {
            if (columnStr.length() > 0) {
                columnIndex--;
            }
            columnStr = ((char) (columnIndex % 26 + (int) 'A')) + columnStr;
            columnIndex = (int) ((columnIndex - columnIndex % 26) / 26);
        } while (columnIndex > 0);
        return columnStr;
    }

    public static int getPeriod(int parameterPeriod, Integer yearOffset, FormulaContext formulaFunctionContext) {
        yearOffset = 0;
        if (parameterPeriod == -99) {
            return parameterPeriod;
        } else if (parameterPeriod <= -1) {
            int period = parameterPeriod + formulaFunctionContext.getPeriod();
            while (period <= 0) {
                yearOffset--;
                period += 12;
                //throw new NotImplementedException();
            }

            return period;
        } else if (parameterPeriod == 0) {
            return formulaFunctionContext.getPeriod();
        } else {
            return parameterPeriod;
        }
    }

    public static int getYear(int parameterYear, FormulaContext formulaContext) {
        if (parameterYear <= -1) {
            return parameterYear + formulaContext.getYear();
        } else if (parameterYear == 0) {
            return formulaContext.getYear();
        } else {
            return parameterYear;
        }
    }

    /**
     * 获取小数点后后几位小数
     *
     * @param val            目标数据
     * @param resultType     键值对结果类型
     * @param ifRound        是否精确小数
     * @param formulaContext 公式计算上下文
     * @return
     */
    public static BigDecimal roundValue(BigDecimal val, KeyValueConfigResultType resultType, Boolean ifRound, FormulaContext formulaContext) {
        BigDecimal rtn = null;
        if (ifRound == null) {
            ifRound = formulaContext.getIfRound();
        }

        if (val != null) {
            if (ifRound) {
                if (resultType.equals(KeyValueConfigResultType.Percentage)) {
                    rtn = val.setScale(4, BigDecimal.ROUND_UP);
                } else if (resultType.equals(KeyValueConfigResultType.Accounting)) {
                    rtn = val.setScale(2, BigDecimal.ROUND_UP);
                } else if (resultType.equals(KeyValueConfigResultType.Integer)) {
                    rtn = val.setScale(0);
                } else {
                    rtn = val.setScale(2, BigDecimal.ROUND_UP);
                }
            } else {
                rtn = val;
            }
        }
        return rtn;
    }

    public static Long getCellTemplateIdWithTemplateCodeAndColumnIndexAndRowIndex(String templateCode, int columnIndex, int rowIndex, int period, FormulaContext formulaContext) {
        Map<String, Object> map = new HashMap<>();
        map.put("code", templateCode);
        map.put("templateGroupId", formulaContext.getReportTemplateGroupId());
        map.put("period", period);
        map.put("columnIndex", columnIndex);
        map.put("rowIndex", rowIndex);
        map.put("projectId", formulaContext.getProjectId());
        return SpringContextUtil.periodCellTemplateMapper.getCellTemplateIdWithMap(map);
    }
}
