package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.DataSourceName;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.BBParasBo;
import pwc.taxtech.atms.dto.vatdto.CurrentPeriodBo;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.WPNAMEParasBo;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.FormulaException;
import pwc.taxtech.atms.vat.entity.PeriodCellData;
import pwc.taxtech.atms.vat.entity.PeriodDataSource;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

public class WPNAME extends FunctionBase implements FreeRefFunction {
    static final Logger LOGGER = LoggerFactory.getLogger(WPNAME.class);
    final static ValueEval defaultEval = new StringEval("0");
    final static BigDecimal defaultBigDecimal = new BigDecimal("0");

    public WPNAME(FormulaContext context) {
        super(context);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        try {
            return wrapExceptionEval(args, ec);
        } catch (Exception e) {
            if (e instanceof FormulaException)
                LOGGER.warn("Formula Exception || {}", e.getMessage());

            e.printStackTrace();
            return defaultEval;
        }
    }

    public ValueEval wrapExceptionEval(ValueEval[] args, OperationEvaluationContext ec) throws Exception {
        List<Object> ds = new ArrayList<>();
        return new NumberEval(bb(new WPNAMEParasBo(args, ec), ec, ds).doubleValue());
    }

    public BigDecimal bb(WPNAMEParasBo bo, OperationEvaluationContext ec, List<Object> dataSource) throws Exception {
        ReportCellDataSourceDto nullCellDto = new ReportCellDataSourceDto();
        dataSource.add(nullCellDto);
        BigDecimal cellValue = BigDecimal.ZERO;
        CellTemplatePerGroupDto cellTemplateData = null;
        try {
            List<CellTemplatePerGroupDto> cellTemplateDataList;
            //根据目标列+名称定位数据行，再根据参数列+数据行定位sheet单元格
            cellTemplateDataList = agent.getCellTemplateByNameAndIndex(formulaContext.getReportTemplateGroupId(),
                        formulaContext.getProjectId(), bo.getReportCode(), bo.getRowColumnIndex()-1,bo.getRowName(), bo.getColumnIndex() - 1, formulaContext.getPeriod());
            MyAsserts.assertNotEmpty(cellTemplateDataList, Exceptions.BB_CELL_TEMP_NULL);
            cellTemplateData = cellTemplateDataList.get(0);
            int index = ec.getWorkbook().getSheetIndex(bo.getReportCode());
            cellValue = getCellValue(index, ec, formulaContext, agent, cellTemplateData.getRowIndex(), bo.getColumnIndex() - 1,
                        Long.parseLong(cellTemplateData.getCellTemplateId()));
            nullCellDto.extractFromGroup(bo, formulaContext.getPeriod(), formulaContext.getYear(), cellTemplateData);
            nullCellDto.setAmount(cellValue);
            return cellValue;
        } catch (Exception e) {
            logger.warn("[BB_Exception] some error {}", bo.toString());
            throw e;
        } finally {
                LOGGER.warn("[BB_Exception] error for bb cacls for {}", bo.toString());
                Long dataSourceId = saveDataSource(ec, dataSource, FormulaDataSourceDetailType.ReportCellDataSourceDto,
                        cellValue, formulaContext.getPeriod(),
                        formulaContext.getReportTemplateGroupId(), cellTemplateData.getColumnIndex(), cellTemplateData.getRowIndex(),
                        formulaContext.getProjectId());
                saveFormulaBlock(formulaContext.getPeriod(), ec,
                        bo.expression(), cellValue, dataSourceId, formulaContext.getProjectId());
        }
    }

}
