package pwc.taxtech.atms.dao;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;

import pwc.taxtech.atms.MyMapper;
import pwc.taxtech.atms.dpo.DimensionValueOrgDto;
import pwc.taxtech.atms.dpo.UserOrgRoleDto;
import pwc.taxtech.atms.entity.UserOrganization;

@Mapper
public interface MyUserMapper extends MyMapper {

    @Select("select ur.id as id, ur.is_accessible as IsAccessible, ur.has_original_role as HasOriginalRole, "
            + "org.id as OrganizationID, org.name as OrganizationName from user_organization ur "
            + "join organization org on ur.organization_id = org.id "
            + "where ur.user_id = #{userId} and org.is_active = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "IsAccessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "HasOriginalRole", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "OrganizationID", property = "organizationID", javaType = String.class),
            @Result(column = "OrganizationName", property = "organizationName", javaType = String.class) })
    List<Map<String, Object>> selectUserOrganizationMapByUserId(@Param(value = "userId") String userId);

    @Select("select urd.id as id, urd.dimension_value_id as DimensionValueID, urd.dimension_id as DimensionID," +
            " urd.user_id as UserID, urd.is_accessible as IsAccessible, urd.has_original_role as HasOriginalRole, u.user_name as UserName"
            + " from user_dimension_value urd " + "join user u on urd.user_id = u.id " + "where u.status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "DimensionValueID", property = "dimensionValueId", javaType = String.class),
            @Result(column = "DimensionID", property = "dimensionId", javaType = String.class),
            @Result(column = "UserID", property = "userId", javaType = String.class),
            @Result(column = "IsAccessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "HasOriginalRole", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "UserName", property = "userName", javaType = String.class) })
    List<Map<String, Object>> selectUserDimensionValueMap();

    @Select("select x.role_id as RoleID, y.name as RoleName from user_role x " + "join role y on x.role_id = y.id "
            + "where x.user_id = #{userId} and x.project_id = \"\"")
    @Results({ @Result(column = "RoleID", property = "roleID", javaType = String.class),
            @Result(column = "RoleName", property = "roleName", javaType = String.class) })
    List<Map<String, String>> selectUserRoleListByUserId(@Param(value = "userId") String userId);

    @Select("select q.id as id, q.organization_id as OrganizationID, q.user_id as UserID, q.is_accessible as IsAccessible, " +
            "q.has_original_role as HasOriginalRole from user_organization q join User u on q.user_id = u.id "
            + "where u.status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "OrganizationID", property = "organizationID"),
            @Result(column = "HasOriginalRole", property = "hasOriginalRole"),
            @Result(column = "UserID", property = "userID"),
            @Result(column = "IsAccessible", property = "isAccessible") })
    List<UserOrganization> selectUserOrganizationByUserId();


    @Select("select p.Dimension_Value_ID, p.Dimension_ID, d.Name as DimensionName, p.Dimension_Value_Name as Dimension_Value "
            + "from (select id as Dimension_Value_ID, name as Dimension_Value_Name, is_active as Is_Active, #{dimensionId} as Dimension_ID, "
            + "create_time as Create_Time, update_time as Update_Time from business_unit where is_active = 1 " + "union "
            + "select id as DimensionValueID, name as DimensionValueName, is_active as Is_Active, dimension_id as Dimension_ID, create_time as Create_Time, update_time as Update_Time"
            + " from dimension_value) p " + "join dimension d on p.Dimension_ID = d.id " + "where d.is_active = 1")
    @Results({ @Result(column = "DimensionValueID", property = "dimensionValueID"),
            @Result(column = "DimensionID", property = "dimensionId"),
            @Result(column = "DimensionName", property = "dimensionName"),
            @Result(column = "DimensionValue", property = "dimensionValue") })
    List<DimensionValueOrgDto> selectUserRoleDimensionValueByDimensionId(
            @Param(value = "dimensionId") String dimensionId);

    @Select("select urd.id, urd.organization_id as OrgID, urd.user_id as UserID, urd.is_accessible as IsAccessible, urd.has_original_role as HasOriginalRole, u.user_name as UserName "
            + "from user_organization urd join user u on urd.user_id = u.id where u.status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "OrgID", property = "orgID", javaType = String.class),
            @Result(column = "UserID", property = "userID", javaType = String.class),
            @Result(column = "IsAccessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "HasOriginalRole", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "UserName", property = "userName", javaType = String.class) })
    List<Map<String, Object>> selectUserOrganizationMap();

    @Select("select ur.user_id as UserID, r.id as RoleID, r.name as RoleName from user_role ur join role r on ur.role_id = r.id")
    @Results({ @Result(column = "UserID", property = "userID"), @Result(column = "RoleID", property = "roleID"),
            @Result(column = "RoleName", property = "roleName") })
    List<UserOrgRoleDto> selectUserOrgRoleDto();
}
