﻿basicDataModule
.controller('customerListManageController', ['$scope', '$log', 'SweetAlert', 'customerService', 'enterpriseAccountService', 'uiGridConstants', '$translate', 'Upload', 'apiInterceptor', '$interval', '$timeout','$q',
    function ($scope, $log, SweetAlert, customerService, enterpriseAccountService, uiGridConstants, $translate, Upload, apiInterceptor, $interval, $timeout, $q) {
        'use strict';
         
        var successAddMessage = $translate.instant('OrganizationStructureAddSuccess');
        var successEditMessage = $translate.instant('OrganizationStructureEditSuccess');
        var successDeleteMessage = $translate.instant('OrganizationStructureDeleteSuccess');
        var confirmToDelete = $translate.instant('ConfirmToDelete');
        var ImportErrorMsg = $translate.instant('ImportErrorMsg');
        var noItemSelected = $translate.instant('NoItemSelected');
        var depulicateNote = $translate.instant('CustomerDuplicateNode');
        var confirmImportOverwriteTitle = $translate.instant('ConfirmImportOverwriteTitle');
        var confirmImportOverwriteText = $translate.instant('ConfirmImportOverwriteText');

        var resumable = true;
        var uploadUrl = apiInterceptor.webApiHostUrl + '/customer/Upload';
        //var uploadUrl = apiInterceptor.webApiHostUrl + '/init/Upload';

        var showError = function (resultMsg) {

            var errMsg = $translate.instant(resultMsg);
            SweetAlert.warning(errMsg);
        };

        //获取企业账套列表
        var loadEnterpriseAccountSetList = function () {
            enterpriseAccountService.getEnterpriseAccountSetList().success(function (data) {
                if (data && data.length > 0) {
                    $scope.enterpriseAccountSetList = data;
                }
            });
        };

        //ng-repeat,根据字符串转换为json对象
        var getJson = function (obj) {
            if (obj) {
                return JSON.parse(obj);
            }

        }
        //根据账套ID获取客户列表
        var loadCustomerListBySetID = function () {

            resetErrorTab();
            resetUIGridStatus();

            var id = $scope.accountSetID;

            $scope.hasCustomerList = false;
            customerService.getByID(id).success(function (data) {
                if (data.customerList && data.customerList.length > 0) {
                    $scope.customerList = data.customerList;
                    $scope.gridOptions.data = data.customerList;

                    //查看是否有错误信息
                    loadImportErrorInfoList(data.validateInfoList);

                    $scope.hasCustomerList = true;
                }
                else {
                    $scope.hasCustomerList = false;
              
                }
            });
        };
      
        //reset hightlight to false.
        var setDefaultTheme = function () {
            if ($scope.gridOptions.data && $scope.gridOptions.data.length > 0) {
                $scope.gridOptions.data.forEach(function (row) {
                    row.isHighLight = false;
                });
            }
        };

        //当选择错误信息时，highlight data
        $scope.chooseRelavantRow = function (err) {

            //setDefaultTheme();

            if (err && err.inValidateCodeList && err.inValidateCodeList.length > 0) {
                err.inValidateCodeList.forEach(function (row) {

                    var matchList = _.filter($scope.gridOptions.data, function (num) {
                        return num.code === row
                    });

                    if (matchList && matchList.length > 0) {
                        matchList.forEach(function (row2) {
                            row2.isHighLight = !row2.isHighLight;
                        });
                    }

                });
            }
        };

        var loadImportErrorInfoList = function (data) {
            
            if (data && data.length > 0) {
                var totalLength = 0;
                var i = 1;
                data.forEach(function (row) {

                    totalLength += row.inValidateCodeList.length;

                    row.detail = row.inValidateCodeList.join(constant.comma);
                    if (row.type === 'CustomerDuplicateNode') {
                        row.isRepeatError = true;
                    } else {
                        row.isRepeatError = false;
                    }

                    row.type = $translate.instant(row.type);
                    row.index = i;
                    i++;
                });

                showErrTab(data,totalLength);

            }
            else {
                resetErrorTab();
            }
        };

        var init = function () {
            $('#importCustomerListSet').modal('hide');
            resetErrorTab();
            loadEnterpriseAccountSetList();
            CheckHasAccountSet();
        };
        //没有选择企业账套时，显示请选择账套
        var CheckHasAccountSet = function () {
            if (!$scope.accountSetID) {
                $scope.hasEnterpriseAccountSet = false;
                //没有选择账套，肯定没有客户列表
                $scope.hasCustomerList = false;
            }
            else {
                $scope.hasEnterpriseAccountSet = true;
            }
        };

        var resetErrorStatus = function () {
            var currentForm = $('#addcustomerForm');
            $.each(currentForm.children(), function (index, element) {
                $(element).find('.has-error').removeClass('has-error');
            });
            validator.resetForm();
        };

        var resources = {
            codeRequired: $translate.instant('CustomerCodeEmptyNode'),
            nameRequired: $translate.instant('CustomerNameEmptyNode'),
            customerDuplicateNode: $translate.instant('CustomerDuplicateNode')
        };


        $.validator.addMethod("codeDuplicate", function (value, element, param) {

            if ($scope.customerList && $scope.customerList.length > 0) {

                var objects = $.grep($scope.customerList, function (n, i) {
                    return $scope.editModel.id != n.id && $.trim(n.code) == $.trim(value);
                });

                return objects && objects.length < 1;  //if < 1, then pass the validation
            }
            return true;
        });


        var validator = $("#addcustomerForm").validate({
            errorClass: "has-error",
            rules: {
                code: {
                    required: true,
                    codeDuplicate: true
                },
                name: {
                    required: true
                }
            },
            messages: {
                code: {
                    required: resources.codeRequired,
                    codeDuplicate: depulicateNote
                },
                name: {
                    required: resources.nameRequired
                }
            },
            errorPlacement: function (error, element) {
                setErrorStyle(error, element);
            }
        });

        var setErrorStyle = function (error, element) {
            if (element.hasClass('has-error')) {
                element.parent().addClass('has-error');
                error.insertAfter(element);
                error.addClass('label');
            }
        };

        $scope.hideErrorLine = function () {
            $scope.ImportErrorLine = false;
            $scope.ImportErrorTab = false;  //不显示
            setErrorWrapCssDefault();
        };

        var resetErrorTab = function () {
            $scope.ImportErrorTab = false;
            $scope.ImportErrorTag = false;
            $scope.importErrorTips = null;
            $scope.ImportErrorLine = false;
            // $scope.errorGridOptions.data = [];
            resetErrorStatus();
        };

        $scope.search = function () {
            $scope.gridApi.grid.refresh();
        };

        var $contentResizerCustomer=$("#content-resizer-customer");
        var $topIcon = $("#customer-topIcon");
        var $customerErrorWarp = $("#customer-error-warp");
        //点击Icon切换显示错误列表
        $scope.toggleErrorTab = function () {
            $scope.ImportErrorTab = !$scope.ImportErrorTab;
            //topIcon and content-resize gapBottom 15px
            if (!$scope.ImportErrorTab) {
                setErrorWrapCssDefault();
            } else {

                var resizeBottemHeight = $contentResizerCustomer.css('bottom');
                $log.debug('resizeBottemHeight: ' + resizeBottemHeight);
                if (parseInt(resizeBottemHeight) < 130)
                {
                    $contentResizerCustomer.css('bottom', '130px');
                    $topIcon.css({
                        bottom: '124px',
                    });

                    $customerErrorWarp.css('height', '130px');
                }
            }
        };

        var setErrorWrapCssDefault = function () {
            $contentResizerCustomer.css('bottom', '26px');
            $topIcon.css({
                bottom: '20px',
            });

            $customerErrorWarp.css('height', '26px');
        };

        //enterprise list onchange then repopulate the customer list
        $scope.loadCustomerListBySetID = function () {
            CheckHasAccountSet();
        
            loadCustomerListBySetID();
            setErrorWrapCssDefault();
        };

        var resetUIGridStatus = function () {
            //切换 的时候设置全选为false
            $scope.gridApi.grid.selection.selectAll = false;
            $scope.gridOptions.data = [];
            $scope.gridErrorOptions.data = [];
            $scope.customerList = null;
        };

        //when import failed, then show error list
        var showErrTab = function (data,errorLength) {
            $scope.ImportErrorTab = false;  //默认不显示
            $scope.ImportErrorTag = true;
            $scope.ImportErrorLine = true;
            $scope.importErrorTips = $translate.instant('ImportErrorTips').format(errorLength);
            $scope.gridErrorOptions.data = data;
        };
       
        $scope.popupImportFileModel = function () {
            if ($scope.file) $scope.file = null;
            $('#importCustomerListSet').modal('show');
        };
    
        //pop up model to add or edit customer
        var fillModel = function () {
            resetErrorStatus();
            $scope.editModel = {};

            var set = _.find($scope.enterpriseAccountSetList, function(num){return num.id === $scope.accountSetID;});

            $scope.editModel.enterpriceAccountSetName = set.name;
            $scope.editModel.enterpriseAccountSetID = $scope.accountSetID;
        };

        $scope.popupNewCustomer = function ()
        {
            $scope.isAdd = true;
            fillModel();
            $('#addCustomerModel').modal('show');
        }
       
        $scope.popupEditCustomer = function (customer) {

            $scope.gridApi.selection.clearSelectedRows();
            $scope.isAdd = false;
            fillModel();
            $scope.editModel = angular.extend($scope.editModel, customer);

            $('#addCustomerModel').modal('show');
        }

        //save or edit customer
        $scope.saveCustomer = function () {

            if (!($('#addcustomerForm').valid())) {
                return;
            }

            var customer = [];

            var enterpriseAccountSetID = $scope.accountSetID;
            var code = $scope.editModel.code;
            var name = $scope.editModel.name;
            if (!code || !name) return false;

            //add customer
            if ($scope.isAdd) {
                customer.push({ id: PWC.newGuid(), code: code, name: name, enterPriseAccountID: enterpriseAccountSetID });
                customerService.addRange(customer).success(function (data) {
                    var result = data[0];
                    if (result && result.data && result.result == false && result.resultMsg != null) {
                        showError(result.resultMsg);
                        return false;
                    }
                    $('#addCustomerModel').modal('hide');
                    SweetAlert.success(successAddMessage);
                    loadCustomerListBySetID();
                });
            }
                //edit customer
            else {
                customer.push({ id: $scope.editModel.id, code: code, name: name, enterPriseAccountID: enterpriseAccountSetID });

                customerService.updateRange(customer).success(function (data) {
                    var result = data[0];
                    if (result && result.data && result.result == false && result.resultMsg != null) {
                        showError(result.resultMsg);
                        return false;
                    }
                    $('#addCustomerModel').modal('hide');
                    SweetAlert.success(successEditMessage);
                    loadCustomerListBySetID();
                });
            }
        };

        var confirmWarningWindow = function (title, text) {
            var deferred = $q.defer();
            SweetAlert.swal({
                title: title,
                text: text,
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                allowOutsideClick:false,
                confirmButtonText: $translate.instant('Confirm'),
                cancelButtonText: $translate.instant('Cancel'),
                closeOnConfirm: false,
                closeOnCancel: true
            },
              function (isConfirm) {
                  deferred.resolve(isConfirm);
              });

            return deferred.promise;
        };

        $scope.deleteSelectedCustomers = function () {

            var selectedRows = $scope.gridApi.selection.getSelectedRows();
            if (selectedRows.length === 0) {
                SweetAlert.warning(noItemSelected);
                return;
            }

            confirmWarningWindow(confirmToDelete, '').then(function (data) {
                if (data) {
                    var customerArr = [];
                    selectedRows.forEach(function (item) {
                        customerArr.push(item);
                    });

                    deleteCustomer(customerArr);
                }
            });
        };
        //delete customer action
        var deleteCustomer = function (customerArr) {

            customerService.deleteRange(customerArr).success(function (result) {
                SweetAlert.success(successDeleteMessage);
                loadCustomerListBySetID();
            });
        };

        //delete single row customer
        $scope.removeCustomer = function (customer) {

            $scope.gridApi.selection.clearSelectedRows();

            confirmWarningWindow(confirmToDelete, '').then(function (data) {
                if (data) {
                    var customerArr = [];
                    customerArr.push(customer);
                    deleteCustomer(customerArr);
                }
            });
        };

        //覆盖导入
        $scope.ImportCustomerOverwrite = function () {
            if (!$scope.file || !$scope.file.name) {

                SweetAlert.warning($translate.instant('SelectCustomerFileRequired'));
                return;
            }
            confirmWarningWindow(confirmImportOverwriteTitle, confirmImportOverwriteText).then(function (data) {
                if (data) {
                    ImportCustomer(constant.import.overwrite);
                }
            });
        };

        //追加导入
        $scope.ImportCustomerAppend = function () {

            ImportCustomer(constant.import.append);
        }
        //upload file
        var ImportCustomer = function (action) {
          
            if (!$scope.file || !$scope.file.name) {

                SweetAlert.warning($translate.instant('SelectCustomerFileRequired'));
                return;
            }

            var tempFileName = PWC.newGuid() + '.dat';
         
            var token = $('input[name="__RequestVerificationToken"]').val();
            // updateProgressToZero();
            Upload.upload({
                url: uploadUrl,
                data: {
                    cancel: false,
                    enterpriseAccountID: $scope.accountSetID,
                    filename: $scope.file.name,
                    tempFileName: tempFileName,
                    action:  action
                },
                file: $scope.file,
                resumeChunkSize: resumable ? $scope.chunkSize : null,
                headers: {
                    'Access-Control-Allow-Origin': '*',
                    Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                    __RequestVerificationToken: token,
                    withCredentials: true
                }, withCredentials: true
            }).then(function (resp) {
                // $log.debug('Success ' + resp.config.data.file.name + 'uploaded. Response: ' + JSON.stringify(resp.data));
                var ret = resp.data;
                if (ret && ret.length == 0) {
                    // 全部导入成功
                    $('#importCustomerListSet').modal('hide');
                    SweetAlert.success($translate.instant('ImportSuccess'));
                } else {
                    // 提示没有导入成功的数据
                    //var errorInfo = $translate.instant('ImportPartialSuccess');
                    var errorMsg = '';
                    if (ret[0].resultMsg && constant.customer.errorArray.indexOf(ret[0].resultMsg) > -1) {
                        errorMsg = ret[0].resultMsg;
                    }
                    else {
                        errorMsg = 'ImportPartialSuccess';
                    }
                    var errorInfo = $translate.instant(errorMsg);
                    SweetAlert.swal({
                        title: $translate.instant('BasicDataCustomer'),
                        text: errorInfo,
                        type: "warning",
                        showCancelButton: false,
                        closeOnConfirm: false,
                        closeOnCancel: true,
                        html: true
                    });
                   $("#importCustomerListSet").modal('hide');
                }
                loadCustomerListBySetID();

            }, function (resp) {
                console.log('Error status: ' + resp.status);
            }, function (evt) {
                var progressPercentage = parseInt(100.0 * evt.loaded / evt.total);
                $log.debug('progress: ' + progressPercentage + '% ' + evt.config.data.file.name);
            });

        };


        var initErrorUIGrid = function () {
            $scope.gridErrorOptions = {
                rowHeight: constant.UIGrid.errorRowHeight,
                selectionRowHeaderWidth: constant.UIGrid.errorRowHeaderWidth,
                enableColumnMenus: false,
                enableSorting: true,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: false, // 行选择是否可用，默认为true; 如果想选择任意位置和勾勾，需要设置为false
                enableRowHeaderSelection: false,
               // enableFullRowSelection: true, //是否点击行任意位置后选中,默认为false
                enableSelectAll: false, // 选择所有checkbox是否可用，默认为true; 
                multiSelect: false,// 是否可以选择多个,默认为true;
                columnDefs: [
                    {
                        field: 'SequenceNo',
                        name: $translate.instant('SequenceNo'),
                        width: '15%',
                        cellTemplate: '<div  class="text-align-left-padding" ng-click="grid.appScope.chooseRelavantRow(row.entity)"><span>{{row.entity.index}}</span></div>'
                    },
                   {
                       field: 'type',
                       name: $translate.instant('ErrorMesssage'),
                       width: '20%',
                       cellTemplate: '<div  class="text-align-left-padding" ng-click="grid.appScope.chooseRelavantRow(row.entity)"><span>{{row.entity.type}}</span></div>'
                   },
                    {
                        field: 'detail',
                        name: $translate.instant('CustomerCode'),
                        cellTemplate: '<div  class="text-align-left-padding" ng-click="grid.appScope.chooseRelavantRow(row.entity)">' +
                          '<span title="{{row.entity.detail}}">{{row.entity.detail | limitString:95 }}</span>' +
                      '</div>',
                        width: '65%'

                    }
               

                ],

                onRegisterApi: function (gridApi) {
                    $scope.grideErrorApi = gridApi;

                    $interval(function () {
                        $scope.grideErrorApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            }
        };


        $scope.getGridHeight = function () {

            if ($scope.isLoadComplete && $scope.hasCustomerList) {
                var y = $("#customerDataList").height();

                // Enough space
                if (y > constant.UIGrid.gapHeight) {
                    y = y - constant.UIGrid.gapHeight;
                    return { height: y + "px" };
                } else {

                    return { height: '0px' };
                }
            }

            return {};
        };

        $scope.getErrorGridHeight = function () {

            if ($scope.isLoadComplete) {
                var y = $("#customer-error-warp").height();

                // Enough space
                if (y > constant.UIGrid.gapHeight) {
                    y = y - constant.UIGrid.gapHeight;
                    return { height: y + "px" };
                } else {

                    return { height: '0px' };
                }
            }

            return {};
        };

        var initListUIGrid = function () {

            var operateStr = '';

            if ($scope.hasEditPermission){
                operateStr = '<div class="ui-grid-cell-contents> <a class="operate-btn" href="javascript:void(0)">\
                                <i class="material-icons">&#xE254;</i>&nbsp;\
                                <span ng-click="grid.appScope.popupEditCustomer(row.entity)">{{"Edit"|translate}}</span>\ </a>\&nbsp;&nbsp;| &nbsp;&nbsp;\
                             <a class="operate-btn" href="javascript:void(0)">\
                                <i class="material-icons highlight-tag">&#xE872;</i>&nbsp;\
                                <span ng-click="grid.appScope.removeCustomer(row.entity)">{{"Delete"|translate}}</span>\ </a>'+
                                '</div>';

            } else{
                                
                operateStr = '<div class="ui-grid-cell-contents '+constant.noPermissionClass + '">\
                                <a class="operate-btn" href="javascript:void(0)" >\
                                <i class="material-icons">&#xE254;</i>&nbsp;\
                                <span>{{"Edit"|translate}}</span>\ </a>\&nbsp;&nbsp;| &nbsp;&nbsp;\
                             <a class="operate-btn" href="javascript:void(0)">\
                                <i class="material-icons highlight-tag">&#xE872;</i>&nbsp;\
                                <span>{{"Delete"|translate}}</span>\ </a>'+
                                '</div>';
            }

            $scope.gridOptions = {
                rowHeight: constant.UIGrid.rowHeight,
                selectionRowHeaderWidth: constant.UIGrid.selectionRowHeaderWidth,
                enableSorting: true,
                enableColumnMenus: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: false, // 行选择是否可用，默认为true; 如果想选择任意位置和勾勾，需要设置为false
                enableRowHeaderSelection: true,
                enableFullRowSelection: true, //是否点击行任意位置后选中,默认为false
                enableSelectAll: true, // 选择所有checkbox是否可用，默认为true; 
                multiSelect: true,// 是否可以选择多个,默认为true;
                virtualizationThreshold: 60,
                rowTemplate: '<div ng-repeat="(colRenderIndex, col) in colContainer.renderedColumns track by col.colDef.name" class="ui-grid-cell" ng-class="{\'ui-grid-row-header-cell\':col.isRowHeader,\'red-color\':row.entity.isHighLight }"  ui-grid-cell></div>',

                columnDefs: [
                    {
                        field: 'code',
                        name: $translate.instant('CustomerCode'),
                        width: '35%',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.code}}">{{row.entity.code}}</span></div>'
                    },
                    {
                        field: 'name',
                        name: $translate.instant('CustomerName'),
                        width: '40%',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.name}}">{{row.entity.name}}</span></div>'
                    },
                    {
                        field: 'Operation',
                        name: $translate.instant('Operation'),
                        cellTemplate: operateStr,

                        width: '25%',
                        headerCellClass: 'operate-text-align-center'
                         
                    }
                ],

                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    $scope.gridApi.grid.registerRowsProcessor($scope.singleFilter, 200);

                    var onRowsRenderedOff = gridApi.core.on.rowsRendered(null, function () {
                        onRowsRenderedOff(); // run once
                        PWC.triggerRowSelectOnClick('#customerlistUIGrid'); 
                    });

                    $interval(function () {
                        $scope.gridApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };

            $scope.singleFilter = function (renderableRows) {
              
                var matcher = new RegExp($scope.searchEASText);
                renderableRows.forEach(function (row) {

                    try {
                        var match = false;
                        ['code', 'name'].forEach(function (field) {
                            if (row.entity[field].match(matcher)) {
                                match = true;
                            }
                        });
                        if (!match) {
                            row.visible = false;
                        }
                    } catch (e) {
                        $log.debug('singleFilter-match:' + JSON.stringify(row.entity));
                    }
                });
                return renderableRows;
            };

        };
 

         var havePermission = function() {
                $scope.$root.checkUserPermission(constant.adminPermission.basicData.customerList.editCode).success(function(data) {
                    $scope.hasEditPermission = data;

                    initListUIGrid();
                    initErrorUIGrid();
                    init();
           
                });
            };

        (function initialize() {

            $scope.hasEditPermission = false;
            havePermission();

            $scope.isLoadComplete = false;
            $log.debug('customerListManageController.ctor()...');
            $timeout(function () {

                $scope.isLoadComplete = true;

            }, 500);

        })();
    }
]);