﻿infrastructureModule
    .controller('storeArchitectureViewController', ['$scope', '$log', 'SweetAlert', '$translate', '$timeout', 'stockService', '$document', '$state', '$stateParams', '$interval', 'roleService', '$q', 'userService', '$uibModal', 'dimensionService', 'CacheFactory','ackMessageBox','ackUibModal','ackLib',
        function ($scope, $log, SweetAlert, $translate, $timeout, stockService, $document, $state, $stateParams, $interval, roleService, $q, userService, $uibModal, dimensionService, CacheFactory, ackMessageBox, ackUibModal,ackLib) {
            'use strict';

            var charContainerID = '#store-chart-container';
             
            $scope.entity = {
                stockDate: new Date(),
                currentDate: new Date()
            };

            $scope.stockOptions = {
                empty: 0,
                modify: 1, //修改  
                addHistory: 2 //变更
            }

            $scope.stockStatus = $scope.stockOptions.empty;

            var template = {
                topNodeTemplate: $("#topNode-name").html(),  //最顶层节点
                nameNodeEdit: $("#node-name-edit").html(),  //name,机构名称
                nameNodeQuery: $("#node-name-query").html(),  //
                titleNode: $("#title-name-node").html() //普通行
            };

            var ReplaceObj = {
                //顶部节点
                topNodeName: '{{#topNodeName#}}',
                //机构名称
                organizationName: '{{#organizationName#}}',
                organizationID: '{{#organizationID#}}',
                moneyNumber: '{{#moneyNumber#}}',    //多少钱 stockPercent
                stockPercent: '{{#stockPercent#}}',
            };


            // 关于日志部分
            $scope.showOperateLogPop = function () {
                $scope.isShowLog = true;
            };


            $(document).on('blur', '#stockList .dx-texteditor-input', function (e) {
                $scope.stockGridInstance.focus($(this));
            });

            var orgchartService = {
                //获取stock架构的列表
                getDataSource: function () {
                    var deferred = $q.defer();
                    var param = {
                        currentDate: $scope.entity.stockDate || ''
                    };

                    stockService.getStockList(param).success(function (data) {
                        if (data) {
                            //console.log(JSON.stringify(data));
                            deferred.resolve(data);
                        }
                    });
                    return deferred.promise;
                },

                //创建orgchart节点
                createNewNode: function (row, templateNameNode, templateTitleNode) {

                    var temp = [];
                    var relationship = '110';
                    if (row.children && row.children.length > 0) {
                        relationship = '111';
                    }

                    var node = {
                        'name': templateNameNode.replaceToEnd(ReplaceObj.organizationName, row.name)
                                        .replaceToEnd(ReplaceObj.organizationID, row.id),
                        'stockAmount': row.stockAmount,
                        'capitalPercent': row.capitalPercent,
                        //'title': templateTitleNode.replaceToEnd(ReplaceObj.moneyNumber, row.capitalAmount).replaceToEnd(ReplaceObj.stockPercent, row.capitalPercent == undefined ? '0' : row.capitalPercent),

                        'children': [],
                        'relationship': relationship
                    };

                    return node;
                },

                //递归生成子节点
                createNodeRecursion: function (childrenList, parentNode, templateNameNode, templateTitleNode) {

                    if (childrenList.length == 0) return;

                    childrenList.forEach(function (row) {
                        var newNode = orgchartService.createNewNode(row, templateNameNode, templateTitleNode);
                        parentNode.children.push(newNode);

                        if (row.children && row.children.length > 0) {
                            orgchartService.createNodeRecursion(row.children, newNode, templateNameNode, templateTitleNode);
                        }
                    });
                },

                populateOrgChart: function () {

                    this.getDataSource().then(function (data) {

                        //如果没有编辑维度值的权限，则不显示编辑的笔icon
                        var templateNameNode = template.nameNodeQuery;
                        if ($scope.hasDimensionValueEditPermission) {
                            templateNameNode = template.nameNodeEdit;
                        };
                        var templateTitleNode = template.titleNode;

                        // orgService.getOrgDashboard($scope.parentDimensionID).success(function (data) {
                        orgchartService.renderOrgChart({});

                        if (data !== undefined && data !== null && data.length > 0) {
                            //console.log(JSON.stringify(data));
                            var topNodeName = template.topNodeTemplate.replace(ReplaceObj.topNodeName, data[0].name);

                            var datasource = {
                                'name': topNodeName,
                                'children': []
                            };
                            data.forEach(function (row) {

                                var newNode = orgchartService.createNewNode(row, templateNameNode, templateTitleNode);
                                datasource.children = newNode.children;

                                if (row.children && row.children.length > 0) {
                                    orgchartService.createNodeRecursion(row.children, newNode, templateNameNode, templateTitleNode);
                                }

                            });
                            //生成orgchart
                            if (datasource.children.length > 0) {
                                $scope.stockArchitectureCache = datasource;
                                orgchartService.renderOrgChart(datasource);
                            }
                        }
                    });
                },

                //开始生产org chart树
                renderOrgChart: function (datasource) {

                    $(charContainerID).html('');
                    //渲染orgchart
                    $(charContainerID).orgchart({
                        'data': datasource,
                        'pan': false,
                        'zoom': false,
                        "draggable": false,
                        'exportButton': false,
                        'direction': 'l2r',
                        'nodeContent': 'title',
                        'exportFilename': 'SportsChart',
                        'parentNodeSymbol': 'fa-th-large',
                        'zoomoutLimit': 2,
                        'zoominLimit': 7,
                        'depth': 5,
                        'createNode': function ($node, data) {
                            if (data.relationship != '001') {
                                var amount = data.stockAmount <= 0 ? '-' : data.stockAmount;
                                var leftMenu = '<div class="node-line-left"><span >' + amount + '</span>';
                                var rightMenu = '<div class="node-line-right"><span >' + data.capitalPercent + '%</span>';
                                $node.append(leftMenu).append(rightMenu);
                            }
                        }
                    });
                    var horizontalCenter = Math.floor(window.innerHeight / 2);
                    console.log('horizontalCenter: ' + horizontalCenter);
                    $(charContainerID).scrollTop(horizontalCenter + 'px');
                },

            };

            $scope.modalService = {
                stockModal: {
                    open: function () {
                        $scope.stockModalInstance = ackUibModal($scope, 'stockEdit.html', 'stockEditModal', '.store-architecture-view', 'static')
                        $scope.stockModalInstance.open();
                    },
                    save: function (result) {
                        
                        $scope.saveStock();
                
                       
                    },
                    cancel: function () {
                        $scope.stockModalInstance.cancel();
                    }
                }
            };

            var getService = {
                getStockListByOrgID: function (param) {
                    var deferred = $q.defer();
                    stockService.getStockListByOrgID(param).success(function (data) {
                        if (data) {
                            //console.log(JSON.stringify(data));
                            deferred.resolve(data);
                        }
                    });
                    return deferred.promise;
                }
            };

            var updateService = {
                updateStock: function (stock) {
                    var deferred = $q.defer();
                    stockService.updateStock(stock).success(function (data) {
                        if (data) {
                           // console.log(JSON.stringify(data));
                            deferred.resolve(data);
                        }
                    });
                    return deferred.promise;
                },
                //删除机构
                deleteOrgList: function (stockList, orgID) {
                    var deferred = $q.defer();
                    if (stockList.length > 0) {
                        stockService.deleteOrgList(stockList, orgID).success(function (data) {
                            if (data) {
                                deferred.resolve(data);
                            }
                        });
                    }
                    else {
                        deferred.resolve(true);
                    }
                    return deferred.promise;
                },
                //添加机构
                addOrgList: function (stockList, orgID) {
                    var deferred = $q.defer();
                    if (stockList.length > 0) {
                        stockService.addOrgList(stockList, orgID).success(function (data) {
                            if (data) {
                                deferred.resolve(data);
                            }
                        });
                    }
                    else {
                        deferred.resolve(true);
                    }
                    return deferred.promise;
                },
                updateOrgList: function (stockList, orgID) {
                    var deferred = $q.defer();
                    if (stockList.length > 0) {
                        stockService.updateOrgList(stockList, orgID).success(function (data) {
                            if (data) {
                                deferred.resolve(data);
                            }
                        });
                    }
                    else {
                        deferred.resolve(true);
                    }
                    return deferred.promise;
                }

            };


            $scope.fullScreenText = '全屏展示';
            $scope.showBefore = true;

            //顶部的维度状态改变，更新orgchart
            $scope.$on('DimensionUpdate', function (event, data) {
                initData();
                setParentDimension();
            });

            //通知父层去更新统计
            var notifyTopPage = function () {
                $scope.$emit("hasDimensionUpdated", { isupdated: true });
            };

            //隐藏机构general info
            var hideOrgGeneralInfo = function () {
                $(orgGeneralInfoPopId).hide();
            };

            // 显示机构general info，并设置好位置
            var showOrgCard = function (popCard, row, topOffset, leftOffset) {

                popCard.css("display", "block");

                var top = row.offset().top + topOffset;
                if (top + popCard.height() > window.innerHeight) {
                    top = window.innerHeight - popCard.height() - topOffset;
                    //top = row.offset().top - topOffset - popCard.height();
                }

                popCard.css('top', top + 'px');

                var left = row.offset().left + leftOffset;
                if (left + popCard.width() > window.innerWidth) {
                    left = row.offset().left - popCard.width() - leftOffset;
                }

                popCard.css('left', left + 'px');

                popCard.show();

                $(document).one('click', function (e) {
                    hideOrgGeneralInfo();
                });
            };


            //设置维度的名称和值，和类型
            var setParentDimension = function () {
                var deferred = $q.defer();
                dimensionService.getDimensionById($scope.parentDimensionID).success(function (data) {
                    if (data) {
                        $scope.parentDimensionName = data.name;
                        $scope.parentDimensionType = data.dimensionType;
                        deferred.resolve(data);
                    }
                });
                return deferred.promise;
            }
     

            //点击编辑的时候，弹框编辑
            $scope.editOrgStock = function (orgID) {

                $scope.selectedOrgID = orgID;
                $scope.editStockModal = {};
                populateModal(orgID);
         
                $scope.modalService.stockModal.open();
            };

            var resetStatus = function () {
                $scope.stockGridInstance.columnOption('删除', 'visible', false);
                $scope.stockGridInstance.columnOption('showStockAmount', 'allowEditing', false);
                $scope.stockStatus = $scope.stockOptions.empty;
            };

            var populateModal = function (orgID) {
                resetEditModalData();
                $scope.stockStatus = $scope.stockOptions.empty;
                var param = {
                    currentDate: $scope.entity.stockDate,
                    orgID: orgID

                };
                getService.getStockListByOrgID(param).then(function (data) {
                    if (data) {
                        populateTabData(data, orgID);
                    }
                });
            };

            var populateTabData = function (data,orgID) {
           
                resetStatus();

                //当前记录
                data.parentList.forEach(function (item) {
                    item.id = item.id || PWC.newGuid();
                    item.effectiveDate = ackLib.validation.isEmpty(item.effectiveDate) ? '' : moment(item.effectiveDate).format(constant.date.dateFormatyyyyMMddUppper);
                    item.expiredDate = ackLib.validation.isEmpty(item.expiredDate) ? '' : moment(item.expiredDate).format(constant.date.dateFormatyyyyMMddUppper);
                    item.stockAmount = parseFloat(item.stockAmount).toFixed(constant.toFixedNumber2);
                });

                //历史记录
                data.historyParentList.forEach(function (item) {
                    item.id= item.id || PWC.newGuid();
                    item.effectiveDate = ackLib.validation.isEmpty(item.effectiveDate) ? '' : moment(item.effectiveDate).format(constant.date.dateFormatyyyyMMddUppper);
                    item.expiredDate = ackLib.validation.isEmpty(item.expiredDate) ? '' : moment(item.expiredDate).format(constant.date.dateFormatyyyyMMddUppper);
                    item.alterDate = ackLib.validation.isEmpty(item.alterDate) ? '' : moment(item.alterDate).format(constant.date.dateFormatyyyyMMddUppper);
                    item.stockAmount = parseFloat(item.stockAmount).toFixed(constant.toFixedNumber2);
                });

                var tabList = [];

                var orderIndex = 0;

                var currentNode = data.stockDto;

                var beforeText = "之前";
                var firstEntity = {
                    id: orderIndex,
                    name: currentNode.name,
                    code: currentNode.code,
                    capitalAmount: currentNode.capitalAmount.toFixed(constant.toFixedNumber2),  //注册资本
                    showCapitalAmount: currentNode.capitalAmount <= 0 ? '-' : currentNode.capitalAmount.toFixed(constant.toFixedNumber2),
                    capitalPercent: currentNode.capitalPercent, //股权比例
                    stockDataGridDataSource: data.parentList,
                    index: orderIndex,
                    atlerDate:'最新',
                    text:'最新',
                    stockStatus: $scope.stockOptions.empty,
   
                }

                tabList.push(firstEntity);

                var historyAlterDate = _.unique(_.pluck(data.historyParentList, 'alterDate'), function (item) {
                    return item;
                });
           
                historyAlterDate.forEach(function (alterDate) {
                    
                    //根据某一天来获取当天的所有数据
                    var findAlterDate = _.filter(data.historyParentList, function (item) {
                        return item.alterDate == alterDate;
                    });

                    var first = findAlterDate[0]
                    if (first) {
                        orderIndex = orderIndex + 1;
                        var entity = {
                            id: orderIndex,
                            name: firstEntity.name,
                            code: firstEntity.code,
                            capitalAmount: first.capitalAmount.toFixed(constant.toFixedNumber2),  //注册资本
                            showCapitalAmount: first.capitalAmount <= 0 ? '-' : first.capitalAmount.toFixed(constant.toFixedNumber2),
                            capitalPercent: first.capitalPercent, //股权比例
                            stockDataGridDataSource: findAlterDate,
                            index: orderIndex,
                            atlerDate: first.alterDate,
                            text: first.alterDate + beforeText,
                            stockStatus: $scope.stockOptions.empty,
                        }
                        tabList.push(entity);
                    }
                });

                $scope.tabInstance.selectedItemKeys = [firstEntity.index];
                $scope.stockTabList = tabList;

                $scope.editStockModal = firstEntity;
                populdateStockDataSource(firstEntity.stockDataGridDataSource);

                $scope.selectTab = 0;
                $scope.tabInstance.repaint();
            };

            var populdateStockDataSource = function (dataSource) {
         
                $scope.stockDataSource = dataSource;
                $scope.oldStockData = populateDataObject($scope.stockDataSource);
                $scope.stockGridInstance.refresh();
            };

            //保存股权信息
            $scope.saveStock = function () {
                ////存一个历史记录
                //// Check to make sure the cache doesn't already exist
                //stockCache.put('oldStock', $scope.stockArchitectureCache);

                var oldData = $scope.oldStockData;
                var newData = populateDataObject($scope.stockDataSource);

                if (angular.equals(oldData, newData) && $scope.stockStatus==$scope.stockOptions.empty) {
                    //nothing to do
                    //  alert('same data');
                    $scope.stockModalInstance.close();
                } else {

                    var deleteList = [];
                    var addList = [];
                    var updateList = [];
                    //旧数据在新数据中没有找到，则放到addList
                    newData.forEach(function (row) {

                        updateList.push(row);

                        var findResult = _.find(oldData, function (item) {
                            return item.id == row.id;
                        });
                        //旧数据没有找到，说明是新加的数据
                        if (!findResult) {
                            addList.push(row);

                        }
                        //else {
                        //    //如果旧数据找到了，查看是否相等,，不等的话，放到update list
                        //    if (!angular.equals(row, findResult)) {
                        //        updateList.push(row);
                        //    }
                        //}

                    });
                    //新数据在旧数据中没有找到，则放到deleteList
                    oldData.forEach(function (row) {

                        var findResult = _.find(newData, function (item) {
                            return item.id == row.id;
                        });
                        //新数据没有找到，说明是删除的数据
                        if (!findResult) {
                            deleteList.push(row);
                        }
                    });

                    var addHistory = function () {
                        var deferred = $q.defer();
                        //变更
                        if ($scope.stockStatus == $scope.stockOptions.addHistory) {
                            var stockParam = {
                                currentDate: $scope.entity.currentDate,
                                orgID: $scope.selectedOrgID
                            }
                            stockService.addHistoryStocks(stockParam).success(function (data) {
                                if (data) {
                                    deferred.resolve(data);
                                    ////刷新历史变更记录
                                    //populateModal($scope.selectedOrgID);
                                }
                            });
                        }
                            //修改
                        else if ($scope.stockStatus !== $scope.stockOptions.addHistory) {
                            deferred.resolve(true);
                            $scope.stockModalInstance.close();
                        }
                        
                        return deferred.promise;
                    };


                    //console.log('addList: ' + JSON.stringify(addList));
                    //console.log('updateList: ' + JSON.stringify(updateList));
                    //console.log('deleteList: ' + JSON.stringify(deleteList));

                    //添加历史记录
                    addHistory().then(function (data) {
                         //添加新增的记录
                        updateService.addOrgList(addList, $scope.selectedOrgID).then(function (data) {

                            updateList.forEach(function (item) {
                                item.capitalAmount = $scope.editStockModal.capitalAmount;
                            });
                            //修改记录
                            updateService.updateOrgList(updateList, $scope.selectedOrgID).then(function (data) {
                                //删除记录
                                updateService.deleteOrgList(deleteList, $scope.selectedOrgID).then(function (data) {

                                    if (data) {
                                        ackMessageBox.success(ackLib.Helper.translate("SaveSuccess"));
                                     
                                        //刷新历史变更记录
                                        populateModal($scope.selectedOrgID);
                                        //刷新
                                        orgchartService.populateOrgChart();
                                    }
                                });
                            });
                        })
                    });
                }
            };

            //删除某一个机构(jack delete)
            $scope.removeStock = function (stock) {

                var stockList = [];
                var deleteObj = { id: stock.id, name: stock.name };
                stockList.push(deleteObj);

                var confirmToDelete = $translate.instant('ConfirmToDelete');

                ackMessageBox.confirm(confirmToDelete, '').then(function (data) {
                    if (data) {

                        var tempData = $scope.stockDataSource;
                        for (var i = 0; i < tempData.length; i++) {
                            if (tempData[i].id == stock.id) {
                                tempData.splice(i, 1);
                                break;
                            }
                        }
                        $scope.stockDataSource = tempData;
                    }
                });
            };

            //添加现有机构
            $scope.$watch('addExistOrgIsUpdate', function (newValue, oldValue) {
                if (newValue) {
                    if ($scope.selectedExistOrgList && $scope.selectedExistOrgList.length > 0) {
                        //console.log(JSON.stringify($scope.selectedExistOrgList));
                        addOrgToStockList($scope.selectedExistOrgList);
                    }
                }
            });

            //修改注册资本的时候，比例要重新计算
            $scope.$watch('editStockModal.showCapitalAmount', function (newValue, oldValue) {

                if (!$scope.editStockModal || !$scope.stockDataSource) return;

                if (newValue && newValue > 0) {
                    $scope.editStockModal.capitalAmount = newValue;
                }
                else {
                    // $scope.editStockModal.showCapitalAmount = '-';

                    $scope.editStockModal.capitalAmount = 0;
                }
                
           
                $scope.stockDataSource.forEach(function (row) {

                    if (row.stockAmount > 0) {

                        row.showStockAmount = row.stockAmount;

                        if (newValue && newValue > 0) {

                            row.capitalPercent = parseFloat(row.stockAmount / newValue * 100).toFixed(constant.toFixedNumber2);
                            row.canEditPercent = false;
                        }
                        else {
                            //如果任意一个资本没有填，股权占比就允许手工填入，可以直接修改股权占比
                            row.canEditPercent = true;
                        }
                    }
                    else {
                        row.canEditPercent = true;
                        row.showStockAmount = '-';
                    }
                });


                $scope.stockGridInstance.refresh();

            });

            ////修改注册资本的时候，比例要重新计算
            $scope.$watch('stockDataSource', function (newValue, oldValue) {
                if (newValue && oldValue && newValue !== oldValue) {
   
                    for (var i = 0; i < newValue.length; i++) {
                        
                        var row = newValue[i];
                        if (!row.showStockAmount)
                        {
                            row.showStockAmount = row.stockAmount;
                        }

                        if (row.showStockAmount != '-' && row.showStockAmount > 0) {
                            row.stockAmount = row.showStockAmount;
                        }
                        else {
                            row.stockAmount = 0;
                        }

                        if (row.stockAmount > 0) {
                            if ($scope.editStockModal.capitalAmount > 0) {
                                row.capitalPercent = parseFloat(row.stockAmount / $scope.editStockModal.capitalAmount * 100).toFixed(constant.toFixedNumber4);
                                row.canEditPercent = false;
                            }
                            else {

                            }
                        }
                        else {
                            row.canEditPercent = true;
                            row.showStockAmount = '-';
                        }

                    }
                    $scope.stockGridInstance.refresh();
                }
            },true);

            var populateDataObject = function (data) {
                var arraryList = [];

                if (data) {
                    data.forEach(function (row) {
                        var newObject = {};
                        newObject.id = row.id;
                        newObject.name = row.name;
                        newObject.capitalAmount = row.capitalAmount;
                        newObject.capitalPercent = row.capitalPercent;
                        newObject.parentID = row.parentID;
                        newObject.stockAmount = row.stockAmount;
                        arraryList.push(newObject);
                    });
                }
                return arraryList;

            }

            //添加机构列表 (jack add)
            var addOrgToStockList = function (orgList) {

                var stockList = [];
                var index = 20;
                var childrenAmount = $scope.editStockModal.capitalAmount;

                var hasExist = false;

                for (var i = 0; i < orgList.length; i++) {
                    var row = orgList[i];

                    var findExist = _.find($scope.stockDataSource, function (item) {
                        return item.id == row.id;
                    });

                    if (findExist) {
                        hasExist = true;
                        continue;
                    }

                    if (!findExist) {
                        var randomNum = parseFloat(Math.random() * 10000).toFixed(constant.toFixedNumber2);
                        if (randomNum > childrenAmount) {
                            randomNum = childrenAmount;
                        }
                        var newData = {
                            id: row.id,
                            name: row.name,
                            code: row.code,
                            capitalAmount: randomNum,
                            capitalPercent: index,
                            parentID: $scope.selectedOrgID,
                            stockAmount: parseFloat(childrenAmount * index / 100).toFixed(constant.toFixedNumber2),
                        }
                        stockList.push(newData);
                        $scope.stockDataSource.push(newData);
                    }
                }

                //if (hasExist)
                //{
                //    showWarning("不能添加已经存在的机构");
                //}

                //updateService.addOrgList(stockList, $scope.selectedOrgID).then(function (data) {
                //    if (data) {
                //        populateModal($scope.selectedOrgID);
                //    }
                //});
            };

            //添加现有机
            $scope.addExistOrg = function () {
                $scope.selectedKeyItems = [];

                var selectedKeyItem = [];
                $scope.stockDataSource.forEach(function (row) {
                    selectedKeyItem.push(row.id);
                });
                $scope.selectedKeyItems = selectedKeyItem;
                $scope.addExistOrgOperateType = constant.Operation.Add;
                $scope.addExistOrgIsUpdate = false;
            }


            var initStockDatagrid = function () {

                var commonOptions = {
                    bindingOptions: {
                    },
                    selection: {
                        mode: "single"
                    },
                    columns: [],
                    loadPanel: {
                        enabled: false,
                    },
                    keyExpr: "id",
                    scrolling: {
                        mode: "virtual"
                    },
                    allowColumnResizing: true,
                    noDataText: $translate.instant('NoDataText'),
                    showColumnLines: true,
                    showRowLines: true,
                    rowAlternationEnabled: true,
                    showBorders: true,
                    sorting: {
                        mode: "none"
                    },


                };


                var commonColumns = [
                                {
                                    dataField: "name",
                                    caption: "出资人名称",
                                    alignment: 'center',
                                    allowEditing: false,
                                },
                                {
                                    dataField: "code",
                                    caption: "代码",
                                    alignment: 'center',
                                    allowEditing: false,
                                },
                                {
                                    dataField: "showStockAmount",
                                    caption: "出资额(万元)",
                                    alignment: 'center',
                                    allowEditing: true,
                                    format: { type: 'currency', precision: constant.toFixedNumber2 }
                                },
                                {
                                    dataField: "capitalPercent",
                                    caption: "股权占比(%)",
                                    alignment: 'center',
                                    allowEditing: false,
                                    format: { type: 'currency', precision: constant.toFixedNumber4 }
                                },
                                {
                                    dataField: "id",
                                    caption: $translate.instant('Delete'),
                                    alignment: 'center',
                                    width: 50,
                                    visible: $scope.stockStatus !== $scope.stockOptions.empty,
                                    allowEditing: false,
                                    cellTemplate: function (container, options) {
                                        try {
                                            var template = '<i class="material-icons highlight-tag" style="font-size:18px; cursor:pointer;">&#xE872;</i>';
                                            $(template).on('dxclick', function () {
                                                $scope.removeStock(options.row.data);
                                            }).appendTo(container);
                                        }
                                        catch (e) {
                                            $log.error(e);
                                        }
                                    }
                                }
                ];

                var options = angular.copy(commonOptions);
                var historyOptions = angular.copy(commonOptions);
                //当前的
                $scope.dataGridStockOptions = angular.extend(options, {
                    bindingOptions: {
                        dataSource: 'stockDataSource'
                    },
                    keyExpr: "id",
                    columns: commonColumns,
                    summary: {
                        totalItems: [
                            {
                                column: "showStockAmount",
                                displayFormat: "{0}",
                                valueFormat: {
                                    type: 'currency',
                                    precision: constant.toFixedNumber2
                                },
                                summaryType: "sum"
                            }, {
                                column: "capitalPercent",
                                displayFormat: "{0}",
                                valueFormat: {
                                    type: 'currency',
                                    precision: constant.toFixedNumber4
                                },
                                summaryType: "sum"
                            }, {
                                showInColumn: "name",
                                displayFormat: "合计"
                            }
                        ]
                    },

                    onInitialized: function (e) {
                        $scope.stockGridInstance = e.component;
                    },
                    editing: {
                        mode: "cell",
                        allowUpdating: true,
                        //texts:{
                        //    editRow: "编辑",
                        //    cancelRowChanges: "取消",
                        //    saveRowChanges: "确定",
                        //},
                    },
                    onCellClick: function (e) {
                        if (e.rowType == "data" && e.column.dataField == "capitalPercent") {
                            if (e.data.canEditPercent) {
                                e.isEditing = true;
                                $scope.stockGridInstance.editCell(e.rowIndex, e.columnIndex);
                                $('.dx-texteditor-input').focus();
                            }
                            else {
                                e.isEditing = false;
                            }
                        }

                    },
                });


                //历史记录
                $scope.historyDataGridStockOptions = angular.extend(historyOptions, {
                    bindingOptions: {
                        dataSource: 'historyStockDataSource'
                    },
                    keyExpr: "id",
                    columns: [
                                {
                                    dataField: "name",
                                    caption: "出资人名称",
                                    alignment: 'center',
                                },
                                {
                                    dataField: "code",
                                    caption: "代码",
                                    alignment: 'center',
                                },
                                {
                                    dataField: "stockAmount",
                                    caption: "出资额(万元)",
                                    alignment: 'center',
                                    format: { type: 'currency', precision: constant.toFixedNumber2 } 
                              
                                },
                                {
                                    dataField: "capitalPercent",
                                    caption: "股权占比(%)",
                                    alignment: 'center',
                                    format: { type: 'currency', precision: constant.toFixedNumber4 }
                                },
                                {
                                    dataField: "effectiveDate",
                                    caption: "有效起始日",
                                    alignment: 'center',
                                },

                                {
                                    dataField: "expiredDate",
                                    caption: "有效截止日",
                                    alignment: 'center',
                                },
                    ],

                    summary: {
                       
                        totalItems: [
                            {
                                column: "stockAmount",
                                displayFormat: "{0}",
                                valueFormat: {
                                    type: 'currency',
                                    precision: constant.toFixedNumber2
                                },
                                summaryType: "sum"
                            }, {
                                column: "capitalPercent",
                                displayFormat: "{0}",
                                valueFormat: {
                                    type: 'currency',
                                    precision: constant.toFixedNumber4
                                },
                                summaryType: "sum"
                            }, {
                                showInColumn: "name",
                                displayFormat: "合计",
                            }
                        ]
                    },
                    onInitialized: function (e) {
                        $scope.historyStockGridInstance = e.component;
                    },
                });
            };

            var initData = function () {
                orgchartService.populateOrgChart();
                initStockDatagrid();
         
                // $scope.modalService.stockModal.open();
            };

            var resetEditModalData = function () {
                $scope.currentNavItem = "page0";
                $scope.stockStatus = $scope.stockOptions.empty;
                $scope.historyStockDataSource = [];
                $scope.stockDataSource = [];
            
            }

            //初始化控件
            var initTextBoxContorls = function () {

                var isShowClearButton = true;
                var boxHeight = '35px';

                $scope.entityOptions = {
                    txtStockeDateOptions: {
                        bindingOptions: {
                            value: 'entity.stockDate',
                        },
                        valueChangeEvent: "change",
                        onValueChanged: function (e) {
                            var stockDate = e.value;
                            if (stockDate != e.previousValue) {
                                if (!stockDate || (stockDate && stockDate != '' && ackLib.validation.isDate(stockDate))) {
                                    //刷新
                                    orgchartService.populateOrgChart();
                                }
                            }
                        },
                        displayFormat: constant.date.dateFormatyyyyMMdd,
                        dateSerializationFormat: constant.date.dateFormatyyyyMMdd,
                        placeholder: $scope.pleaseInput,
                        invalidDateMessage: "请输入正确的日期格式(如1990/01/01)",
                        showClearButton: isShowClearButton,
                        height: boxHeight,

                    },

                    //变更日期
                    txtStockUpdateOptions: {
                        bindingOptions: {
                            value: 'entity.currentDate',
                        },
                        valueChangeEvent: "change",
                        onValueChanged: function (e) {
                            var stockDate = e.value;
           
                        },
                        displayFormat: constant.date.dateFormatyyyyMMdd,
                        dateSerializationFormat: constant.date.dateFormatyyyyMMdd,
                        placeholder: $scope.pleaseInput,
                        invalidDateMessage: "请输入正确的日期格式(如1990/01/01)",
                        showClearButton: isShowClearButton,
                        height: boxHeight,
                    }
                },

                $scope.modalStock = {
                    //修改
                    modifyStock: function () {
                        $scope.stockStatus = $scope.stockOptions.modify;
                        $scope.stockGridInstance.columnOption('删除', 'visible', true);
                        $scope.stockGridInstance.columnOption('showStockAmount', 'allowEditing', true);
                    },
                    //变更
                    addHistoryStock: function () {
                        $scope.stockStatus = $scope.stockOptions.addHistory;
                        $scope.stockGridInstance.columnOption('删除', 'visible', true);
                        $scope.stockGridInstance.columnOption('showStockAmount', 'allowEditing', true);
                    },
                    cancelChange: function () {
                        resetStatus();
                    }
                }

                $scope.stockTab = {
                    bindingOptions: {
                        dataSource: 'stockTabList',
                        selectedIndex :'selectTab'
                    },
                
                    scrollByContent: true,
                    showNavButtons: true,
                    selectedIndex: 0,         //默认选中第一个
                    //width:500,
                    onContentReady:function(e)
                    {
                        e.component.selectedItemKeys=[0];
                    },
                    onInitialized: function (e) {
                        $scope.tabInstance = e.component;
                    },
                    onSelectionChanged:function(e)
                    {
                        if (e.addedItems.length > 0 && e.addedItems[0]) {
                            var itemData = e.addedItems[0];
                            $scope.currentNavItem = 'page' + itemData.index;
                            $scope.editStockModal = itemData;
                            if (itemData.index == 0) {
                                //当前数据
                                $scope.stockDataSource = itemData.stockDataGridDataSource;
                                $scope.stockGridInstance.refresh();

                                $scope.oldStockData = populateDataObject($scope.stockDataSource);
                            }
                            else {
                                //历史数据
                                $scope.historyStockDataSource = itemData.stockDataGridDataSource;
                                $scope.historyStockGridInstance.refresh();
                            }
                        }
                    },
                    //点击行，选中
                    onItemClick: function (e) {
    

                    },
                };

            };


            //权限check
            var checkUserPermission = function () {

                var list = [];
                $scope.hasAddPermission = false;
                $scope.hasQueryPermission = false;
                $scope.hasEditPermission = false;

                $scope.hasAttributeQueryPermission = false;

                $scope.hasAccessSettingQueryPermission = false;
                $scope.hasAccessSettingEditPermission = false;

                //查看机构的权限
                var tempOrg = constant.adminPermission.infrastructure.organizationManage;
                list.push(tempOrg.addCode);
                list.push(tempOrg.queryCode);
                list.push(tempOrg.editCode);

                //自定义显示属性的权限
                var tempAttr = constant.adminPermission.infrastructure.customAttribute;
                list.push(tempAttr.queryCode);

                //权限设置的权限
                var tempAcess = constant.adminPermission.infrastructure.accessSetting;
                list.push(tempAcess.queryCode);
                list.push(tempAcess.editCode);

                $scope.hasDimensionValueEditPermission = false;
                //编辑自定义维度值的时候的权限
                if ($scope.parentDimensionType == constant.dimensionType.BusinessUnit) {
                    var tempDV = constant.adminPermission.basicData.businessUnit;
                    list.push(tempDV.editCode);
                }
                else {
                    //自定义维度的权限
                    var tempDV = constant.adminPermission.basicData.selfDimension;
                    list.push(tempDV.editCode);
                }

                $scope.$root.checkUserPermissionList(list).success(function (data) {
                    // console.log('data: ' + JSON.stringify(data));
                    $scope.hasAddPermission = data[tempOrg.addCode];
                    $scope.hasQueryPermission = data[tempOrg.queryCode];
                    $scope.hasEditPermission = data[tempOrg.editCode];

                    $scope.hasAttributeQueryPermission = data[tempAttr.queryCode];

                    $scope.hasAccessSettingQueryPermission = data[tempAcess.queryCode];
                    $scope.hasAccessSettingEditPermission = data[tempAcess.editCode];

                    $scope.hasDimensionValueEditPermission = data[tempDV.editCode];

                    if ($scope.hasQueryPermission) {
                        initOrgChartData();
                    }
                });
            };


            (function initialize() {
                $log.debug('storeArchitectureViewController.ctor()...');
                $log.debug($state.params.dimensionID);
                var dimensionID = 'c61a5bd6-a996-4952-9869-d053995237e5'; //集团为事业部
                //  $scope.parentDimensionID = $state.params.dimensionID || 'c61a5bd6-a996-4952-9869-d053995237e8';
                var parentDimensionID = $state.params.dimensionID || dimensionID;
                $scope.parentDimensionID = parentDimensionID;
                $scope.showAttributeDimensionID = $scope.parentDimensionID;
                $scope.canEditPercent = false;
                initTextBoxContorls();
                initData();
                //事业部和自定义维度的权限
                setParentDimension().then(function (data) {
                    //checkUserPermission();
                });

               // $scope.editOrgStock("231c835b-169e-4963-b6f5-1601c0795fa6");
            })();
        }
    ]);