﻿citModule.controller('citReportAnalysisDashboardConfigController', ['$scope', '$log', '$translate', '$timeout', '$q', '$interval',
    'apiInterceptor', 'Upload', 'vatImportService', 'SweetAlert', 'uiGridConstants', 'citSessionService', 'enums', 'vatOperationLogService',
    'projectService', 'vatCommonService', 'vatDashboardService', '$stateParams', '$state', 'indexAnalysisDetailService',
function ($scope, $log, $translate, $timeout, $q, $interval, apiInterceptor, Upload, vatImportService, SweetAlert, uiGridConstants,
    citSessionService, enums, vatOperationLogService, projectService, vatCommonService, vatDashboardService, $stateParams, $state,
    indexAnalysisDetailService) {
    'use strict';

    $scope.config = {
        id: '',
        chartName: '',
        modelItems: [],
        analysisItems: [],
        isShowData: 0,
        isShowGrid:0
    };

    //取当前的dashboard机构。
    $scope.selectedOrgId = citSessionService.dashboardSingelOrganizationId;

    $scope.chartRenderOption = {
        id: '',
        name: $scope.config.chartName,
        isShowData: $scope.config.isShowData,
        header: { v1: $translate.instant('OrgName') },
        headerObj: [{ name: $translate.instant('OrgName'), dataType: 1, chartType: 'line' }],
        data: []
    };

    //图表名称text box option
    $scope.chartNameOption = {
        width: 280,
        placeholder: $translate.instant('PleaseInputChartName'),
    }

    var year = (new Date()).getFullYear();
    var yearList = [{ id: year, name: year }, { id: year - 1, name: year - 1 }, { id: year - 2, name: year - 2 }, { id: year - 3, name: year - 3 }];

    //年份筛选select box option
    $scope.yearTreeView = null;
    var initalYearTree = function () {
        if ($scope.yearTreeView) {
            $scope.fromSyncSelection = true;
            if (!$scope.selectedYears || $scope.selectedYears.length === 0) {
                $scope.yearTreeView.unselectAll();
            }
            _.each($scope.selectedYears, function (year) {
                $scope.yearTreeView.selectItem(year);
            });

            _.each(yearList, function (year) {
                if (!_.contains($scope.selectedYears, year.id) && !_.contains($scope.selectedYears, year.id + '')) {
                    $scope.yearTreeView.unselectItem(year.id);
                }
            });
            $scope.fromSyncSelection = false;
        }
    };

    $scope.selectedYears = [year];
    $scope.yearFilterOptions = {
        bindingOptions: {
            value: 'selectedYears'
        },
        valueExpr: "id",
        displayExpr: "name",
        placeholder: "",
        showClearButton: false,
        dataSource: yearList,
        treeView: {
            dataSource: yearList,
            dataStructure: "plain",
            keyExpr: "id",
            displayExpr: "name",
            selectByClick: true,
            selectNodesRecursive: false,
            showCheckBoxesMode: "normal",
            selectionMode: "multiple",
            onContentReady: function (e) {
                $scope.yearTreeView = e.component;
                initalYearTree();
            },
            onItemSelectionChanged: function (args) {
                if (!$scope.fromSyncSelection) {
                    var value = args.component.getSelectedNodesKeys();
                    value = _.sortBy(value, function (x) { return x; });
                    $scope.selectedYears = value;
                }

            }
        }
    };

    //分析维度select box option
    $scope.selectedDimensionType = 1;
    $scope.dimensionOptions = {
        width: 280,
        bindingOptions: {
            value: 'selectedDimensionType'
        },
        noDataText: '',
        placeholder: $translate.instant('PleaseSelect'),
        items: [{ id: 1, name: $translate.instant('c_month') }, { id: 2, name: $translate.instant('ProjectYearCol') }],
        valueExpr: 'id',
        displayExpr: 'name',
        onItemClick: function (e) {
            if (e) {
                if ($scope.config.analysisItems.length > 1 && $scope.selectedYears.length > 1 && e.itemData.id == 1) {
                    SweetAlert.warning($translate.instant('DimensionalityCheckInfo'));
                    e.component.option('value', 2);
                    e.component.option('text', $translate.instant('ProjectYearCol'));
                    return;
                }
            }
        }
    };

    //获取指定机构的二级分析指标数据信息
    $scope.organizationSubModelDataList = [];
    var getOrganizationSubModelList = function (orgId) {
        vatDashboardService.getOrganizationSubModelList(orgId).success(function (result) {
            if (result && result.data && result.data.length > 0) {
                $scope.organizationSubModelDataList = result.data;
            }
        });
    };

    //添加分析指标下拉树控件
    $scope.selectIdList = [];
    var syncTreeViewSelection = function () {
        if ($scope.treeView) {
            $scope.fromSyncSelection = true;
            if (!$scope.selectIdList || $scope.selectIdList.length === 0) {
                $scope.treeView.unselectAll();
                $scope.fromSyncSelection = false;
                return;
            }

            $scope.selectIdList.forEach(function (key) {
                $scope.treeView.selectItem(key);
            });

            _.each($scope.indexList, function (item) {
                if (!_.contains($scope.selectIdList, item.id) && item.parentId) {
                    $scope.treeView.unselectItem(item.id);
                }
            });

            $scope.fromSyncSelection = false;
        }
    };

    $scope.indexList = [];
    $scope.treeView = null;

    $scope.multiSelectAnalysisOptionHeight = 300;
    $scope.multiSelectAnalysisOption = {
        placeholder: $translate.instant('PleaseSelectTarget'),
        width: 280,
        valueExpr: "name",
        displayExpr: "name",
        showClearButton: true,
        bindingOptions: {
            value: 'selectNameList',
            dataSource: 'indexList',
        },
        onValueChanged: function (args) {
            if (!args.value) {
                $scope.selectIdList = [];
            }
        },
        treeView: {
            bindingOptions: {
                dataSource: 'indexList',
                searchValue: 'searchText',
                height: 'multiSelectAnalysisOptionHeight'
            },
            dataStructure: "plain",
            keyExpr: "id",
            parentIdExpr: "parentId",
            selectionMode: "multiple",
            displayExpr: "name",
            selectByClick: true,
            selectNodesRecursive: true,
            showCheckBoxesMode: "normal",
            onContentReady: function (e) {
                $scope.treeView = e.component;
            },
            onItemSelectionChanged: function (args) {
                if (!$scope.fromSyncSelection) {
                    var value = args.component.getSelectedNodesKeys();
                    if (value.length > 1 && $scope.selectedYears.length > 1 && $scope.dimension == 1) {
                        SweetAlert.warning($translate.instant('MultiYearCheckInfo'));
                        syncTreeViewSelection();
                        return;
                    }
                    $scope.selectIdList = value;
                    $scope.selectNameList = _.filter($scope.indexList, function (nameItem) {
                        return _.contains($scope.selectIdList, nameItem.id) && nameItem.parentId;
                    });
                    $scope.riskDataSource = [];
                    _.each($scope.selectNameList, function (nameItem) {
                        var parentNode = _.where($scope.indexList, { id: nameItem.parentId })[0];
                        if (_.where($scope.riskDataSource, { id: parentNode.id }).length < 1) {
                            if (_.contains($scope.selectRiskIdList, parentNode.id)) {
                                $scope.riskDataSource.push({ id: parentNode.id, name: parentNode.name, selected: true });
                            } else {
                                $scope.riskDataSource.push({ id: parentNode.id, name: parentNode.name, selected: false });
                            }
                        }
                    });

                    $scope.selectNameList = _.map($scope.selectNameList, function (item) {
                        return item.name;
                    });
                }
            }
        }
    };

    $scope.riskDataSource = [];
    $scope.selectRiskNameList = [];
    $scope.selectRiskIdList = [];
    //风险提示Options
    $scope.riskTipMultiSelectAnalysisOption = {
        width: 280,
        valueExpr: "id",
        displayExpr: "name",
        showClearButton: true,
        contentTemplate: 'riskContent',
        bindingOptions: {
            dataSource: 'riskDataSource',
            value: 'selectRiskIdList'
            
        },
        onValueChanged: function (args) {
            if (!args.value) {
                $scope.selectRiskIdList = [];
            }
        },
        treeView: {
            bindingOptions: {
                dataSource: 'riskDataSource'
            },
            height: 300,
            dataStructure: "plain",
            keyExpr: "id",
            selectionMode: "multiple",
            displayExpr: "name",
            selectByClick: true,
            selectNodesRecursive: true,
            showCheckBoxesMode: "normal",
            onContentReady: function (e) {
                var riskTree = $('#riskTipTree').dxTreeView('instance');
                _.each($scope.riskDataSource, function (nameItem) {
                    if (_.contains($scope.selectRiskIdList, nameItem.id)) {
                        riskTree.selectItem(nameItem.id);
                    } else {
                        riskTree.unselectItem(nameItem.id);
                    }
                    return nameItem;
                });
            },
            onItemSelectionChanged: function (args) {
                var value = args.component.getSelectedNodesKeys();
                $scope.selectRiskIdList = value;
                $scope.selectRiskNameList = _.filter($scope.riskDataSource, function (nameItem) {
                    return _.contains($scope.selectRiskIdList, nameItem.id);
                });
                $scope.selectRiskNameList = _.map($scope.selectRiskNameList, function (item) {
                    return item.name;
                });
            }
        }
    };

    $scope.$watch('selectRiskIdList', function () {
        var riskTree = $('#riskTipTree').dxTreeView('instance');
        if (riskTree) {
            _.each($scope.riskDataSource, function (nameItem) {
                if (_.contains($scope.selectRiskIdList, nameItem.id)) {
                    riskTree.selectItem(nameItem.id);
                } else {
                    riskTree.unselectItem(nameItem.id);
                }
                return nameItem;
            });
        }

    });

    //删除分析指标
    var removeAnalysisItem = function (item) {
        $scope.config.analysisItems = _.filter($scope.config.analysisItems, function (analysisItem) {
            return analysisItem.id != item.id;
        });
        $scope.selectIdList = _.filter($scope.selectIdList, function (indexItem) {
            return indexItem != item.id;
        });
    };

    var checkConfigValible = function () {
        //多年月份多指标未不合法配置。
        if ($scope.selectedYears.length > 1 && $scope.dimension === 1 && $scope.config.analysisItems.length > 1) {
            return false;
        }
        return true;
    }

    //保存指标配置
    var saveDashboardConfig = function () {
        if (!checkConfigValible()) {
            SweetAlert.warning($translate.instant('CheckOrganizationDashboardConfig'));
            return;
        }
        if ($scope.config.analysisItems.length < 1) {
            SweetAlert.warning($translate.instant('DeleteMeasureAlert'));
            return;
        }
        if (!$scope.config.chartName) {
            SweetAlert.warning($translate.instant('checkChartName'));
            return;
        }
        if ($scope.selectedYears.length < 1) {
            SweetAlert.warning($translate.instant('checkChartYears'));
            return;
        }
        var userSetting = '';
        var indexIds = '';
        var indexIdArr = [];
        var userSettingArr = [{ name: 'OrgName', dataType: 1, chartType: 'line' }];

        _.each($scope.config.analysisItems, function (item) {
            userSettingArr.push({ name: item.analysisName, dataType: item.dataType, chartType: item.chartType });
            indexIdArr.push(item.id);
        });
        userSetting = JSON.stringify(userSettingArr);
        indexIds = indexIdArr.join(',');

        var data = {};
        data.ID = $scope.config.id;
        data.Name = $scope.config.chartName;
        data.IndexIDs = indexIds;
        data.UserSetting = userSetting;
        data.IsShowData = $scope.config.isShowData;
        data.OrderIndex = 4;
        data.riskTip = $scope.selectRiskIdList.join(',');
        data.isShowGrid = $scope.config.isShowGrid;

        data.Years = $scope.selectedYears.join(',');
        data.DimensionType = $scope.dimension; //1月份，2年份

        vatDashboardService.saveOrganizationConfig(data).success(function () {
            if (citSessionService.urlFromAnalyze) {
                $state.go('citAnalyzeOrganizeDashboard', { id: citSessionService.dashboardSingelOrganizationId });
            } else {
                $state.go('citOrganizationDashboard', { id: citSessionService.dashboardSingelOrganizationId });
            }
        });
    };

    //取消保存
    var cancelConfig = function () {
        if (citSessionService.urlFromAnalyze) {
            $state.go('citAnalyzeOrganizeDashboard', { id: citSessionService.dashboardSingelOrganizationId });
        } else {
            $state.go('citOrganizationDashboard', { id: citSessionService.dashboardSingelOrganizationId });
        }
    };

    //删除图表
    //var deleteChartConfig = function () {
    //    SweetAlert.swal({
    //        title: $translate.instant('ConfirmDelete'),
    //        text: $translate.instant('deleteChart') + "?",
    //        type: "warning",
    //        showCancelButton: true,
    //        confirmButtonColor: "#DD6B55",
    //        confirmButtonText: $translate.instant('Confirm'),
    //        cancelButtonText: $translate.instant('Cancel'),
    //        closeOnConfirm: true
    //    }, function (isConfirm) {
    //        if (isConfirm) {
    //            vatDashboardService.deleteConfigById($scope.config.id).success(function () {
    //                SweetAlert.success($translate.instant('DeleteSuccess'));
    //                if (citSessionService.urlFromAnalyze) {
    //                    $state.go('analyzeOrganizeDashboard', { id: citSessionService.dashboardSingelOrganizationId });
    //                } else {
    //                    $state.go('organizationDashboard', { id: citSessionService.dashboardSingelOrganizationId });
    //                }
    //            });
    //        }
    //    });
    //};

    $scope.chartChanged = true;
    var setAnalysisItems = function () {

        $scope.config.analysisItems = _.filter($scope.config.analysisItems, function (item) {
            return _.contains($scope.selectIdList, item.id);
        });

        _.each($scope.selectIdList, function (selectIndexItem) {
            _.each($scope.organizationSubModelDataList, function (organizItem) {
                if (selectIndexItem === organizItem.id && _.where($scope.config.analysisItems, { id: selectIndexItem }).length === 0) {
                    //添加分析指标
                    var obj = new Object();
                    obj.id = organizItem.id;
                    obj.analysisName = organizItem.name;
                    obj.dataType = organizItem.dataType;
                    obj.chartType = 'bar';
                    obj.chartOptions = {
                        items: [
                            { typeName: $translate.instant("barchart"), typeValue: 'bar' },
                            { typeName: $translate.instant("linechart"), typeValue: 'line' },
                            { typeName: $translate.instant("areachart"), typeValue: 'area' },
                            { typeName: $translate.instant("stepchart"), typeValue: 'step' }
                        ],
                        displayExpr: 'typeName',
                        valueExpr: 'typeValue',
                        itemTemplate: 'chartTypeItemTemplate',
                        value: 'bar',
                        onItemClick: function (e) {
                            obj.chartType = e.itemData.typeValue;
                            $scope.chartChanged = !$scope.chartChanged;
                        }
                    };

                    if (!_.isEmpty($scope.indexList)) {
                        var indexItemObj = _.find($scope.indexList, function (indexItem) {
                            return indexItem.id == selectIndexItem;
                        });

                        if (!_.isEmpty(indexItemObj)) {
                            obj.index = indexItemObj.orderIndex;
                        }
                    }

                    $scope.config.analysisItems.push(obj);
                }
            })
        });

        //对分析指标排序，使分析指标顺序与指标列表中一致。
        $scope.config.analysisItems = _.sortBy($scope.config.analysisItems, 'index');
    };

    //设置分析指标下拉框的显示值
    $scope.$watchGroup(['selectIdList', 'indexList', 'treeView'], function (newItems, oldItems) {
        if (newItems[1].length > 0) {
            $scope.selectNameList = _.filter($scope.indexList, function (nameItem) {
                return _.contains($scope.selectIdList, nameItem.id) && nameItem.parentId;
            });
            $scope.riskDataSource = [];
            _.each($scope.selectNameList, function (nameItem) {
                var parentNode = _.where($scope.indexList, { id: nameItem.parentId })[0];
                if (_.where($scope.riskDataSource, { id: parentNode.id }).length < 1) {
                    if (_.contains($scope.selectRiskIdList, parentNode.id)) {
                        $scope.riskDataSource.push({ id: parentNode.id, name: parentNode.name, selected: true });
                    } else {
                        $scope.riskDataSource.push({ id: parentNode.id, name: parentNode.name, selected: false });
                    }
                }
            });

            $scope.selectNameList = _.map($scope.selectNameList, function (item) {
                return item.name;
            });
        }
        syncTreeViewSelection();

    });

    //分析指标根据选中的指标变化
    $scope.$watchGroup(['selectIdList', 'organizationSubModelDataList', 'indexList'], function () {
        setAnalysisItems();
    });

    $scope.$watch('selectedYears', function (newValue, oldValue) {
        if ($scope.config.analysisItems.length > 1 && $scope.dimension == 1 && $scope.selectedYears.length > 1) {
            SweetAlert.warning($translate.instant('MultilTargetCheckInfo'));
            $scope.selectedYears = oldValue;
            initalYearTree();
            return;
        }
    });

    //显示图标提示信息
    var showChartTipInfo = function () {
        $timeout(function () {
            if ($scope.chartRenderOption.isNoChart) {
                $('.row-block-left').children('div:eq(0)').children('div:eq(0)').css({ 'top': '45%', 'width': '400px', 'height': 'auto', 'left': '42%', 'font-size': '25px', 'color': '#777' }).html($translate.instant('PleaseAddMeasuresRight'));
            }
        }, 1000);
    };

    //分析指标配置变动联动图表
    $scope.$watchGroup(['config.analysisItems', 'config.isShowData', 'dimension', 'selectedYears', 'chartChanged', 'organizationSubModelDataList'], function (newItems, oldItems) {
        if (newItems[0].length > 0 && newItems[2] > 0 && newItems[3].length > 0 && newItems[5].length > 0) {
            ////数据合法性检查
            //if (newItems[0].length != oldItems[0].length) {
            //    if ($scope.selectedYears.length > 1 && $scope.dimension == 1 && newItems[0] > 1) {
            //        SweetAlert.warning($translate.instant('当前已选择多个年份，只允许选择1个指标'));
            //        config.analysisItems = oldItems[0];
            //        return;
            //    }
            //} else if (newItems[3].length != oldItems[3].length) {
            //    if ($scope.config.analysisItems.length > 1 && $scope.dimension == 1 && newItems[3].length > 1) {
            //        SweetAlert.warning($translate.instant('当前已选择多个指标，只允许选择1个年份'));
            //        $scope.selectedYears = oldItems[3];
            //        return;
            //    }
            //} else if (newItems[2] != oldItems[2]) {
            //    if ($scope.config.analysisItems.length > 1 && $scope.selectedYears.length > 1 && newItems[2] == 1) {
            //        SweetAlert.warning($translate.instant('当前已选择多个指标、多个年份，只允许选择年份维度'));
            //        $scope.dimension = oldItems[2];
            //        return;
            //    }
            //}

            var obj = new Object();
            obj.id = $scope.config.id;
            obj.name = $scope.config.chartName;
            obj.isShowData = $scope.config.isShowData;
            obj.header = {};
            obj.data = [];

            if ($scope.selectedYears.length > 1) {//多年
                if ($scope.dimension === 1 && $scope.config.analysisItems.length === 1) {//月份|单指标
                    var analysisItem = $scope.config.analysisItems[0];
                    obj.header["v1"] = $translate.instant('c_month');
                    obj.headerObj = [{ name: $translate.instant('c_month'), dataType: analysisItem.dataType, chartType: analysisItem.chartType }];

                    _.each($scope.selectedYears, function (item, index) {
                        obj.header["v" + (index + 2)] = item + '年';
                        obj.headerObj.push({ name: item + '年', dataType: analysisItem.dataType, chartType: analysisItem.chartType });
                    });

                    for (var i = 1; i < 13; i++) {
                        var mounthModelObj = { v0: '', v1: i };
                        if ($scope.config.analysisItems.length > 0 && $scope.organizationSubModelDataList.length > 0) {
                            _.each($scope.selectedYears, function (year, index) {
                                var flag = true;
                                _.each($scope.organizationSubModelDataList, function (itemData) {
                                    if (itemData.year == year && $scope.config.analysisItems[0].id === itemData.id && i === itemData.period) {
                                        mounthModelObj['v' + (index + 2)] = itemData.data;
                                        flag = false;
                                    }
                                });
                                if (flag) {
                                    mounthModelObj['v' + (index + 2)] = 0;
                                }
                            });
                        }
                        obj.data.push(mounthModelObj);
                    }
                } else if ($scope.dimension === 2) {//年份|多指标
                    obj.header["v1"] = $translate.instant('ProjectYearCol');
                    obj.headerObj = [{ name: $translate.instant('ProjectYearCol'), dataType: 1, chartType: 'line' }];

                    _.each($scope.config.analysisItems, function (value, index) {
                        obj.header['v' + (index + 2)] = value.analysisName;
                        obj.headerObj.push({ name: value.analysisName, dataType: value.dataType, chartType: value.chartType });
                    });

                    _.each($scope.selectedYears, function (year) {
                        var yearModelObj = { v0: '', v1: year + '年' };
                        if ($scope.config.analysisItems.length > 0 && $scope.organizationSubModelDataList.length > 0) {
                            _.each($scope.config.analysisItems, function (value, index) {
                                var total = 0;
                                _.each($scope.organizationSubModelDataList, function (itemData) {
                                    if (itemData.year == year && itemData.period == 13 && value.id === itemData.id && !isNaN(itemData.data)) {
                                        total += parseFloat(itemData.data);
                                    }
                                });
                                yearModelObj['v' + (index + 2)] = total;
                            });
                        }
                        obj.data.push(yearModelObj);
                    });
                }
            } else {//单年
                if ($scope.dimension === 1) {//月份|多指标
                    obj.header["v1"] = $translate.instant('c_month');
                    obj.headerObj = [{ name: $translate.instant('c_month'), dataType: 1, chartType: 'line' }];

                    _.each($scope.config.analysisItems, function (value, index) {
                        obj.header['v' + (index + 2)] = value.analysisName;
                        obj.headerObj.push({ name: value.analysisName, dataType: value.dataType, chartType: value.chartType });
                    });

                    for (var i = 1; i < 13; i++) {
                        var mounthModelObj = { v0: '', v1: i };
                        if ($scope.config.analysisItems.length > 0 && $scope.organizationSubModelDataList.length > 0) {
                            _.each($scope.config.analysisItems, function (value, index) {
                                var flag = true;
                                _.each($scope.organizationSubModelDataList, function (itemData) {
                                    if (itemData.year == $scope.selectedYears[0] && value.id === itemData.id && i === itemData.period) {
                                        mounthModelObj['v' + (index + 2)] = itemData.data;
                                        flag = false;
                                    }
                                });
                                if (flag) {
                                    mounthModelObj['v' + (index + 2)] = 0;
                                }
                            });
                        }
                        obj.data.push(mounthModelObj);
                    }

                } else {//年份|多指标
                    obj.header["v1"] = $translate.instant('ProjectYearCol');
                    obj.headerObj = [{ name: $translate.instant('ProjectYearCol'), dataType: 1, chartType: 'line' }];

                    _.each($scope.config.analysisItems, function (value, index) {
                        obj.header['v' + (index + 2)] = value.analysisName;
                        obj.headerObj.push({ name: value.analysisName, dataType: value.dataType, chartType: value.chartType });
                    });

                    _.each($scope.selectedYears, function (year) {
                        var yearModelObj = { v0: '', v1: year + '年' };
                        if ($scope.config.analysisItems.length > 0 && $scope.organizationSubModelDataList.length > 0) {
                            _.each($scope.config.analysisItems, function (value, index) {
                                var total = 0;
                                _.each($scope.organizationSubModelDataList, function (itemData) {
                                    if (itemData.year == year && itemData.period == 13 && value.id === itemData.id && !isNaN(itemData.data)) {
                                        total += parseFloat(itemData.data);
                                    }
                                });
                                yearModelObj['v' + (index + 2)] = total;
                            });
                        }
                        obj.data.push(yearModelObj);
                    });
                }
            }
            if (obj.data && obj.data.length > 0) {
            obj.isNoChart = false;
            $scope.chartRenderOption = obj;
        } else {
                obj.isNoChart = true;
                showChartTipInfo();
            }
        } else {
            $scope.chartRenderOption = { isNoChart: true };
            showChartTipInfo();
        }

    });

    (function initialize() {
        //获取机构的所有指标信息
        indexAnalysisDetailService.getIndexAnalysisDropDownList().success(function (data) {
            var dataOrder = 0;
            data = _.map(data, function (item) {
                item.orderIndex = dataOrder;
                dataOrder++;
                return item;
            });
            $scope.indexList = data;

            $scope.multiSelectAnalysisOptionHeight = 300;
            if ($scope.indexList && $scope.indexList.length <= 15) {
                $scope.multiSelectAnalysisOptionHeight = 20 * $scope.indexList.length;
            }
        });

        getOrganizationSubModelList($scope.selectedOrgId);

        $scope.removeAnalysisItem = removeAnalysisItem;
        $scope.saveDashboardConfig = saveDashboardConfig;
        $scope.cancelConfig = cancelConfig;
        //$scope.deleteChartConfig = deleteChartConfig;

        if ($stateParams.id) {
            $scope.config.id = $stateParams.id;
            //获取指标汇总配置
            vatDashboardService.getOrganizationConfig($scope.config.id).success(function (result) {
                if (result && result.data) {
                    var selectedAnalysisItems = JSON.parse(result.data.userSetting);
                    selectedAnalysisItems.splice(0, 1);
                    var ids = result.data.indexIDs.split(',');
                    $scope.selectedIndexIds = result.data.indexIDs;
                    _.each(ids, function (id, index) {
                        //添加分析指标
                        var obj = new Object();
                        obj.id = id;
                        obj.analysisName = selectedAnalysisItems[index].name;
                        obj.dataType = selectedAnalysisItems[index].dataType;
                        obj.chartType = selectedAnalysisItems[index].chartType;
                        obj.chartOptions = {
                            items: [
                                { typeName: $translate.instant("barchart"), typeValue: 'bar' },
                                { typeName: $translate.instant("linechart"), typeValue: 'line' },
                                { typeName: $translate.instant("areachart"), typeValue: 'area' },
                                { typeName: $translate.instant("stepchart"), typeValue: 'step' }
                            ],
                            displayExpr: 'typeName',
                            valueExpr: 'typeValue',
                            itemTemplate: 'chartTypeItemTemplate',
                            value: obj.chartType,
                            onItemClick: function (e) {
                                obj.chartType = e.itemData.typeValue;
                                $scope.chartChanged = {};
                            }
                        };
                        $scope.config.analysisItems.push(obj);
                    });

                    if (result.data.indexIDs && ids.length > 0) {
                        $scope.selectIdList = ids;
                    }

                    var riskTipIds = result.data.riskTip ? result.data.riskTip.split(',') : null;
                    if (riskTipIds && riskTipIds.length > 0) {
                        $scope.selectRiskIdList = riskTipIds;
                    }

                    $scope.config.chartName = result.data.name;
                    $scope.config.isShowData = result.data.isShowData;
                    $scope.config.isShowGrid = result.data.isShowGrid ? result.data.isShowGrid : 0;

                    $scope.selectedDimensionType = result.data.dimensionType;
                    $scope.dimension = result.data.dimensionType;
                    $scope.selectedYears = result.data.years.split(',');
                    $timeout(function () {
                        $('#year-filter').dxDropDownBox('instance').field().val($scope.selectedYears.join(', '));
                    });
                }
            });
        }
    })();
}
]);