﻿// key value select tree view, select key value pair(s) with checkbox(es)
commonModule.directive('keyValueSelector', ['$log', '$q', '$timeout', '$translate', 'KeyValueConfigService',
    function ($log, $q, $timeout, $translate, KeyValueConfigService) {
        'use strict';
        $log.debug('keyValueSelector.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/common/controls/key-value-selector/key-value-selector.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                isShow: '=',
                title: '=',
                selectorOptions: '=?',
                selectorApi: '=?', // In: onSelectorClose, onSelectorConfirm; Out: getSelection
                selectedKeyCode: '='
            },
            link: function (scope, element, attrs) {
                $log.debug('keyValueSelector.link()...');

                var newSelectionIndicies = [];
                var gridInstance = null;
                scope.keyValueDataSource = [];

                // Handle id of the element to make sure it is unique.
                // keyValueSelector is used to add the style tag to work-around
                // DX grid height issue
                if (_.isEmpty(attrs['id'])) {
                    var randomStr = Math.random().toString().replace(/\./g, '');
                    element.attr('id', 'grid' + randomStr);
                    scope.gridSelector = '#grid' + randomStr;
                }
                else {
                    scope.gridSelector = '#' + attrs['id'];
                }

                // Register api functions in internalApi
                // If selectorApi passed is not undefined, we will use it as internalApi
                scope.internalApi = scope.selectorApi || {};

                // Work-around for DX grid height issue.
                element.find('.modal').on('shown.bs.modal', function () {
                    scope.getDataGrid().refresh();
                });

                // Clear selected key value pairs and related rows. 
                var resetSelection = function () {
                    scope.getDataGrid().clearSelection();

                };

                var scrollToRowElement = function (gridInstance) {
                    if (newSelectionIndicies.length > 0) {
                        var scrollable = gridInstance.getScrollable();
                        var selectedRowElements = gridInstance.getCellElement(newSelectionIndicies[0], 0);
                        $timeout(function () {
                            scrollable.scrollToElement(selectedRowElements.parent());
                        });
                        newSelectionIndicies = [];
                    }
                };

                var selectRow = function (gridInstance, selectedObj) {
                    var dataSourceInstance = $.extend({}, gridInstance.getDataSource());
                    dataSourceInstance.paginate(false);
                    var absoluteRowIndex = 0;
                    dataSourceInstance.load().done(function (result) {
                        absoluteRowIndex = _.findIndex(result, function (item) {
                            return angular.equals(item, selectedObj);
                        });
                        if (absoluteRowIndex >= 0) {
                            var pgSize = gridInstance.pageSize();
                            var pageIndex = Math.floor(absoluteRowIndex / pgSize);
                            var visibleRowIndex = absoluteRowIndex - (pageIndex * pgSize);
                            newSelectionIndicies = [visibleRowIndex];
                            gridInstance.beginUpdate();
                            var key = gridInstance.keyOf(result[absoluteRowIndex]);
                            gridInstance.selectRows([key], true);

                            if (pageIndex !== gridInstance.pageIndex()) {
                                gridInstance.pageIndex(pageIndex);
                            } else {
                                //当前页内容发生改变时才会触发 onContentReady 事件,所以当选择的焦点行就在当前显示页的时候,需要主动执行一次 scrollToRowElement
                                scrollToRowElement(gridInstance);
                            }
                            gridInstance.endUpdate();
                        }
                    }).fail(function (error) {
                        console.log(error);
                    });
                };
                scope.gridOptions = {
                    bindingOptions: {
                        dataSource: {
                            deep: false,
                            dataPath: 'keyValueDataSource'
                        },
                        height: 'selectorOptions.gridHeight',
                        width: 'selectorOptions.gridWidth',
                        'paging.pageSize': 'selectorOptions.pageSize'
                    },
                    columns: [
                        {
                            dataField: 'id',
                            visible: false
                        }, {
                            caption: $translate.instant('KeyName'),
                            dataField: 'name'
                        }, {
                            caption: $translate.instant('DataSourceName'),
                            dataField: 'dataSourceStr'
                        }, {
                            caption: $translate.instant('Formula'),
                            dataField: 'formula'
                        }, {
                            caption: $translate.instant('Description'),
                            dataField: 'description'
                        }, {
                            caption: $translate.instant('ApplicationScope'),
                            calculateCellValue: function (rowData) {
                                return $translate.instant(rowData.scopeSummary);
                            },
                            calculateFilterExpression: function (filterValue, selectedFilterOperation) {
                                return [this.calculateCellValue, selectedFilterOperation || 'contains', filterValue];
                            }
                        }, {
                            caption: $translate.instant('ApplyService'),
                            dataField: 'serviceTypes'
                        }
                    ],
                    loadPanel: {
                        enabled: false
                    },
                    pager: {
                        showNavigationButtons: true
                    },
                    scrolling: {
                        mode: "standard"
                    },
                    searchPanel: {
                        visible: true,
                        width: 240
                    },
                    selection: {
                        mode: 'single',
                        deferred: false
                    },
                    sorting: {
                        mode: 'single'
                    },
                    filterRow: {
                        visible: true
                    },
                    allowColumnResizing: true,
                    hoverStateEnabled: true,
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true,
                    onContentReady: function (e) {
                        scrollToRowElement(e.component);
                        gridInstance = e.component;
                    }
                };

                var loadData = function () {
                    return KeyValueConfigService.getAll().then(function (res) {
                        if (res.data && res.data.length > 0) {
                            res.data.forEach(function (item) {
                                if (item.dataSource) {
                                    var arr = item.dataSource.split('_');
                                    if (arr.length >= 2) {
                                        item.dataSourceStr = arr[1];
                                    }
                                }
                            });
                        }
                        scope.keyValueDataSource = res.data;
                        return $q.when(scope.keyValueDataSource);
                    }, function (response) {
                        return $q.reject($translate.instant("DataLoadingError"));
                    });
                };

                // Adjust the selector options
                var adjustOptions = function () {

                    // Adjust the CSS styles of the selector, currently only support width, height, top and left.

                    var width = 730;

                    var computeWidth = parseInt(window.window.innerWidth * 0.7);

                    if (computeWidth > width && computeWidth < 1500) {
                        width = computeWidth;
                    }


                    scope.selectorOptions = _.extend({
                        // Default options
                        height: 556,
                        width: width,
                        pageSize: 7
                    }, scope.selectorOptions);
                    if (_.isNumber(scope.selectorOptions.height)) {
                        scope.selectorOptions.gridHeight = scope.selectorOptions.height > 145
                            ? (scope.selectorOptions.height - 145) : 1;
                    }

                    if (_.isNumber(scope.selectorOptions.width)) {
                        scope.selectorOptions.gridWidth = scope.selectorOptions.width - 30;
                    }

                    scope.cssOptions = _.pick(scope.selectorOptions, 'width', 'height', 'top', 'left');
                };

                var getSelectionRowData = function () {
                    var selectedRows = gridInstance.getSelectedRowsData();
                    return selectedRows.length ? selectedRows[0] : null;
                };

                scope.getDataGrid = function () {
                    return element.find(".key-value-selector-grid").dxDataGrid("instance");
                };

                scope.selectorClose = function () {
                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorClose)) {
                        scope.internalApi.onSelectorClose();
                    }

                    scope.isShow = false;
                };

                scope.selectorConfirm = function () {
                    var selectedRow = getSelectionRowData();
                    scope.selectedKeyCode = selectedRow === null ? null : selectedRow.keyCode;

                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorConfirm)) {
                        scope.internalApi.onSelectorConfirm();
                    }

                    scope.isShow = false;
                };

                scope.$watch('isShow', function (newValue, oldValue) {
                    if (newValue) {
                        // refresh the dxDataGrid data source every time when the modal is opening
                        loadData().then(function (data) {
                            //refresh dxDataGrid,so we need user give the selected item id
                            element.children('.modal').modal('show');
                            if (scope.internalApi) {
                                scope.internalApi.getSelection = getSelectionRowData;
                            }

                            resetSelection();

                            //获取指定的选中行,并高亮焦点显示该行
                            var absoluteRowIndex = _.findIndex(data, function (item) { return item.keyCode === scope.selectedKeyCode; });
                            if (gridInstance) {
                                gridInstance.refresh();
                                if (absoluteRowIndex >= 0) {
                                    selectRow(gridInstance, data[absoluteRowIndex]);
                                }
                                else if (data.length > 0) {
                                    gridInstance.pageIndex(0);
                                }
                            }

                            $(".key-value-selector-wrapper .modal-content").draggable({ cursor: "cursor", scroll: true, cancel: '.btn, input, .form-control, .key-value-selector-grid, .close' });
                        });
                    }
                });

                adjustOptions();
            }
        };
    }
]);