﻿commonModule.
controller('regionSelectorController', ['$scope', 'regionService', '$translate', '$timeout',
    function($scope, regionService, $translate, $timeout) {

        var showRegionTree = function() {

            //默认全部不选择
            $scope.gridInstance.unselectAll();

            //如果之前的操作有展开的情况，全部收缩
            var data = $scope.dsCheckBoxTreeView;
            data.forEach(function(item) {
                if (item.expanded) {
                    $scope.gridInstance.collapseItem(item);
                }
            });

            $timeout(InitSelectedNode, 10);

            //展开和隐藏区域选择框
            if ($('.region-tree-wrapper').is(":hidden")) {
                $(".region-tree-wrapper").show();

            } else {
                $(".region-tree-wrapper").hide();
            }

            $(document).one("click", function() {
                $(".region-tree-wrapper").hide();
            });
            event.stopPropagation();
        };

        //当进入编辑状态的时候，前端会传一个选中的城市列表过来，然后根据前端传过来的选中列表去初始化选中的状态
        //所以检测该变量,一旦该变量发生改变
        $scope.$watch('selectedCityList', function(newValue, oldValue) {
            if (newValue) {

                //初始化选中的节点
                $scope.selectedNodeList = [];

                //如果是编辑状态，进入界面前先初始化选中的城市节点
                 $timeout(InitSelectedNode, 10);
                // InitSelectedNode();
            }
        });

        var InitSelectedNode = function() {
            //属于该区域的需要选中
            //selectedCityList选中当前节点之后通过directive传过来的
            $scope.cityNameStr = '';
            if ($scope.selectedCityList != undefined) {
                var citiesOnly = _.filter($scope.selectedCityList, function(item) {
                    return item.level == 2;
                });

                if (citiesOnly && citiesOnly.length > 0)
                {
                    var cityNameList = _.uniq(_.pluck(citiesOnly, 'name'));

                    $scope.cityNameStr = cityNameList.join(',');
                }
                 

                var provincesOnly = _.uniq(_.pluck($scope.selectedCityList, 'parentId'));

                citiesOnly.forEach(function(item) {
                    $scope.gridInstance.selectItem(item.id);
                });

                $scope.selectedNodeList = citiesOnly;



                provincesOnly.forEach(function(item) {
                    // $scope.gridInstance.expandItem(item.id);
                    if (item) {
                        $scope.gridInstance.expandItem(item);
                    }
                });

                // $scope.selectedNodeList = $scope.selectedCityList;
            }
        };

        var getSelectedOrgDisplayText = function() {
            if ($scope.selectedNodeList !== undefined && $scope.selectedNodeList.length > 0) {
                var cityTextList = _.map(_.filter($scope.selectedNodeList, function(node) {
                    return node.level == 2;
                }), function(item) {
                    return item.name;
                });
                return cityTextList.join(',');
            } else {
                return '';
            }
        };

        //加载省市数据
        var loadProvinceAndCity = function() {
            regionService.getProvinceAndCityTreeList().success(function(data) {
                $scope.dsCheckBoxTreeView = data;
            });
        };

        $scope.searchOptions = {
            bindingOptions: {
                value: "searchValue"
            },
            placeholder: $translate.instant('Search'),
            width: 240,
            mode: "search",
            valueChangeEvent: "keyup"
        };

        var dataSource = new DevExpress.data.DataSource({
            load: function(loadOptions) {
                var d = new $.Deferred();

                regionService.getProvinceAndCityTreeList().success(function(data) {
                    $scope.dsCheckBoxTreeView = data;
                    d.resolve(data);
                });

                // regionService.getSettingRegionTree().success(function(data) {
                //     initParam();
                //     setAreaRegionTreeColor(data);
                //     //$scope.areaRegionTreeData = data;

                //     $timeout(function() {

                //         if ($scope.selectedBranch) {
                //             $scope.gridInstance.selectItem($scope.selectedBranch.key);
                //         } else {
                //             $scope.gridInstance.selectItem(data[0].id);
                //         }
                //     }, 20);

                //     d.resolve(data);

                // });

                return d.promise();
            }
        });


        //加载区域选择树形结构
        var loadRegionCheckboxTreeViewSettings = function() {

            $scope.regionCheckboxTreeViewOptions = {

                 bindingOptions: {
                     dataSource: 'dsCheckBoxTreeView',
                     searchValue: 'searchValue'
                 },
                //dataSource: dataSource,
                selection: {
                    mode: "multiple"
                },

                loadPanel: {
                    enabled: true
                },
                scrolling: {
                    mode: "virtual"
                },
                keyExpr: "id",
                showRowLines: true,
                showColumnLines: true,
                expandAllEnabled: true,
                rowAlternationEnabled: true,
                expandedRowKeys: [1],
                showBorders: true,
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                displayExpr: 'text', //显示的属性名称，默认为text
                itemsExpr: 'items', //子层的数组名称，默认为items
                //scrollDirection:'vertical',  //Accepted Values: 'vertical' | 'horizontal' | 'both'
                selectNodesRecursive: true, //级联选择
                showCheckBoxesMode: 'normal', //Accepted Values: 'none' | 'normal' | 'selectAll' 
                onInitialized: function(e) {
                    $scope.gridInstance = e.component;
                },
                onItemSelectionChanged: function(e) {
                    if (e.node.selected) {
                        //如果选中的是叶子节点 
                        if (e.node.children.length == 0) {
                            var selectedNode = {
                                id: e.node.key,
                                parentId: e.node.parent == undefined ? null : e.node.parent.key,
                                name: e.node.text,
                                level: 2
                            };

                            //检查是否重复，只添加不存在的
                            if (!checkNodeExistence($scope.selectedNodeList, selectedNode)) {
                                $scope.selectedNodeList.push(selectedNode);
                            }
                        } else {
                            //选中的是父级节点，需要包含所有的子节点
                            e.node.children.forEach(function(item) {

                                var selectedNode = {
                                    id: item.key,
                                    parentId: item.parent.key,
                                    name: item.text,
                                    level: 2
                                };

                                //检查是否重复，只添加不存在的
                                if (!checkNodeExistence($scope.selectedNodeList, selectedNode)) {
                                    $scope.selectedNodeList.push(selectedNode);
                                }
                            });
                        }
                    } else {
                        //如果只是取消叶子节点
                        if (e.node.children.length == 0) {
                            var selectedNodeList = $scope.selectedNodeList;
                            var findUncheckedOne = _.find(selectedNodeList, function(item) {
                                return item.id === e.node.key && item.name == e.node.text;
                            });
                            $scope.selectedNodeList = _.without(selectedNodeList, findUncheckedOne);
                        } else {
                            //如果取消父级节点 
                            var selectedNodeList = $scope.selectedNodeList;

                            var idList = _.map(e.node.children, function(item) {
                                return item.key;
                            });

                            var findUncheckedList = _.filter(selectedNodeList, function(item) {
                                return idList.indexOf(item.id) >= 0;
                            });

                            $scope.selectedNodeList = _.difference(selectedNodeList, findUncheckedList);
                        }
                    }
                },
                //点击行，选中
                onItemClick: function(e) {
                    var data = e.itemData;
                },
                onContentReady: function(object, container) {
                    //object.element.find('.dx-checkbox-checked').addClass('dx-state-disabled');
                }
            };
        };

        var checkNodeExistence = function(allSelectedNodeList, node) {
            var filteredItem = _.find(allSelectedNodeList, function(item) {
                return item.id == node.id && item.name == node.name;
            });
            if (filteredItem) {
                return true;
            }
            return false;
        };

        (function() {
            $(".region-tree-wrapper").on("click", function(e) {
                e.stopPropagation();
            });
            $scope.isfirstShow = true;
            $scope.checkedRegionsNameSet = {};

            $scope.cityNameStr = '';

            //加载dev Treeview的配置
            loadRegionCheckboxTreeViewSettings();

            //加载省市数据
             loadProvinceAndCity();

            $scope.showRegionTree = showRegionTree;
            $scope.getSelectedOrgDisplayText = getSelectedOrgDisplayText;

        })();
    }
]);
