﻿// Common multiple select tree view, select multiple items with checkboxes
commonModule.directive('treeMultiSelector', ['$log', '$timeout', '$interval', '$translate',
    function($log, $timeout, $interval, $translate) {
        'use strict';
        $log.debug('treeMultiSelector.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/common/controls/tree-multi-selector/tree-multi-selector.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                columnDefs: '=',
                treeDataSource: '=*',
                isShow: '=',
                selectedItems: '=',
                title: '=?',
                parentCheckable: '=?',
                isSingleMode: '=?',
                selectorOptions: '=?',
                selectorApi: '=?' // In: onSelectorClose, onSelectorConfirm; Out: getGridInstance
            },
            link: function(scope, element, attrs) {
                $log.debug('treeMultiSelector.link()...');

                var treeList = undefined;

                // Handle id of the element to make sure it is unique.
                // treeListSelector is used to add the style tag to work-around
                // DX grid height issue
                if (_.isEmpty(attrs['id'])) {
                    var randomStr = Math.random().toString().replace(/\./g, '');
                    element.attr('id', 'tree-list' + randomStr);
                    scope.treeListSelector = '#tree-list' + randomStr;
                } else {
                    scope.treeListSelector = '#' + attrs['id'];
                }

                scope.showParentCheckbox = scope.parentCheckable ? '' : 'display: none;';

                // Register api functions in internalApi
                // If selectorApi passed is not undefined, we will use it as internalApi
                scope.internalApi = scope.selectorApi || {};
                scope.internalApi.getGridInstance = function() {
                    return element.find('.tree-multi-selector-grid').dxTreeList('instance');
                };

                // Work-around for DX grid height issue.
                element.find('.modal').on('shown.bs.modal', function() {
                    scope.internalApi.getGridInstance().refresh();
                });


                var getsubNodesRecursive = function(key) {
                    var keyList = [];

                    var subList = _.filter(scope.treeDataSource, function(row) {
                        return row[scope.parentKey] === key
                    });


                    if (subList && subList.length > 0) {

                        subList.forEach(function(row) {

                            var temp = getsubNodesRecursive(row[scope.idField]);
                            keyList.push(row[scope.idField]);

                            if (temp && temp.length > 0) {
                                keyList = _.union(keyList, temp);
                            }
                        });
                    }

                    return keyList;
                };


                var getParentRecursive = function(key) {

                    var parentList = [];
                    var node = _.find(scope.treeDataSource, function(row) {

                        return row[scope.idField] === key;
                    });

                    if (node && node[scope.parentKey]) {
                        parentList.push(node[scope.parentKey]);

                        var temp = getParentRecursive(node[scope.parentKey]);

                        if (temp && temp.length > 0) {
                            parentList = parentList.concat(temp);
                        }
                    }

                    return parentList;
                };


                var getchildren = function(selectKeyList) {

                    var keyList = [];
                    if (selectKeyList && selectKeyList.length > 0) {

                        selectKeyList.forEach(function(key) {
                            keyList.push(key);
                            var temp = getsubNodesRecursive(key);
                            if (temp && temp.length > 0) {
                                keyList = _.union(keyList, temp);
                            }
                        });

                    }

                    return keyList;
                };

                var getParentList = function(keys) {
                    if (keys && keys.length > 0) {
                        // 子节点全部选中，父节点选中
                        // 子节点未选中，父节点不选中

                        var selectNodeList = _.filter(scope.treeDataSource, function(row) {
                            return keys.indexOf(row[scope.idField]) > -1;
                        });

                        var parentKeyList = _.uniq(_.pluck(selectNodeList, scope.parentKey));

                        parentKeyList = _.filter(parentKeyList, function(row) {
                            if (row && row !== '') {
                                return true;
                            } else {
                                return false;
                            }
                        });

                        // 没有被选中的Key
                        parentKeyList = _.difference(parentKeyList, keys);


                        var parentSelectedKeyList = [];

                        var parentUncheckList = [];
                        parentKeyList.forEach(function(row) {
                            var subList = getsubNodesRecursive(row);

                            var noCheck = _.difference(subList, keys);

                            if (!noCheck || noCheck.length === 0) {
                                parentSelectedKeyList.push(row);
                            }

                            if (noCheck && noCheck.length === subList.length) {
                                parentUncheckList.push(row);
                            }

                        });

                        keys = _.union(keys, parentSelectedKeyList);
                        keys = _.difference(keys, parentUncheckList);

                        return keys;

                    }

                    return keys;

                };


                var getCheckParentList = function(key, selectKeyList) {
                    var parentCodeList = getParentRecursive(key);
                    
                    var checkParentList = [];

                    if (parentCodeList && parentCodeList.length > 0) {

                        var uncheckParentCodeList =  _.difference(parentCodeList, selectKeyList);
                        for (var i = 0; i < uncheckParentCodeList.length; i++) {

                            var row = uncheckParentCodeList[i];
                            var subList = getsubNodesRecursive(row);

                            if (subList && subList.length > 0) {
                                var noCheck = _.difference(subList, selectKeyList);

                                if (!noCheck || noCheck.length === 0) {
                                     selectKeyList.push(row);
                                     checkParentList.push(row);
                                } else{
                                    break;
                                }

                            }
                        }

                    }

                    return checkParentList;
                };

                var isAutoSetting = false;

                var gridOptions = function() {
                    return {
                        dataSource: scope.treeDataSource,
                        keyExpr: scope.idField,
                        parentIdExpr: scope.parentKey,
                        height: scope.selectorOptions.gridHeight,
                        columns: scope.columnDefs,
                        loadPanel: {
                            enabled: false
                        },
                        scrolling: {
                            mode: "virtual"
                        },
                        searchPanel: {
                            visible: true
                        },
                        selection: {
                            mode: scope.isSingleMode ? "single" : "multiple",
                            allowSelectAll: false
                        },
                        sorting: {
                            mode: 'single'
                        },
                        filterRow: {
                            visible: false
                        },
                        //cacheEnabled: true,
                        allowColumnResizing: true,
                        columnAutoWidth: true,
                        showRowLines: true,
                        showColumnLines: true,
                        rowAlternationEnabled: true, //单双行颜色
                        showBorders: true,
                        expandedRowKeys: [1],
                        onSelectionChanged: function(item) {

                            var selectKeyList = item.selectedRowKeys;

                            if (item.selectedRowKeys && item.selectedRowKeys.length ===0){
                                isAutoSetting = false;
                            }

                            if (!isAutoSetting) {

                                if (scope.selectorOptions.selectNodesRecursive) {

                                    var unCheckList = [];
                                    var checkList = selectKeyList;
                                    var checkOnlyList = [];
                                    if (item.currentDeselectedRowKeys && item.currentDeselectedRowKeys.length > 0) {
                                        // uncheck children
                                        unCheckList = getchildren(item.currentDeselectedRowKeys);

                                        // uncheck parent
                                        var parentList = getParentRecursive(item.currentDeselectedRowKeys[0]);
                                        unCheckList = _.union(unCheckList, parentList);
                                        var pureUnCheckList = _.filter(unCheckList, function(row){
                                            return selectKeyList.indexOf(row) > -1;
                                        });

                                        checkOnlyList = _.difference(checkList, unCheckList);

                                        if (pureUnCheckList && pureUnCheckList.length > 0) {
                                            isAutoSetting = true;
                                            item.component.deselectRows(pureUnCheckList);
                                        }

                                    } else {

                                        // check
                                        var checkChildren = getchildren(item.currentSelectedRowKeys);

                                        checkList = _.union(checkList, checkChildren);

                                        // check parent
                                        var checkParentList = getCheckParentList(item.currentSelectedRowKeys[0],checkList);

                                        checkList = _.union(checkList, checkParentList);
                                        checkOnlyList = checkList;

                                        var auoCheckList = _.difference(checkOnlyList, selectKeyList);

                                        if (auoCheckList && auoCheckList.length > 0) {
                                            isAutoSetting = true;
                                            item.component.selectRows(checkOnlyList, false);
                                        }
                                    }

                                    selectKeyList = checkOnlyList;
                                }
                            } else {
                                isAutoSetting = false;
                            }

                            scope.tmpSelectedItems = selectKeyList;
                        }
                    };
                };

                // Try to get identity column field from columnDefs
                var getIdField = function(columnDefs) {
                    if (_.isEmpty(columnDefs)) {
                        return null;
                    }

                    var idColumn = _.findWhere(columnDefs, { isIdentity: true });
                    if (_.isEmpty(idColumn)) {
                        idColumn = columnDefs[0];
                    }

                    return idColumn.dataField;
                }

                // Try to get parent key column field from columnDefs
                var getParentKey = function(columnDefs) {
                    if (_.isEmpty(columnDefs)) {
                        return null;
                    }

                    var keyColumn = _.findWhere(columnDefs, { isParentKey: true });
                    if (_.isEmpty(keyColumn)) {
                        keyColumn = columnDefs[0];
                    }

                    return keyColumn.dataField;
                };

                var loadColumnDefs = function(columnDefs) {
                    scope.idField = getIdField(columnDefs);
                    if (_.isEmpty(scope.idField)) {
                        throw new Error('Identity field must be defined!');
                    }

                    scope.parentKey = getParentKey(columnDefs);
                    if (_.isEmpty(scope.parentKey)) {
                        throw new Error('Parent key field must be defined!');
                    }

                    if (scope.idField === scope.parentKey) {
                        throw new Error('Parent key field should not be the same as identity field!');
                    }
                };

                var loadData = function(data) {
                    var idList = _.pluck(data, scope.idField);
                    _.each(data, function(d) {
                        if (!_.contains(idList, d[scope.parentKey])) {
                            d[scope.parentKey] = null;
                        }
                    });
                };

                var initSelections = function() {
                    scope.tmpSelectedItems = angular.copy(scope.selectedItems);
                    scope.internalApi.getGridInstance().selectRows(scope.tmpSelectedItems || []);
                };

                // Adjust the selector options
                var adjustOptions = function() {

                    // Adjust the CSS styles of the selector, currently only support width, height, top and left.
                    scope.selectorOptions = _.extend({
                        // Default options
                        height: 400,
                        width: 730
                    }, scope.selectorOptions);
                    if (_.isNumber(scope.selectorOptions.height)) {
                        scope.selectorOptions.gridHeight = scope.selectorOptions.height > 90 ?
                            (scope.selectorOptions.height - 90) : 1;
                    }

                    scope.cssOptions = _.pick(scope.selectorOptions, 'width', 'height', 'top', 'left');
                };

                scope.$watchCollection('treeDataSource', function(newValue, oldValue) {
                    var internalDataSource = angular.copy(newValue);
                    loadData(internalDataSource);
                    loadColumnDefs(scope.columnDefs);
                    if (treeList) {
                        treeList.option({
                            keyExpr: scope.idField,
                            parentIdExpr: scope.parentKey,
                            columns: scope.columnDefs,
                            dataSource: internalDataSource
                        });
                    } else {
                        element.find('.tree-multi-selector-grid').dxTreeList(gridOptions());
                        treeList = element.find('.tree-multi-selector-grid').dxTreeList('instance');
                        treeList.option('columns', scope.columnDefs);
                    }

                    // Use $timeout to make sure hash of the data generated before initSelections
                    $timeout(initSelections, 10);
                });

                scope.$watch('isShow', function(newValue, oldValue) {
                    if (newValue) {
                        var internalDataSource = angular.copy(scope.treeDataSource);
                        loadData(internalDataSource);
                        loadColumnDefs(scope.columnDefs);
                        if (treeList) {
                            treeList.option({
                                keyExpr: scope.idField,
                                parentIdExpr: scope.parentKey,
                                columns: scope.columnDefs,
                                dataSource: internalDataSource
                            });
                        } else {
                            element.find('.tree-multi-selector-grid').dxTreeList(gridOptions());
                            treeList = element.find('.tree-multi-selector-grid').dxTreeList('instance');
                            treeList.option('columns', scope.columnDefs);
                        }
                        element.find('.modal').modal('show');

                        // Use $timeout to make sure hash of the data generated before initSelections
                        $timeout(initSelections, 10);
                    } else {
                        element.find('.modal').modal('hide');
                    }
                });

                scope.selectorClose = function() {
                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorClose)) {
                        scope.internalApi.onSelectorClose();
                    }

                    scope.isShow = false;
                };

                scope.selectorConfirm = function() {
                    // Copy selected items to selectedItems
                    scope.selectedItems.length = 0;
                    for (var i = 0; i < scope.tmpSelectedItems.length; i++) {
                        scope.selectedItems.push(scope.tmpSelectedItems[i]);
                    }

                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorConfirm)) {
                        scope.internalApi.onSelectorConfirm();
                    }

                    scope.isShow = false;
                };

                adjustOptions();
                if (!scope.treeDataSource) {
                    scope.treeDataSource = [];
                }
            }
        };
    }
]);