﻿//常用公用方法
commonModule.factory('ackLib', ['$translate', function ($translate) {
    'use strict';

    var ackLib = {};

    //辅助方法
    ackLib.Helper = {
        //翻译，仅此一个入口
        translate: function (value) {
            return $translate.instant(value);
        }
    }

    //验证
    ackLib.validation = {
        //是否为日期验证，如果验证成功，返回true,否则返回false
        isDate: function (value, dateSeparator) {
            try {
                var dateSplit = dateSeparator || constant.date.dateFormatSeparator;
                var arr = value.split(dateSplit);
                var year = parseInt(arr[0]);
                var month = parseInt(arr[1]);
                var day = parseInt(arr[2]);
                var date = new Date(year, month - 1, day);
                if (date.getFullYear() == year && date.getMonth() + 1 == month && date.getDate() == day) {
                    return true;
                } else {
                    return false;
                }
            }
            catch (e) {
                return false;
            }
        },
        isEmpty: function (str) {
            if (!str || str === '') {
                return true;
            }
            else {
                return false;
            }
        }
    };

    //翻译,所有的翻译都放这，到时直接调用,不用每个页面都去做翻译
    ackLib.translation = {};
    //常用翻译
    ackLib.translation.common = {
        Remark: ackLib.Helper.translate('Remark'), //备注
        DownloadTemplateFail: ackLib.Helper.translate('DownloadTemplateFail'), //下载模板失败
        FileUploadSuccess: ackLib.Helper.translate('FileUploadSuccess'), //文件上传成功
        SelectAtLeastOneRecord: ackLib.Helper.translate('SelectAtLeastOneRecord'), //请至少选择一条记录
        OperateSuccess: ackLib.Helper.translate('OperateSuccess'), //操作成功
        UploadSuccess: ackLib.Helper.translate('UploadSuccess'), //上传成功
        PleaseSelectFile: ackLib.Helper.translate('PleaseSelectFile'), //请先选择文件
        NoDataText: ackLib.Helper.translate('NoDataText'), //没有数据
        pleaseSelect: ackLib.Helper.translate('ChoosePlaceholder'), //-请选择-
        pleaseInput: ackLib.Helper.translate('InputPlaceholder'), //-请输入-
        PleaseSelectRecordToDelete: ackLib.Helper.translate('PleaseSelectRecordToDelete'), //请选择需要删除的记录
        FileName: ackLib.Helper.translate('FileNameTitle'), //文件名称
        ErrorMessage: ackLib.Helper.translate('ErrorMessage'), //错误消息
        Confirm: ackLib.Helper.translate('Confirm'), //错误消息
        Cancel: ackLib.Helper.translate('Cancel'), //错误消息
    };

    //发票相关
    ackLib.translation.inputInvoice = {
        AllowRefundInvoiceInfo: ackLib.Helper.translate('AllowRefundInvoiceInfo'), //只有状态为识别成功，识别失败，无法识别,待退票或待匹配的发票才可以做退票操作
        SpecialToInvoiceRecognizeSuccessInfo: ackLib.Helper.translate('SpecialToInvoiceRecognizeSuccessInfo'), //对于发票来源为“发票章错误，待退票”且发票状态为【待退票】的发票，只支持单张退票，不支持与其他发票一起批量退票
        SpecialToInvoicePendingRefundInfo: ackLib.Helper.translate('SpecialToInvoicePendingRefundInfo'), //对于发票来源为“发票章错误，待退票”且发票状态为【识别成功】的发票，支持批量退票，不支持将此类发票与其他发票一起批量退票。
        AllowInvoiceRecognizeWithUpload: ackLib.Helper.translate('AllowInvoiceRecognizeWithUpload'), //只有已上传或者已补录的发票可以做发票识别
        RecognizeInvoiceSuccess: ackLib.Helper.translate('RecognizeInvoiceSuccess'), //发票识别成功
        VerifyInvoiceSuccess: ackLib.Helper.translate('VerifyInvoiceSuccess'), //发票验真成功
        CannotExpireInvoice: ackLib.Helper.translate('CannotExpireInvoice'), //已认证、已失效、已清理、已匹配、已退票的发票不能做失效操作
        ConfirmToExpireInvoice: ackLib.Helper.translate('ConfirmToExpireInvoice'), //确认要把选中的发票进行失效处理吗
    }

    //浏览器检测
    ackLib.Browser = {
        isIE: (function () {
            if (navigator.userAgent.match(/msie/i) || navigator.userAgent.match(/trident/i)) {
                return true;
            }
            return false;
        })(),
    };

    //小数点位数
    var fixedNumber = constant.toFixedNumber;
    //数字的格式化
    ackLib.Number = {
        toFixed: function (num) {
            return num.toFixed(fixedNumber);
        },
    }


    //阻止冒泡
    ackLib.stopPropagation = function (event) {
        event = event || window.event;
        if (event.stopPropagation) {
            event.stopPropagation();
        } else {
            event.cancelBubble = true;
        }
    };

    //动态构造table
    ackLib.populateTable = function (columnList, valueList) {

        var table = '';
        var tableHeader = '<table class="table table-striped table-hover table-bordered" style="max-height:200px; overflow:auto; text-align:left">';
        var tbody = '';
        var th = '';

        columnList.forEach(function (item) {
            th += '<th>' + item + '</th>';
        });
        var thead = '<tr>' + th + '</tr>';

        valueList.forEach(function (row) {
            var tr = '';
            columnList.forEach(function (item) {
                tr += '<td>' + row[item] + '</td>';
            });

            tbody += '<tr>' + tr + '</tr>';
        });

        table = tableHeader + thead + tbody + '</table>';

        // console.log(table);

        return table;
    };


    return ackLib;

}]);


/*文件上传:目前支持单文件上传，支持跨域上传
创建于:2017/09/08
*/
commonModule.factory('ackFileUploader', ['Upload', '$q', 'apiInterceptor', 'ackLib', function (Upload, $q, apiInterceptor, ackLib) {
    'use strict';

    var defaultOptions = {
        chunkSize: '10MB',
        resumable: true,
        token: $('input[name="__RequestVerificationToken"]').val(),
        url: '',
        cancel: false,
        maximumSize: 1024 * 1024 * 10,  //10MB,
        isAppend: true, //默认为追加
    };

    defaultOptions.headers = {
        'Access-Control-Allow-Origin': '*',
        Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
        __RequestVerificationToken: defaultOptions.token,
        withCredentials: true
    }


    var ackFileUploader = {

        //上传单个文件
        upload: function (myfile, options) {
            var deferred = $q.defer();
            options = $.extend(true, {}, defaultOptions, options);
            var fileName = myfile.name;

            var self = ackFileUploader;

            if (!fileName || fileName.size == 0) {
                deferred.reject("FileIsEmpty");
            }
            if (myfile.size > options.maximumSize) {
                deferred.reject("FileSizeExceedMaximum");
            }
            var optionsData = {
                cancel: options.cancel,
                filename: fileName,
            }

            optionsData = $.extend(true, {}, optionsData, options.data);
            Upload.upload({
                url: options.url,
                data:optionsData,
                file: myfile,
                resumeChunkSize: options.resumable ? options.chunkSize : null,
                headers: options.headers,
                withCredentials: true
            }).then(function (resp) {
                deferred.resolve(resp);
            });
            return deferred.promise;
        },

        //uploadMultiple: function (files, options) {
        //    if (files && files.length > 0) {
        //        for (var i = 0; i < files.length; i++) {
        //            ackFileUploader.upload(files[i], options);
        //        }
        //    }
        //}
    };

    return ackFileUploader;

}]);


//消息弹框
commonModule.factory('ackMessageBox', ['$translate', 'SweetAlert', '$q', 'ackLib', function ($translate, SweetAlert, $q, ackLib) {
    'use strict';
    //只负责弹框,保持独立性
    var messageBox = {
        success: function (title, message) {
            if (arguments.length == 1) { // message is undefined
                SweetAlert.success(title, '');
            }
            else if (arguments.length == 2) { // message is undefined
                SweetAlert.success(title, message);
            }
        },

        warning: function (title, message) {
            if (arguments.length == 1) { // message is undefined
                SweetAlert.warning(title, '');
            }
            else if (arguments.length == 2) { // message is undefined
                SweetAlert.warning(title, message);
            }
        },

        close: function () {
            swal.close();
        },

        //有确认和取消按钮
        confirm: function (title, text, customClass) {
            var deferred = $q.defer();
            SweetAlert.swal({
                title: title,
                text: text,
                html: true,
                type: "warning",
                customClass: customClass || '',
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                allowOutsideClick: false,
                confirmButtonText: ackLib.translation.common.Confirm,
                cancelButtonText: ackLib.translation.common.Cancel,
                closeOnConfirm: true,
                closeOnCancel: true
            },
              function (isConfirm) {
                  deferred.resolve(isConfirm);
              });

            return deferred.promise;
        },

        //没有取消按钮
        info: function (title, text, customClass) {
            var deferred = $q.defer();

            SweetAlert.swal({
                title: title,
                text: text,
                html: true,
                type: "warning",
                customClass: customClass || 'swal-info',
                showCancelButton: false,
                confirmButtonColor: "#DD6B55",
                allowOutsideClick: false,
                confirmButtonText: ackLib.translation.common.Confirm,
                cancelButtonText: ackLib.translation.common.Cancel,
                closeOnConfirm: true,
                closeOnCancel: false
            },
              function (isConfirm) {
                  deferred.resolve(isConfirm);
              });

            return deferred.promise;
        },
    };

    return messageBox;

}]);


//UIModal模态框
commonModule.factory('ackUibModal', ['$translate', 'SweetAlert', '$q', '$uibModal', '$document', function ($translate, SweetAlert, $q, $uibModal, $document) {
    'use strict';


    //模态框
    /*参数currentScope：页面的scope
    templateUrl:定义弹框的模板，
    windowClass:定义弹框的css，
    parentSelector:父框，
    isbackrop:，‘static’, or 'trye'
    resetCallbackFunc: 回调函数
    */
    var createModalInstance = function (currentScope, templateUrl, windowClass, parentSelector, isbackrop, resetCallbackFunc) {
        var thisModalService = new Object();
        isbackrop = isbackrop ? isbackrop : true;

        var parentElem = parentSelector ? angular.element($document[0].querySelector(parentSelector)) : undefined;

        thisModalService.open = function () {
            var modalInstance = $uibModal.open({
                animation: true,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                backdrop: isbackrop,  //点击父页面不会自动关闭 
                templateUrl: templateUrl, //script模板
                windowClass: windowClass, //弹框页面的css
                scope: currentScope,
                appendTo: parentElem,
            });

            thisModalService.modalInstance = modalInstance;

            //modal关闭之后接收返回值的函数
            modalInstance.result.then(function (data) {
                //call close的时候调用这里

            }, function () {
                //取消的时候触发这里

            }).finally(function () {
                resetCallbackFunc(); //回调函数
                thisModalService = null;
            });
        };

        thisModalService.close = function (result) {
            if (thisModalService.modalInstance) {
                thisModalService.modalInstance.close(result);
            }
        };

        thisModalService.cancel = function () {
            if (thisModalService.modalInstance) {
                thisModalService.modalInstance.dismiss('cancel');
            }
        }
        return thisModalService;
    };

    return createModalInstance;


}]);