﻿webservices.factory('vatReportCacheService', ['$log', '$q', '$http', '$cacheFactory', 'vatCommonService', 'vatSessionService',
    'vatReportService', 'modelConfigurationService', 'apiConfig', 'enums',
    function ($log, $q, $http, $cacheFactory, vatCommonService, vatSessionService, vatReportService, modelConfigurationService, apiConfig, enums) {
        'use strict';
        $log.debug('vatReportCacheService.ctor()...');
        var reportCache = $cacheFactory('report-cache');

        var setProjectStatus = function () {
            var status = vatSessionService.project.projectStatusList[vatSessionService.month];
            //已生成，已提交=>原状态
            if (_.isEqual(vatSessionService.project.projectStatusList[vatSessionService.month], constant.ProjectStatusEnum.Generated) ||
                _.isEqual(vatSessionService.project.projectStatusList[vatSessionService.month], constant.ProjectStatusEnum.ReportSubmitted)) {
                status = vatSessionService.project.projectStatusList[vatSessionService.month];
            }

            //已审核及后续状态=>已生成
            if (vatSessionService.project.projectStatusList[vatSessionService.month] >= constant.ProjectStatusEnum.ReportApproved) {
                status = constant.ProjectStatusEnum.Generated;
            }

            return vatCommonService.setProjectStatus(vatSessionService.project.id, vatSessionService.month, status, null, null);
        };

        return {

            // 修改保存机制后，需要添加缓存机制的方法：
            // 1: addCellManualData
            // 2: deleteCellVoucher
            // 3: deleteCellInvoice
            // 4: deleteCellDatasource
            // 5: addDataSource
            // 6: addDataSourceItems
            // 7: updateDataSourceAmount
            // 8: modelConfigurationService.deleteModelBackFill
            // 9: modelConfigurationService.modelBackFill

            // 初始化report cache
            initReportCache: function () {
                reportCache.removeAll();
                reportCache.put('report-cache', []);
            },

            // 检查是否存在report cache
            hasReportCache: function () {
                var cache = reportCache.get('report-cache');
                return !_.isEmpty(cache);
            },

            // 保存report cache中的数据
            saveReportCache: function (dirtyData, reportId, generator) {
                var promise = $q.when();
                var cache = reportCache.get('report-cache');

                // 按顺序执行缓存的service
                while (!_.isEmpty(cache)) {
                    var serviceItem = cache.shift();
                    (function (i) {
                        if (_.isFunction(vatReportService[i.serviceType])) {
                            promise = promise.then(function () {
                                return vatReportService[i.serviceType].apply(vatReportService, i.params);
                            });
                        }
                        else if (_.isFunction(modelConfigurationService[i.serviceType])) {
                            promise = promise.then(function () {
                                return modelConfigurationService[i.serviceType].apply(modelConfigurationService, i.params);
                            });
                        }
                    }(serviceItem));
                }

                if (!_.isEmpty(dirtyData)) {
                    promise = promise.then(function () {
                        return vatReportService.updateReportCellValue(dirtyData, reportId, generator);
                    });
                }

                promise = promise.then(setProjectStatus);

                this.initReportCache();
                return promise;
            },

            //updateConfig: function (projectId, period, generator) {
            //    return $http.post('/Report/updateConfig/' + projectId + '/' + period + '?generator=' + generator, {}, apiConfig.createVat({ ignoreLoadingBar: true }));
            //},
            //generate: function (projectId, templateId, period, generator) {
            //    return $http.post('/Report/generate/' + projectId + '/' + templateId + '/' + period + '?generator=' + generator, {}, apiConfig.createVat({ ignoreLoadingBar: true }));
            //},
            //getReportData: function (reportId) {
            //    return $http.get('/Report/reportData/' + reportId, apiConfig.createVat());
            //},
            //CalculateSingleCellFormula: function (cellId) {
            //    return $http.get('/Report/formulaDataSource/byCell/' + cellId, apiConfig.createVat());
            //},
            //getTemplate: function (projectId, period) {
            //    return $http.get('/Report/template/' + projectId + '/' + period, apiConfig.createVat());
            //},
            //getCellVoucher: function (cellDataID) {
            //    return $http.get('/Report/cellVoucher/' + cellDataID, apiConfig.createVat());
            //},
            //getCellInvoice: function (cellDataID, invoiceType) {
            //    return $http.get('/Report/cellInvoice/' + cellDataID + '/' + invoiceType, apiConfig.createVat());
            //},
            addCellManualData: function (manualData, logInfo) {
                var cache = reportCache.get('report-cache');
                var existingItem = _.findWhere(cache, { serviceType: 'addCellManualData', id: manualData.cellTemplateID });
                var paramDs = [angular.copy(manualData), angular.copy(logInfo)];
                if (existingItem) {
                    existingItem.params = paramDs;
                }
                else {
                    cache.push({
                        serviceType: 'addCellManualData',
                        id: manualData.cellTemplateID,
                        params: paramDs
                    });
                }

                reportCache.put('report-cache', cache);
                var manualDataSource = {
                    name: manualData.name,
                    amount: manualData.amount,
                    description: manualData.description,
                    projectID: manualData.projectID,
                    serviceTypeID: manualData.serviceTypeID,
                    cellDataID: manualData.cellID,
                    cellTemplateID: manualData.cellTemplateID,
                    dataSourceType: enums.cellDataSourceType.KeyIn
                };

                return $q.when(manualDataSource);
                //return $http.post('/Report/addCellManualData', manualData, apiConfig.createVat());
            },
            deleteCellVoucher: function (voucherId, dataSourceId) {
                var cache = reportCache.get('report-cache');
                cache.push({
                    serviceType: 'deleteCellVoucher',
                    id: voucherId + "," + dataSourceId,
                    params: [voucherId, dataSourceId]
                });

                reportCache.put('report-cache', cache);
                var defer = $q.defer();
                var promise = defer.promise;
                defer.resolve()
                return $q.when({ detailId: voucherId, dataSourceId: dataSourceId, opType: 0 });
                //return $http.post('/Report/deleteCellVoucher/' + voucherId + '/' + datasourceId, {}, apiConfig.createVat());
            },
            deleteCellInvoice: function (invoiceId, dataSourceId) {
                var cache = reportCache.get('report-cache');
                cache.push({
                    serviceType: 'deleteCellInvoice',
                    id: invoiceId + "," + dataSourceId,
                    params: [invoiceId, dataSourceId]
                });
                reportCache.put('report-cache', cache);
                return $q.when({ detailId: invoiceId, dataSourceId: dataSourceId, opType: 1 });
                //return $http.post('/Report/deleteCellInvoice/' + invoiceId + '/' + datasourceId, {}, apiConfig.createVat());
            },
            deleteCellDatasource: function (dataSourceId, cellId) {
                var cache = reportCache.get('report-cache');
                cache.push({
                    serviceType: 'deleteCellDatasource',
                    id: dataSourceId + "," + cellId,
                    params: [dataSourceId, cellId]
                });
                reportCache.put('report-cache', cache);
                return $q.when({ dataSourceId: dataSourceId, cellId: cellId });
                //return $http.post('/Report/deleteCellDatasource/' + datasourceID + '/' + cellID, {}, apiConfig.createVat());
            },
            deleteModelBackFill: function (dataSourceId, cellId) {
                var cache = reportCache.get('report-cache');
                cache.push({
                    serviceType: 'deleteModelBackFill',
                    id: dataSourceId + "," + cellId,
                    params: [dataSourceId, cellId]
                });
                reportCache.put('report-cache', cache);
                return $q.when({ dataSourceId: dataSourceId, cellId: cellId });
                //return $http.post('/Report/deleteCellDatasource/' + datasourceID + '/' + cellId, {}, apiConfig.createVat());
            },
            //getAllDataItems: function (dataSourceType) {
            //    return $http.get('/Report/getAllDataItems/' + dataSourceType, apiConfig.createVat())
            //},
            addDataSource: function (dataSource) {
                var cache = reportCache.get('report-cache');
                var existingItem = _.findWhere(cache, { serviceType: 'addDataSource', id: dataSource.ID });
                var paramDs = angular.copy(dataSource);
                if (existingItem) {
                    existingItem.params = [paramDs];
                }
                else {
                    cache.push({
                        serviceType: 'addDataSource',
                        id: dataSource.ID,
                        params: [paramDs]
                    });
                }

                reportCache.put('report-cache', cache);
                var manualDataSource = {
                    id: dataSource.id,
                    name: dataSource.name,
                    dataSourceType: dataSource.dataSourceType,
                    cellDataID: dataSource.cellDataID,
                    projectID: dataSource.projectID,
                    serviceTypeID: dataSource.serviceTypeID,
                    amount: 0,
                    creator: dataSource.creator,
                    updater: dataSource.updater,
                    operationType: dataSource.operationType
                };

                return $q.when(manualDataSource);
                //return $http.post('/Report/addDataSource', dataSource, apiConfig.createVat());
            },
            addDataSourceItems: function (dataSource) {
                var cache = reportCache.get('report-cache');
                var existingItem = _.findWhere(cache, { serviceType: 'addDataSourceItems', id: dataSource.id });
                var paramDs = angular.copy(dataSource);
                delete paramDs.extraItems;
                if (existingItem) {
                    existingItem.params = [paramDs];
                }
                else {
                    cache.push({
                        serviceType: 'addDataSourceItems',
                        id: dataSource.id,
                        params: [paramDs]
                    });
                }

                reportCache.put('report-cache', cache);
                var manualDataSource = {
                    id: dataSource.id,
                    name: dataSource.name,
                    items: dataSource.items,
                    cellDataID: dataSource.cellDataID,
                    projectID: dataSource.projectID,
                    amount: dataSource.amount,
                    serviceTypeID: dataSource.serviceTypeID,
                    dataSourceType: dataSource.dataSourceType,
                    operationType: dataSource.operationType
                };

                if (dataSource.dataSourceType === enums.cellDataSourceType.OutputInvoice
                    || dataSource.dataSourceType === enums.cellDataSourceType.InputInvoice
                    || dataSource.dataSourceType === enums.cellDataSourceType.CustomInvoice
                    || dataSource.dataSourceType === enums.cellDataSourceType.Voucher) {
                    manualDataSource.extraItems = dataSource.extraItems;
                }

                return $q.when(manualDataSource);
                //return $http.post('/Report/addDataSourceItems', dataSource, apiConfig.createVat());
            },
            //getCellAccountRange: function (reportTemplateId, period, rowIndex, columnIndex) {
            //    return $http.get('/Report/getCellAccountRange/' + reportTemplateId + '/' + period + '/' + rowIndex + '/' + columnIndex, apiConfig.createVat());
            //},
            //getCellTemplateConfig: function (reportTemplateId, period, rowIndex, columnIndex) {
            //    return $http.get('/Report/getCellTemplateConfig/' + reportTemplateId + '/' + period + '/' + rowIndex + '/' + columnIndex, apiConfig.createVat());
            //},
            //updateCellAccountRangeAndSummary: function (cellTemplateId, period, value, operation) {
            //    return $http.post('/Report/updateCellAccountRangeAndSummary/' + cellTemplateId + '/' + period + '/' + value + '/' + operation, {}, apiConfig.createVat());
            //},
            updateDataSourceAmount: function (dataSource, amount) {
                var cache = reportCache.get('report-cache');
                var existingItem = _.findWhere(cache, { serviceType: 'updateDataSourceAmount', id: dataSource.id });
                if (existingItem) {
                    existingItem.params = [dataSource.id, amount];
                }
                else {
                    cache.push({
                        serviceType: 'updateDataSourceAmount',
                        id: dataSource.id,
                        params: [dataSource.id, amount]
                    });
                }

                reportCache.put('report-cache', cache);
                var dataSourceInfo = {
                    id: dataSource.id,
                    name: dataSource.name,
                    amount: amount,
                    description: dataSource.description,
                    projectID: dataSource.projectID,
                    serviceTypeID: dataSource.serviceTypeID,
                    cellDataID: dataSource.cellID,
                    cellTemplateID: dataSource.cellTemplateID,
                    dataSourceType: dataSource.dataSourceType
                };

                return $q.when(dataSourceInfo);
                //return $http.post('/Report/updateDataSourceAmount/' + datasourceId + '/' + amount, {}, apiConfig.createVat());
            },
            modelBackFill: function (dataSource) {
                var cache = reportCache.get('report-cache');
                var paramDs = angular.copy(dataSource);
                var par = {
                    dataSourceID: paramDs.id,
                    cellDataID: paramDs.cellDataID,
                    backFillList: paramDs.modelWriteBackData
                };
                var existingItem = _.findWhere(cache, { serviceType: 'modelBackFill', id: paramDs.id });
                if (existingItem) {
                    existingItem.params = [par];
                }
                else {
                    cache.push({
                        serviceType: 'modelBackFill',
                        id: paramDs.id,
                        params: [par]
                    });
                }

                reportCache.put('report-cache', cache);
                var dataSourceInfo = {
                    id: paramDs.id,
                    name: paramDs.name,
                    amount: paramDs.amount,
                    cellDataID: paramDs.cellDataID,
                    cellTemplateID: paramDs.cellTemplateId,
                    dataSourceType: paramDs.dataSourceType,
                    modelWriteBackData: paramDs.modelWriteBackData
                };

                return $q.when(dataSourceInfo);
            },
            //getStdAccountByIndustryID: function (industryID) {
            //    return $http.get('/Report/getStdAccountByIndustry/' + industryID, apiConfig.createVat());
            //},
            //getTemplateReferences: function (period) {
            //    return $http.get('/Report/templateReferences/' + period, apiConfig.createVat());
            //},
            //citUpdateConfig: function (projectId, period) {
            //    var periodParam = '';
            //    if (_.isNumber(period)) {
            //        periodParam = '/' + period.toString();
            //    }

            //    return $http.post('/Report/citUpdateConfig/' + projectId + periodParam, {}, apiConfig.createVat({ ignoreLoadingBar: true }));
            //},
            //citGenerate: function (projectId, templateId, param, generator) {
            //    var paramStr = '';
            //    if (!_.isEmpty(param)) {
            //        paramStr = '/' + param;
            //    }

            //    return $http.post('/Report/citGenerate/' + projectId + '/' + templateId + paramStr + '?generator=' + generator, {}, apiConfig.createVat({ ignoreLoadingBar: true }));
            //},
            //citGenerateByType: function (projectId, reportType, generator) {
            //    var paramStr = '';
            //    if (_.isNumber(reportType)) {
            //        paramStr = '/' + reportType;
            //    }

            //    return $http.post('/Report/citGenerate/byTotal/' + projectId + paramStr + '?generator=' + generator, {}, apiConfig.createVat({ ignoreLoadingBar: true }));
            //},
            //citGetReportData: function (reportId) {
            //    return $http.get('/Report/citReportData/' + reportId, apiConfig.createVat());
            //},
            //citCalculateSingleCellFormula: function (cellId) {
            //    return $http.get('/Report/citFormulaDataSource/byCell/' + cellId, apiConfig.createVat());
            //},
            //citGetTemplate: function (projectId, param) {
            //    var paramStr = '';
            //    if (!_.isEmpty(param)) {
            //        paramStr = '/' + param;
            //    }

            //    return $http.get('/Report/citTemplate/' + projectId + paramStr, apiConfig.createVat());
            //},
            //citGetCellVoucher: function (cellDataID) {
            //    return $http.get('/Report/citCellVoucher/' + cellDataID, apiConfig.createVat());
            //},
            //citGetCellInvoice: function (cellDataID, invoiceType) {
            //    return $http.get('/Report/citCellInvoice/' + cellDataID + '/' + invoiceType, apiConfig.createVat());
            //},
            //citAddCellManualData: function (mauanlData) {
            //    return $http.post('/Report/citAddCellManualData', mauanlData, apiConfig.createVat());
            //},
            //citDeleteCellVoucher: function (voucherId, datasourceId) {
            //    return $http.post('/Report/citDeleteCellVoucher/' + voucherId + '/' + datasourceId, {}, apiConfig.createVat());
            //},
            //citDeleteCellInvoice: function (invoiceId, datasourceId) {
            //    return $http.post('/Report/citDeleteCellInvoice/' + invoiceId + '/' + datasourceId, {}, apiConfig.createVat());
            //},
            //citDeleteCellDatasource: function (datasourceID, cellID) {
            //    return $http.post('/Report/citDeleteCellDatasource/' + datasourceID + '/' + cellID, {}, apiConfig.createVat());
            //},
            //citGetAllDataItems: function (dataSourceType) {
            //    return $http.get('/Report/citGetAllDataItems/' + dataSourceType, apiConfig.createVat())
            //},
            //citAddDataSource: function (dataSource) {
            //    return $http.post('/Report/citAddDataSource', dataSource, apiConfig.createVat());
            //},
            //citAddDataSourceItems: function (dataSource) {
            //    return $http.post('/Report/citAddDataSourceItems', dataSource, apiConfig.createVat());
            //},
            //citGetCellAccountRange: function (reportTemplateId, rowIndex, columnIndex, param) {
            //    var paramStr = '';
            //    if (!_.isEmpty(param)) {
            //        paramStr = '/' + param;
            //    }

            //    return $http.get('/Report/citGetCellAccountRange/' + reportTemplateId + '/' + period + '/' + rowIndex + '/' + columnIndex + paramStr, apiConfig.createVat());
            //},
            //citGetCellTemplateConfig: function (reportTemplateId, period, rowIndex, columnIndex) {
            //    var periodParam = '';
            //    if (_.isNumber(period)) {
            //        periodParam = '/' + period.toString();
            //    }

            //    return $http.get('/Report/citGetCellTemplateConfig/' + reportTemplateId + '/' + rowIndex + '/' + columnIndex + periodParam, apiConfig.createVat());
            //},
            //citUpdateCellAccountRangeAndSummary: function (cellTemplateId, value, operation, param) {
            //    var paramStr = '';
            //    if (!_.isEmpty(param)) {
            //        paramStr = '/' + param;
            //    }

            //    return $http.post('/Report/citUpdateCellAccountRangeAndSummary/' + cellTemplateId + '/' + value + '/' + operation + paramStr, {}, apiConfig.createVat());
            //},
            //citUpdateDataSourceAmount: function (datasourceId, amount) {
            //    return $http.post('/Report/citUpdateDataSourceAmount/' + datasourceId + '/' + amount, {}, apiConfig.createVat());
            //}
        };

    }]);