﻿// registration web service proxy
webservices.factory('registrationService', ['$http', 'apiConfig', function ($http, apiConfig) {
    'use strict';
    return {
        searchPatient: function (patientName) {
            var config = { params: { patientName: patientName }, isBusyRequest: true };
            return $http.get('/registration/patients/search', apiConfig.create(config));
        },
        saveNewRegistration: function (registration) {
            var config = {isBusyRequest: true };
            return $http.post('/registration/orders/newRegistration', registration, apiConfig.create(config));
        },
        getPatient:function(id){
            var config = { isBusyRequest: true };
            return $http.get("/registration/patients/"+id, apiConfig.create(config));
        },
        updatePatient: function (patient,order) {
            var config = { isBusyRequest: true };
            var data = { patient: patient, order: order };
            return $http.put("/registration/patients/" + patient.uniqueID, data, apiConfig.create(config));
        },
        updateOrder: function (order) {
            var config = {isBusyRequest: true };
            return $http.put("/registration/orders/" + order.uniqueID, order, apiConfig.create(config));
        },
        getPatientNO: function (site) {
            var config = {params:{}, isBusyRequest: true };
            return $http.get('/registration/orders/generate/patientNo/'+site, apiConfig.create(config));
        },
        getAccNo: function () {
            var config = { params: {}, isBusyRequest: true };
            return $http.get('/registration/orders/generate/accNo', apiConfig.create(config));
        },
        getProcedures: function (patientId) {
            return $http.get('/registration/patients/procedures/' + patientId, apiConfig.create());
        },
        getProcedure: function (id) {
            var config = { isBusyRequest: true };
            return $http.get('/registration/procedures/' + id, apiConfig.create());
        },
        updateProcedure: function (id, proceduresRelated) {
            var config = { isBusyRequest: true };       
            return $http.put("/registration/procedures/" + id, proceduresRelated, apiConfig.create(config));
        },
        addProcedure:function(proceduresRelated){
            var config = { isBusyRequest: true };
            return $http.post("/registration/procedures", proceduresRelated, apiConfig.create(config));
        },
        deleteProcedure:function(id){
            return $http.delete('/registration/procedures/' + id, apiConfig.create());
        },
        getEnglishName: function (strChinese, UpperFirstLetter, SeparatePolicy, Separator) {
            var data = { LocalName: strChinese, UpperFirstLetter: UpperFirstLetter, SeparatePolicy: SeparatePolicy, Separator: Separator };
            return $http.post('/registration/orders/simplifiedToEnglish', data, apiConfig.create());
        },
        getRequisitionUrl: function (accNo, modalityType) {
            var config = { params: { accNo: accNo, modalityType: modalityType }, isBusyRequest: true };
            return $http.get('/registration/orders/requisition/url', apiConfig.create(config));
        },
        getBarCodeUrl: function (accNo, modalityType) {
            var config = { params: { accNo: accNo, modalityType: modalityType }, isBusyRequest: true };
            return $http.get('/registration/orders/barcode/url', apiConfig.create(config));
        },
        getOrder: function (orderId) {
            var config = { isBusyRequest: true };
            return $http.get('/registration/orders/' + orderId, apiConfig.create(config));
        },
        getRegistrationInfo: function (orderId) {
            var config = { isBusyRequest: true };
            return $http.get('/registration/' + orderId, apiConfig.create(config));
        },
        getProcedureCodes: function () {
            return $http.get('/registration/procedurecodes', apiConfig.create());
        },
        getBodySystemMaps: function () {
            return $http.get('/registration/bodySystemMaps', apiConfig.create());
        },
        getAccountDemoOne: function (accountId) {
            var config = { isBusyRequest: true };
            return $http.get('/NewAccount/' + accountId, apiConfig.create(config));
        },
        getAccountDemoList: function () {
            var config = { isBusyRequest: true };
            return $http.get('/NewAccount', apiConfig.create(config));
        },
        getAccountGroup : function(){
            var config = { isBusyRequest: true };
            return $http.get('/AccountGroup', apiConfig.create(config));
        },
        finishExam: function (order) {
        var config = { isBusyRequest: true };       
        return $http.put("/registration/finishexam/" + order.uniqueID, order, apiConfig.create(config));
    },
    };
}]);